# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

import MySQLdb, logging, pickle

from threading import Thread, currentThread, Semaphore
from time import sleep
from JobQueue_db import *

class JobQueueCleaner(Thread):
	def __init__(self, db, delay=60):
		self.db = db
		self.delay = delay
		self.queues = []
		self.stopped = False
		self.dead = False
	 
		Thread.__init__(self)
		self.setDaemon(True)

	def add_queue(self, queue):
		self.queues.append(queue)

	def remove_queue(self, queue):
		self.queues.remove(queue) 

	def stop(self):
		self.stopped = True
		while not self.dead:
			sleep(0.5)

		self.stopped = False
		self.dead = False

	def run(self):
		currentThread().setName('JobQueueCleaner_Thread')
		self.db.create_connection()

		while not self.stopped:
			sleep(self.delay)
			self.db.commit() 	# start a new transaction

			for queue in self.queues:
				logging.info('Cleaning out queue %d.' % queue.session_id)
				self.clean_results(queue)
				self.clean_db(queue)
				
		self.dead = True

	def clean_results(self, queue):
		for id in queue.results.keys():
			dependents = queue.get_job_dependents(id)
			
			can_delete = True
			for dep_id in dependents:
				try: 
					# first check the results hash
					queue.results[dep_id]
				except: 
					# failing that, check the database
					dep_status = queue.get_job_status(dep_id)

					if not dep_status in (AVAILABLE, DISCARDED):
						can_delete = False
						break
			
			if can_delete:
				logging.debug('Deleting memory cache of job %d.' % id)
				del queue.results[id]

	def clean_db(self, queue):
		job_data = self.db.execi('SELECT ID,Status,Dependents FROM jobs_s%d;' % queue.session_id)
		for id, status, deps_pickled in job_data:
			if status == AVAILABLE:
				dependents = pickle.loads(deps_pickled)
				
				can_delete = True
				for dep_id in dependents:
					dep_status = queue.get_job_status(dep_id)
					
					if not dep_status in (AVAILABLE, DISCARDED):
						can_delete = False
						break

				if can_delete:
					logging.debug('Deleting DB entry for job %d.' % id)
					self.db.execd('UPDATE jobs_s%d SET Status=%d, Dependents="", Result="" WHERE ID = %d;' %
							(queue.session_id, DISCARDED, id))
		# commit the changes
		self.db.commit() 



