# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

import os, tarfile, array, bz2
import zipfile, py_compile, random
import CHOMP, MySQLdb, logging
#import cPickle as pickle
import pickle

from threading import Thread, currentThread
from Queue import Queue
from os import path
from time import sleep
from Serialize import *
from JobQueue_db import DistributedJob

### Superceded by server configuration files

MAX_RUN = 100
MAX_CLONE = 10
WU_MAX_SIZE = 5 * 1024 * 1024 - 1024


### Lookup MAX_RUN and MAX_CLONE from the server configuration file

projconf = open('/home/server/server2/project.conf', 'r')
projs = []
for line in projconf:
        words = line.split()
        if words:
                if words[0] == 'SERVER_CONF':
                        # Get the number out (serverXXXX.conf)
                        projs.append(words[1][6:10])
projconf.close()
for proj in projs:
        serverconf = open('/home/server/server2/server' + proj + '.conf', 'r')
        for line in serverconf:
                words = line.split()
                if words:
                        if words[0] == 'NUM_RUNS':
                                MAX_RUN = int(words[1])
                        if words[0] == 'NUM_CLONES':
                                MAX_CLONE = int(words[1])
        serverconf.close()

### Utility Classes ###

class QueueData:
	def __init__(self, queue, name, default_project, jobs_per_wu, max_wu_length, local_files):
		self.queue = queue
		self.name = name
		self.default_project = default_project
		self.jobs_per_wu = jobs_per_wu
		self.max_wu_length = max_wu_length
		self.local_files = local_files

### Main JobQueueDriver class ###
### WARNING: Not safe to use across threads! ###

class JobQueueDriver:
	def __init__(self, db, data_dir, message_dir, session_id=0, delay=5):
		self.session_id = session_id
		self.db = db
		
		currentThread().setName('JobQueueDriver_Main_Thread')
		self.db.create_connection()
		self._init_dbase()

		self.dir_template       = path.join(data_dir, 'PROJ%d', 'RUN%d', 'CLONE%d')
		self.wu_template 	= path.join(self.dir_template, 'priority%d.swu')
		self.return_template	= path.join(self.dir_template, 'priority%d.shn')
                
		self.init_ready_msg     = path.join(message_dir, 'SHARPEN_init_ready')
		self.init_complete_msg  = path.join(message_dir, 'SHARPEN_init_complete')
		self.ready_msg 		= path.join(message_dir, 'SHARPEN_ready_to_receive')
		self.sent_msg 		= path.join(message_dir, 'SHARPEN_work_units_sent')
		self.update_file 	= path.join(message_dir, 'WU_update')
 
		self.delay = delay
		self.queues = {}
 
		self.run_number = 0
		self.clone_number = 0

		# Set initial values based on defaults in server code
		self.wu_per_batch = 20
		self.init_wus = 100
		
		# Parse project.conf file
		projconf = open( path.join(message_dir, 'project.conf'), 'r' )
		for line in projconf:
			fields = line.split()
			if (len(fields) > 0):
				if (fields[0] == 'SHARPEN_INIT_WUS'):
					self.init_wus = int(fields[1])
				elif (fields[0] == 'SHARPEN_JOB_DEPL_LIMIT'):
					self.wu_per_batch = int(fields[1])
 
		self.local_thread = LocalJobDriverThread(self)
		self.remote_thread = RemoteJobDriverThread(self)
		self.return_thread = WUReceiverThread(self)

		self.local_thread.start()
		self.remote_thread.start()
		self.return_thread.start()
		logging.info('JobQueueDriver initialized for session %d.' % self.session_id)

	
	### Database Initialization ###
	
	def _init_dbase(self):
		self._create_wus_table()	

	def _create_wus_table(self):
		# first check to see if the table for this session is already present
		results = self.db.execi('SHOW TABLES;')
		for result in results:
			if result[0] == ( 'wu_jobs_s%d' % self.session_id ):
				logging.info('JobQueueDriver Resuming from database for session %d.' % self.session_id)
				return
	
		logging.info('JobQueueDriver Starting new database for session %d.' % self.session_id)
	
		# if the table isn't present, we need to create it.
		self.db.execi("""CREATE TABLE wu_jobs_s%d 
			( 	Job_ID			BIGINT UNSIGNED,
				WU_Desc			TINYBLOB,
				Queue_Name		TINYBLOB,
				Timestamp		TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
				KEY index_Job_ID (Job_ID)
			) ENGINE = INNODB;""" % self.session_id)

	### Database Management ###
	
	def reset_session(self):
		logging.warning('Clearing out database for JQD session %d. (User Request)' % self.session_id)
		self.db.execi('DROP TABLE wu_jobs_s%d;' % self.session_id )
		self._init_dbase()

	def inc_wu_desc(self):
		## TODO: keep track of a run/clone 'cursor' on a per-project basis
		self.clone_number += 1
		if self.clone_number == MAX_CLONE:
			self.run_number += 1
			self.clone_number = 0

			if self.run_number == MAX_RUN:
				logging.warning('Looping around to run0/clone0')
				self.run_number = 0


	### Database Accessors ###

	def get_wu_jobs(self, desc):
		result = self.db.execi('SELECT Job_ID FROM wu_jobs_s%d WHERE WU_Desc = %%s;' % self.session_id, 
					pickle.dumps(desc))
		return [ id_field[0] for id_field in result ]

	def get_wu_queue(self, desc):
		result = self.db.execi('SELECT DISTINCT Queue_Name FROM wu_jobs_s%d WHERE WU_Desc = %%s;' % self.session_id,
				 	pickle.dumps(desc))
		assert(len(result)==1)
		return result[0][0]

	def get_job_wu(self, job_id):
		result = self.db.execi('SELECT WU_Desc FROM wu_jobs_s%d WHERE Job_ID = %d;' %
					(self.session_id, job_id))
		assert(len(result)==1)
		return result[0][0]

	def delete_wu(self, desc):
		self.db.execd('DELETE FROM wu_jobs_s%d WHERE WU_Desc = %%s;' % self.session_id, 
				pickle.dumps(desc))

	def insert_wu(self, desc, job_ids, queue_name):
		self.delete_wu(desc)
		self.db.execd('INSERT INTO wu_jobs_s%d (Job_ID, WU_Desc, Queue_Name) VALUES (%%s, %%s, %%s);' % self.session_id,
				[ (job_id, pickle.dumps(desc), queue_name) for job_id in job_ids ], many=True)


	### Main interface ###

	def add_queue(self, queue, script_name, default_project, required_files,
			working_dir, jobs_per_wu=10, max_wu_work=100.0):
		
		new_queue = QueueData(queue, script_name, default_project, jobs_per_wu,
							max_wu_work, required_files)

		# pre-construct a tar archive for each queue, for efficiency
		tar_pkg = tarfile.open( path.join(working_dir, 'package.tar'), 'w' )
		zip_scripts = zipfile.ZipFile( 'mods', 'w' )
		
		for file in required_files:
			base, ext = path.splitext(file)
			if ext == '.py':
				py_compile.compile( path.join(working_dir, file), dfile=base)
				zip_scripts.write( path.join(working_dir, base + '.pyc'), base + '.pyc' ) 
			else:
				tar_pkg.add( path.join(working_dir, file), file )

		zip_scripts.close()
		tar_pkg.add('mods')
		tar_pkg.close()

		os.unlink('mods')
		pkg_file = path.join(working_dir, 'package.tar')
		new_queue.pkg_file = pkg_file

		logging.info('Added JobQueue named %s, at session %d.' % (script_name, queue.session_id))

		# measure the compressed size of the file
		compressed_data = bz2.compress( open(pkg_file).read(), 1 )
		pkg_compressed_size = len( compressed_data )
		new_queue.pkg_compressed_size = pkg_compressed_size 
		
		if pkg_compressed_size >= WU_MAX_SIZE:
			error_msg = 'JobQueue %s has required_files that take up more than 5 MB disk space!' % script_name
			logging.critical(error_msg)
			print error_msg
		else:
			self.queues[script_name] = new_queue


	def remove_queue(self, name):
		os.unlink(self.queues[name].pkg_file)
		del self.queues[name]

		logging.info('Removed JobQueue named %s.' % name)

	def extract_wu_jobs(self, wu_file):
        # strip the signature out of the WU
		os.system('./sign %s -verify %s.temp' % (wu_file, wu_file))
		wu_tar = tarfile.TarFile('%s.temp' % wu_file, 'r')

		data_file = wu_tar.extractfile('wudata.serialized')
		data_string = data_file.read()
		archive = CHOMP.BinaryStringInputArchive(data_string)
		
		n_jobs = int( CHOMP.LoadStringFromArchive(archive) )
		work = float( CHOMP.LoadStringFromArchive(archive) )

		job_arc_str = CHOMP.LoadStringFromArchive(archive)
		job_arc = CHOMP.BinaryStringInputArchive(job_arc_str)

		jobs = {}
		for i in xrange(n_jobs):
			job = DistributedJob.load_from_archive(job_arc)
			jobs[job.id] = job

		os.remove('%s.temp' % wu_file)

		return jobs

### Individual Thread classes ###

class RemoteJobDriverThread(Thread):
	def __init__(self, driver):
		self.driver = driver

		Thread.__init__(self)
		self.setDaemon(True)

	def run(self):
		currentThread().setName('RemoteJobDriver_Thread')
		self.driver.db.create_connection()

                # for now just use 10 for priority
                priority = 10

		# Now enter the main loop looking for updates
                while True:
			sleep(self.driver.delay)
			self.driver.db.commit()    # make sure to start a new transaction

			# make sure we have some queues to read from
			if not self.driver.queues:
				continue

			# check for semaphore files
			if path.isfile(self.driver.init_ready_msg):
				numUnits = self.driver.init_wus
				msg = 'init'
				logging.info('run: server is requesting an initial batch of %d WUs.' % numUnits)
			elif path.isfile(self.driver.ready_msg):
				numUnits = self.driver.wu_per_batch
				msg = 'update'
				logging.info('run: server is requesting a batch of %d WUs.' % numUnits)
			else:
				# server is not ready to receive WU updates yet
				continue
			
			# if we've gotten this far, write out new WUs
			uf = open(self.driver.update_file, 'w')
 
			# write out a batch of work units
			wus_created = 0
			queues = self.driver.queues.keys()
			curr_queue_index = 0

			while wus_created < numUnits:
				logging.info('run: trying to generate the requested WU (%d / %d)' % (wus_created,numUnits))
				queue = self.driver.queues.values()[curr_queue_index]	
					
				## TODO: support per-job project assignments
				wu_desc = (queue.default_project, self.driver.run_number, self.driver.clone_number, priority)
				
				# if this WU slot is already in use, skip it
				if self.driver.get_wu_jobs(wu_desc):
					self.driver.inc_wu_desc()
					continue

				# otherwise, fill a WU from the current queue
				wu_jobs = self.create_WU( wu_desc, queue, 'wudata.serialized', self.driver.delay )

				if wu_jobs == []:
					curr_queue_index += 1
					if curr_queue_index >= len(queues):
						# we're out of queues, so bail out
						logging.info('run: exiting wu creating loop because create_WU returned nothing and we are out of queues %d >= %d' % (curr_queue_index,len(queues)))
						break
					else:
						continue

				logging.info('run: create_WU was successfully run for (%d,%d,%d,%d)' % wu_desc)
				self.driver.insert_wu(wu_desc, wu_jobs, queue.name) 
				
				# assemble the serialized data and Python scripts into a full work unit
				
				# make a temporary copy of the tar archive to add the serialized data into
				os.system('cp -f %s %s.temp' % (queue.pkg_file, queue.pkg_file))
				wu = tarfile.open('%s.temp' % queue.pkg_file, 'a')
				
				# add the data and flush the file
				wu.add('wudata.serialized')
				wu.close()

				# sign the WU; ./sign also copies 
				os.system('./sign %s.temp -sign %s' % (queue.pkg_file, self.driver.wu_template % wu_desc))
				logging.info('run: signing new WU (./sign %s.temp -sign %s))' % (queue.pkg_file, self.driver.wu_template % wu_desc))

				# remove temporary files
				os.remove('wudata.serialized')
				os.remove('%s.temp' % queue.pkg_file)

				# make a record of the WU that was just written
				logging.info('run: adding new WU to WU_update: (%d,%d,%d,%d)' % wu_desc)
				uf.write('(%d,%d,%d,%d)\n' % (queue.default_project, self.driver.run_number, priority, self.driver.clone_number))
				self.driver.inc_wu_desc()
				
				wus_created += 1
				logging.info('run: deposited a WU to disk (%d,%d,%d,%d)' % wu_desc)
 
			uf.close()
			sleep(self.driver.delay)

			if wus_created > 0:
				# write out the WU_sent message
				if msg == 'init':
					os.remove(self.driver.init_ready_msg)
					os.mknod(self.driver.init_complete_msg)
				else:
					os.remove(self.driver.ready_msg)
					os.mknod(self.driver.sent_msg)

				self.driver.db.commit()
				logging.info('run: changed the semaphore to sent after creating %d WUs.' % wus_created)

	def create_WU(self, wu_desc, queue_data, filename, delay=5):
		wu_arc = CHOMP.BinaryOutputArchive(filename)
		data_arc = CHOMP.BinaryStringOutputArchive()
		queue = queue_data.queue

		job_list = []
		bz2c = bz2.BZ2Compressor(1)
		max_size = WU_MAX_SIZE - 100000		     # leave 100 kb of breathing room, just in case
		curr_size = queue_data.pkg_compressed_size   # size of the compressed required_files tar archive

		while len(job_list) < queue_data.jobs_per_wu and curr_size < max_size:
			logging.debug('create_WU: Job_list length: %d' % len(job_list))
			job = queue.pop_job(local=False)
			if job is None:
				logging.debug('create_WU: popped "None" job off the queue')
				#if queue.flush_queue and queue.dependent == {}:
				if queue.flush_queue:
					# if we're completely out of jobs, send out the last work unit
					if len(job_list) == 0:
						logging.debug('create_WU: JobQueue "%s" is completely empty.' % queue_data.name)
						return []
					else:
						logging.warning('create_WU: work unit %s contains only %d jobs.' % (str(wu_desc), len(job_list)))
						break
				else:
					# otherwise, we can wait for more jobs to come in
					logging.debug('create_WU: queue is flushed. Sleeping.')
					sleep(delay)
					continue

			logging.debug('create_WU: placing job %d in WU %s.' % (job.id, str(wu_desc)))
	
			start_pos = data_arc.Tell()
			job.write_to_archive(data_arc)
			end_pos = data_arc.Tell()

			job_str = data_arc.GetSubString(start_pos, end_pos - start_pos)
			compressed = bz2c.compress(job_str)
			if compressed:
				curr_size += len(compressed)
			
			job_list.append(job.id)

		# if we've run way over, we can just snip off the last job
		if curr_size >= max_size + 50000:
			logging.debug('create_WU: trimming job %d from WU %s.' % (job.id, str(wu_desc)))
			arc_str = data_arc.GetSubString(0, start_pos)

			job_list.remove(job.id)
			queue.reinsert_job(job)
		else:
			# if we're within the limit, use all the jobs
			arc_str = data_arc.GetString()
 
		# create the WU data
		CHOMP.SaveToArchive(wu_arc, '', repr(len(job_list)))
		CHOMP.SaveToArchive(wu_arc, '', repr(queue_data.max_wu_length))
		CHOMP.SaveToArchive(wu_arc, '', arc_str)

		return job_list

class LocalJobDriverThread(Thread):
	def __init__(self, driver):
		self.driver = driver
		Thread.__init__(self)
		self.setDaemon(True)

	def run(self):
		currentThread().setName('LocalJobDriver_Thread')
		self.driver.db.create_connection()

		while True:
			sleep(self.driver.delay)
			self.driver.db.commit()  # make sure to start a new transaction

			for name, queue_data in self.driver.queues.items():
				while True:
					job = queue_data.queue.pop_job(local=True)
					if job is None:
						break
	
					logging.debug('Executing local job %d.' % job.id)
					result = job.execute()
					logging.debug('Finished local job %d.' % job.id)
					queue_data.queue.job_returned(job.id, result, nocache=job.nocache)
					self.driver.db.commit()

class WUReceiverThread(Thread):
	def __init__(self, driver):
		self.driver = driver
		Thread.__init__(self)
		self.setDaemon(True)

	def run(self):
		currentThread().setName('WUReceiver_Thread')
		self.driver.db.create_connection()

		while True:
			sleep(self.driver.delay)
			self.driver.db.commit()   # make sure to start a new transaction
			
			query_result = self.driver.db.execi('SELECT DISTINCT WU_Desc FROM wu_jobs_s%d;' % self.driver.session_id)
			wu_descs = [ pickle.loads(pickled[0]) for pickled in query_result ]

			for wu_desc in wu_descs:
				wu_file = self.driver.return_template % wu_desc
				if path.isfile(wu_file):
					queue_name = self.driver.get_wu_queue(wu_desc)
					try: queue = self.driver.queues[queue_name].queue
					except: continue

					logging.info('Queue "%s" received WU %s.' % (queue_name, str(wu_desc)))
					self.process_returned_wu(queue, wu_file, wu_desc)
					self.driver.delete_wu(wu_desc)
					self.driver.db.commit()

	def process_returned_wu(self, queue, wu_filename, wu_desc):
		wu_file = open(wu_filename)
		job_list = self.driver.get_wu_jobs(wu_desc)
		
		total_work = 0.0
		n_jobs = 0
	
		while True:
			try:
				ints = array.array('i')
				floats = array.array('f')
				ints.read(wu_file, 4)
				floats.read(wu_file, 1)

				data_length, hash, id = ints[0], ints[1], ints[2]
				work = floats[0]
				
				result_data = wu_file.read(data_length)
				archive = CHOMP.BinaryStringInputArchive(result_data)	

				# test for an error/traceback in the current job
				if work < 0:
					error_data = Deserialize(archive)
					logging.critical('Error in Job %d, WU %s:' % (id, str(wu_desc)))
					logging.critical(error_data)
					continue

				queue.job_returned(id, Deserialize(archive))
				job_list.remove(id)

				total_work += work
				n_jobs += 1
				
			except EOFError:
				break
			except ValueError:
				break
			except RuntimeError:
				break

		logging.info('Returned WU has %d jobs; %f standardized seconds.' % (n_jobs, total_work))

		if job_list != []:
			wu_jobs = self.driver.extract_wu_jobs(self.driver.wu_template % wu_desc)
			
			for job_id in job_list:	
				job = wu_jobs[job_id]
				queue.reinsert_job(job)
		
			logging.info('Jobs %s were not done. Re-enqueuing.' % str(job_list))

		### TODO: put in some error logging if total_work turns out to be too small



