# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

import sys, os, pickle
from os import path
import logging, time

SERVER_DIR = os.environ['SERVER_DIR']
SESSION_ID = int(os.environ['JQD_SESSION_ID'])
sys.path = pickle.loads(os.environ['PYTHON_SYS_PATH'])

LOG_FILE = os.environ['LOG_FILE']
LOG_MODE = os.environ['LOG_MODE']
LOG_LEVEL = int(os.environ['LOG_LEVEL'])
 
############### SET UP JobQueueDriver ###############

from JobQueue_db import *
from JobQueueDriver_db import *
from JobQueueCleaner_db import *

#### Set up logging ####
logging.basicConfig(level=LOG_LEVEL,
                    format='%(asctime)s %(levelname)s %(message)s',
                    filename=LOG_FILE,
                    filemode=LOG_MODE)

#### Start sending jobs to the server ####
db = DatabaseConnection()
jqd = JobQueueDriver(db, 
		     path.join(SERVER_DIR, 'data'),
		     SERVER_DIR,
		     session_id=SESSION_ID)


#### Create a JobQueueCleaner to free up memory/disk space by
#### periodically deleting unused job info 		     
jqc = JobQueueCleaner(db)
jqc.start()



############### Deployment Interface ###############

jqs = {}
mods = {}
params = {}
started = False

def start():
	global started
	started = True

	for name, suffix in list_deployed_scripts():
		_exec_script(name + '_' + suffix)

def reset():
	if started:
		raise RuntimeError('Please only call reset() before start().')

	jqd.reset_session()
	for name, suffix in list_deployed_scripts():
		reset_deployed_script(name, suffix)

def deploy_script(name, suffix, comment):
	if '_' in suffix or '.' in name+suffix:
		raise ValueError('Name must not contain punctuation.')

	dest_name = name + '_' + suffix
	dest_dir = path.join('deployed', dest_name)
	free_id = _get_free_id()

	if path.exists(dest_dir):
		raise ValueError('Master script %s already deployed.' % dest_name)
	elif not path.exists(name):
		raise ValueError('Master script %s not found.' % name)

	os.system('cp -rf %s %s' % (name, dest_dir))

	_write_file(path.join(dest_dir, 'comments'), comment)
	_write_file(path.join(dest_dir, 'session_id'), str(free_id))

	if not path.exists( path.join(dest_dir, '__init__.py') ):
		_write_file(path.join(dest_dir, '__init__.py'), '')
	
	_init_script(dest_name)
	if started:
		_exec_script(dest_name)

def undeploy_all():
	if started:
		raise RuntimeError('Please only call undeploy_all() before start().')
	
	for name, suffix in list_deployed_scripts():
		undeploy_script(name, suffix, False)

	print 'Scripts undeployed. Please restart the JQD script now.'
	
def undeploy_script(name, suffix, print_warning=True):
	if started:
		raise RuntimeError('Please only call undeploy_script() before start().')

	dest_name, dest_dir = _get_script(name, suffix)
	reset_deployed_script(name, suffix)
	os.system('rm -rf %s' % dest_dir)

	if print_warning:
		print 'Script undeployed. Please restart the JQD script before calling start().'

def reset_deployed_script(name, suffix):
	if started:
		raise RuntimeError('Please only call reset_deployed_script() before start().')

	dest_name, dest_dir = _get_script(name, suffix)
	jqs[dest_name].reset_session()

def list_master_scripts():
	master_scripts = []
	for item in os.listdir('.'):
		if path.isdir(item) and not item in ['deployed', '.svn']:
			master_scripts.append(item)
	return master_scripts

def list_deployed_scripts():
	scripts = os.listdir('deployed')
	for item in ['.svn', '__init__.py', '__init__.pyc', 'README']:
		try:
			scripts.remove(item)
		except:
			pass
	
	split = [s.split('_') for s in scripts]
	return [ ('_'.join(item[:-1]), item[-1]) for item in split ]

def deployed_script_info(name, suffix):
	dest_name, dest_dir = _get_script(name, suffix)
	return open( path.join(dest_dir, 'comments') ).read()

def deployed_script_queue(name, suffix):
	dest_name, dest_dir = _get_script(name, suffix)
	return jqs[dest_name]

def reinsert_all():
    for run in range(MAX_RUN):
        for clone in range(MAX_CLONE):
            possible_wu = '/home/server/our.server2/data/PROJ4800/RUN%d/CLONE%d/priority10.swu' % (run,clone)
            if os.path.isfile(possible_wu):
                print 'Found',possible_wu
                myjobs = jqd.extract_wu_jobs(possible_wu)
                for jid in myjobs.keys():
                    print 'Reinserting',jid
                    jqs.values()[0].reinsert_job(myjobs[jid])

### Internal API ###
	
def _init_script(name):
	script_path = path.join('deployed', name)
	id_file = open( path.join(script_path, 'session_id') )
	session_id = int(id_file.read())

	m = __import__( '.'.join(['deployed', name]), fromlist=['main'] ).main
	mods[name] = m
	params[name] = m.init(db, session_id)

	params[name]['script_name'] = name
	params[name]['working_dir'] = script_path
	jqs[name] = params[name]['queue']
	
def _exec_script(name):
	print 'Running', name
	jqd.add_queue(**params[name])
	mods[name].main()
	
	jqs[name].flush()
	jqc.add_queue(jqs[name])
	
def _get_script(name, suffix):
	dest_name = name + '_' + suffix
	dest_dir = path.join('deployed', dest_name)

	if not path.exists(dest_dir):
		raise ValueError('Master script %s not found.' % dest_name)

	return dest_name, dest_dir

def _get_free_id():
	ids = []
	for base, ext in list_deployed_scripts():
		id_file = open( path.join('deployed', base + '_' + ext, 'session_id') )
		ids.append( int(id_file.read()) )

	if ids == []:
		return 0
	else:
		ids.sort()
		return ids[-1] + 1

def _write_file(filename, contents):
	f = open(filename, 'w')
	f.write(contents)
	f.close()

for name, suffix in list_deployed_scripts():
	_init_script(name + '_' + suffix)
 


