// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "general.h"
#include "atom_type.h"

///////////////////////////////////////////////////////////////////////////
//////                         CONSTRUCTORS                         ///////
///////////////////////////////////////////////////////////////////////////

AtomTypes::Pointer AtomTypes::Create()
{
	return AtomTypes::Pointer(new AtomTypes());
}

AtomTypes::AtomTypes()
{
	#include "codegen/output/atom_types_impl.hxx"

	// turn type->string mappings into string->type mappings
	for (int i = 0; i < N_ATOM_TYPE_SYSTEMS; ++i) {
		for (unsigned int j = 0; j < toString_[i].size(); ++j) {
			toType_[i][toString_[i][j]] = j;
		}
	}

	// copy the data out of the temporary array in 
	// atom_types_impl.hxx and into our real data array
	for (int i = 0; i < N_RAW_ATOM_TYPES; ++i) {
		for (int j = 0; j < N_ATOM_TYPE_SYSTEMS; ++j) {
			data_[i][j] = conversionData[i][j];
		}
	}	
}


///////////////////////////////////////////////////////////////////////////
//////                           ACCESSORS                           //////
///////////////////////////////////////////////////////////////////////////

std::string AtomTypes::ToString(int type, AtomType::TypeSystem typeSystem) const
{
	if (type < 0 || type >= (int)toString_[typeSystem].size())
		throw std::string("AtomTypes.ToString: Type " + to_str(type) + " not found.");

	return toString_[typeSystem][type];
}

int AtomTypes::ToType(std::string const& name, AtomType::TypeSystem typeSystem) const
{
	std::map<std::string, int>::const_iterator i = toType_[typeSystem].find(name);

	if (i == toType_[typeSystem].end())
		throw std::string("AtomTypes.ToType: Type " + name + " not found.");
	
	return i->second;
}


