# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 


class AminoAcidInfo:
	def __init__(self, atomList):
		self.data['nTotalAtoms'] = len(atomList)
		self.data['nHeavyAtoms'] = len( [atom for atom in atomList if self._atom_is_heavy(atom)] )

		self.data['nHBAcceptors'] = len(self.data['acceptors'])
	
		self.data['acceptorList'] = []
		self.data['acceptorBase'] = []

		self.data['nChi'] = len(self.data['chiAtoms'])
	
		self.data['nCentroidAtoms'] = len(self.data['centroidAtoms'])

		self._cook_acceptors()
		self._cook_chidep(atomList)
		self._cook_template(atomList)

	atomOrderEnums = {  
		'trie' : 'CHOMP_ATOM_ORDER',
		'PDB'  : 'PDB_ATOM_ORDER' 
	}
	
	def _atom_is_heavy(self, atom):
		try: 
			int(atom[0])
			return atom[1] != 'H'
		except: 
			return atom[0] != 'H'
	
	def _cook_acceptors(self):
		for key, value in self.data['acceptors'].items():
			self.data['acceptorList'].append(key)
			if isinstance(value, str):
				self.data['acceptorBase'].append([value, -1])
			else:
				self.data['acceptorBase'].append(value)
		
		del self.data['acceptors']

	def _cook_chidep(self, atomList):
		dependents = []
		chiDep = self.data['chiDependent']
		index = len(chiDep) - 1

		result = dict( [ (atom, [False, False, False, False]) for atom in atomList ] )
	
		chiDep.reverse()
		for chi in chiDep:
			dependents += chi
			for dependent in dependents: 
				result[dependent][index] = True
			index -= 1

		self.data['chiDependent'] = result
	
	def _cook_template(self, atomList):
		for atom in atomList:
			if not atom in self.data['templateAtom'].keys():
				self.data['templateAtom'][atom] = [ -1, -1, -1 ]
				

