# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C', 'O',  'N', 'H', 'CA', 'HA', 'CB', '1HB', '2HB', 'CG', '1HG', '2HG', 'CD',  'OE1', 'OE2', 'OXT'],
		'PDB'  : ['N', 'CA', 'C', 'O', 'CB', 'CG', 'CD', 'OE1', 'OE2', 'H',  'HA',  '1HB', '2HB', '1HG', '2HG', 'OXT'] 
	},

        'AA_class' : 3,

	'polar' : True,  

	'acceptors' : { 'O' : 'C', 'OE1' : 'CD', 'OE2' : 'CD' },

	'templateAtom' : 
	{
		'HA'  : [ 'CA', 'N',  'C'  ],
		'1HB' : [ 'CB', 'CA', 'CG' ],
		'2HB' : [ 'CB', 'CA', 'CG' ],
		'1HG' : [ 'CG', 'CB', 'CD' ],
		'2HG' : [ 'CG', 'CB', 'CD' ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'CD', '1HG', '2HG' ],
		[ 'OE1', 'OE2' ] 
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB', 'CG'  ],
		[ 'CA', 'CB', 'CG', 'CD'  ],
		[ 'CB', 'CG', 'CD', 'OE1' ]
	],

	'templateCoords' :
	{
		'N'   :  ( 0.0000  , 7.3000 , 12.6660 )  , 
		'CA'  :  ( 0.0000  , 7.2080 , 14.1210 )  , 
		'C'   :  ( 0.0000  , 8.5910 , 14.7610 )  , 
		'O'   :  ( 0.0000  , 9.6128 , 14.04104)  , 
		'CB'  :  ( 1.2080  , 6.4040 , 14.6070 )  , 
		'CG'  :  ( 1.2950  , 6.2520 , 16.1190 )  , 
		'CD'  :  ( 2.5040  , 5.4530 , 16.5190 )  , 
		'OE1' :  ( 2.6990  , 5.2540 , 17.6940 )  , 
		'OE2' :  ( 3.1710  , 4.9460 , 15.6480 )  , 
		'H'   :  ( 0.0400  , 8.2170 , 12.2440 )  , 
		'HA'  :  ( -0.9090 , 6.7100 , 14.4600 )  , 
		'1HB' :  ( 1.1400  , 5.4180 , 14.1470 )  , 
		'2HB' :  ( 2.0990  , 6.9150 , 14.2420 )  , 
		'1HG' :  ( 1.2970  , 7.2030 , 16.6510 )  , 
		'2HG' :  ( 0.3930  , 5.6970 , 16.3730 )  ,
		'OXT' :  (  0.000 ,  8.638 , 16.010 )
	},

	'centroidAtoms' : [ 'CD', 'OE1', 'OE2' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H       CA      HA      CB      1HB     2HB     CG      1HG     2HG     CD      OE1     OE2	OXT
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,      4,      4,      5,      5,	1],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,      5,      5,      6,      6,	2],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,      4,      4,      5,      5,	3],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,      5,      5,      6,      6,	4],     #H
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,      3,      3,      4,      4,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,      4,      4,      5,      5,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,      2,      2,      3,      3,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,      3,      3,      4,      4,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,      3,      3,      4,      4,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,      1,      1,      2,      2,	4],     #CG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,      2,      2,      3,      3,	5],     #1HG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      0,      2,      3,      3,	5],     #2HG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      2,      0,      1,      1,	5],     #CD
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      3,      1,      0,      2,	6],     #OE1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      3,      1,      2,      0,	6],     #OE2
                [1,     2,      3,      4,      2,      3,      3,      4,      4,      4,      5,      5,      5,      6,      6,      0]      #OXT
        ]

}

