# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',  'O',   'N',   'H',   'CA',  'HA',  'CB',  '1HB', '2HB', 
			  'CG', 'ND1', 'CD2', 'HD2', 'CE1', 'HE1', 'NE2', 'HE2', 'OXT'],

		'PDB'  : ['N', 'CA',  'C',  'O',   'CB',  'CG',  'ND1', 'CD2', 'CE1', 'NE2', 
			  'H', 'HE2', 'HA', '1HB', '2HB', 'HE1', 'HD2','OXT']
	},

        'AA_class' : 6,

	'polar' : True,  

	'acceptors' : { 'O' : 'C', 'ND1' : ['CG', 'CE1'] },

	'templateAtom' : 
	{
		'HE2' : [ 'NE2', 'CD2', 'CE1' ],
		'HA' :  [ 'CA',  'N',   'C'   ],
		'1HB' : [ 'CB',  'CA',  'CG'  ],
		'2HB' : [ 'CB',  'CA',  'CG'  ],
		'HE1' : [ 'CE1', 'ND1', 'NE2' ],
		'HD2' : [ 'CD2', 'CG',  'NE2' ],
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'ND1', 'CD2', 'CE1', 'NE2', 'HE2', 'HE1', 'HD2' ]
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB', 'CG'  ],
		[ 'CA', 'CB', 'CG', 'ND1' ]
	],

	'templateCoords' :
	{
		'N'     :  ( 0.0000  , 12.0140 , 22.5220 )  , 
		'CA'    :  ( 0.0000  , 13.2300 , 23.3160 )  , 
		'C'     :  ( 0.0000  , 12.6360 , 24.7310 )  , 
		'O'     :  ( 0.0000  , 11.3959 , 24.8883 )  , 
		'CB'    :  ( -1.2080 , 14.1050 , 22.9660 )  , 
		'CG'    :  ( -1.2740 , 15.3800 , 23.7480 )  , 
		'ND1'   :  ( -2.2840 , 16.3040 , 23.5800 )  , 
		'CD2'   :  ( -0.4560 , 15.8850 , 24.7010 )  , 
		'CE1'   :  ( -2.0840 , 17.3230 , 24.3980 )  , 
		'NE2'   :  ( -0.9820 , 17.0930 , 25.0880 )  , 
		'H'     :  ( -0.0410 , 11.1270 , 23.0030 )  , 
		'HE2'   :  ( -0.5830 , 17.7010 , 25.7890 )  , 
		'HA'    :  ( 0.9100  , 13.7960 , 23.1190 )  , 
		'1HB'   :  ( -1.1760 , 14.3870 , 21.9130 )  , 
		'2HB'   :  ( -2.1330 , 13.5660 , 23.1690 )  , 
		'HE1'   :  ( -2.7780 , 18.1630 , 24.4180 )  , 
		'HD2'   :  ( 0.4620  , 15.5170 , 25.1600 )  ,
		'OXT' :  (  0.000 , 13.443 , 25.685 )
	},

	'centroidAtoms' : [ 'ND1', 'CD2' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H       CA      HA      CB      1HB     2HB     CG      ND1     CD2     HD2     CE1     HE1     NE2     HE2	OXT
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,      4,      5,      5,      6,      5,      6,	1],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,      5,      6,      6,      7,      6,      7,	2],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,      4,      5,      5,      6,      5,      6,	3],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,      5,      6,      6,      7,      6,      7,	4],     #H
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,      3,      4,      4,      5,      4,      5,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,      4,      5,      5,      6,      5,      6,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,      2,      3,      3,      4,      3,      4,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,      3,      4,      4,      5,      4,      5,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,      3,      4,      4,      5,      4,      5,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,      1,      2,      2,      3,      2,      3,	4],     #CG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,      2,      3,      1,      2,      2,      3,	5],     #ND1
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      0,      1,      2,      3,      1,      2,	5],     #CD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      1,      0,      3,      4,      2,      3,	6],     #HD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      2,      3,      0,      1,      1,      2,	6],     #CE1
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      4,      1,      0,      2,      3,	7],     #HE1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      2,      1,      2,      1,      2,      0,      1,	6],     #NE2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      3,      2,      3,      2,      3,      1,      0,	7],     #HE2
                [1,     2,      3,      4,      2,      3,      3,      4,      4,      4,      5,      5,      6,      6,      7,      6,      7,      0]      #OXT

        ]

}

