# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',   'N',   'H',   'CA',  'HA',  'CB',  '1HB', '2HB', 'CG', 'CD1', 
		          'HD1', 'CD2', 'HD2', 'CE1', 'HE1', 'CE2', 'HE2', 'CZ',  'OH',  'HH', 'OXT'],

		'PDB'  : ['N',  'CA', 'C',  'O',   'CB',  'CG',  'CD1', 'CD2', 'CE1', 'CE2', 'CZ', 
		          'OH', 'H',  'HH', 'HD1', 'HE1', 'HE2', 'HD2', 'HA',  '1HB', '2HB', 'OXT']                                           
	},

        'AA_class' : 19,

	'polar' : False, 

	# note that the base for OH is at a *deeper* branch depth
	# (in the chiDependent array) than the OH itself! This requires
	# special handling for rotamer trie generation.
	'acceptors' : { 'O' : 'C', 'OH' : 'HH' },

	'templateAtom' : 
	{
		'HH'  : [ 'OH',  'CZ',  'CE1' ],
		'HD1' : [ 'CD1', 'CG',  'CE1' ],
		'HE1' : [ 'CE1', 'CD1', 'CZ'  ],
		'HE2' : [ 'CE2', 'CD2', 'CZ'  ],
		'HD2' : [ 'CD2', 'CG',  'CE2' ],
		'HA'  : [ 'CA',  'N',   'C'   ],
		'1HB' : [ 'CB',  'CA',  'CG'  ],
		'2HB' : [ 'CB',  'CA',  'CG'  ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'CD1', 'CD2', 'CE1', 'CE2', 'CZ', 'OH', 'HD1', 'HD2', 'HE1', 'HE2' ],
		[ 'HH' ]
	],

	'chiAtoms' :
	[
		[ 'N',   'CA', 'CB', 'CG'  ],
		[ 'CA',  'CB', 'CG', 'CD1' ],
		[ 'CE2', 'CZ', 'OH', 'HH'  ]
	],

	'templateCoords' :
	{
		'N'   :  ( 0.0000  , 17.7910 , 69.1770 )  , 
		'CA'  :  ( 0.0000  , 17.0770 , 70.4470 )  , 
		'C'   :  ( 0.0000  , 18.0440 , 71.6230 )  , 
		'O'   :  ( 0.0000  , 19.2771 , 71.4182 )  , 
		'CB'  :  ( 1.2090  , 16.1430 , 70.5360 )  , 
		'CG'  :  ( 1.2850  , 15.3600 , 71.8280 )  , 
		'CD1' :  ( 2.3230  , 14.4700 , 72.0620 )  , 
		'CD2' :  ( 0.3190  , 15.5150 , 72.8110 )  , 
		'CE1' :  ( 2.3970  , 13.7520 , 73.2400 )  , 
		'CE2' :  ( 0.3830  , 14.8030 , 73.9930 )  , 
		'CZ'  :  ( 1.4240  , 13.9220 , 74.2040 )  , 
		'OH'  :  ( 1.4930  , 13.2110 , 75.3800 )  , 
		'H'   :  ( 0.0390  , 18.8010 , 69.1950 )  , 
		'HH'  :  ( 0.7680  , 13.4050 , 75.9790 )  , 
		'HD1' :  ( 3.0880  , 14.3410 , 71.2960 )  , 
		'HE1' :  ( 3.2180  , 13.0560 , 73.4100 )  , 
		'HE2' :  ( -0.3870 , 14.9380 , 74.7530 )  , 
		'HD2' :  ( -0.5010 , 16.2120 , 72.6380 )  , 
		'HA'  :  ( -0.9080 , 16.4800 , 70.5360 )  , 
		'1HB' :  ( 1.1450  , 15.4510 , 69.6950 )  , 
		'2HB' :  ( 2.1010  , 16.7600 , 70.4320 )  ,
		'OXT' :  (  0.000 , 17.544 , 72.769 )
	},

	'centroidAtoms' : [ 'CG', 'CZ' ],

	'neighborAtom' : 'CB',
        'bondLengths' :
        [       #C      O       N       H       CA      HA      CB      1HB     2HB     CG      CD1     HD1     CD2     HD2     CE1     HE1     CE2     HE2     CZ      OH      HH	OXT
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	1],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,	2],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	3],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,	4],     #H
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,      3,      2,      3,      3,      4,      3,      4,      4,      5,      6,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,      2,      1,      2,      2,      3,      2,      3,      3,      4,      5,	4],     #CG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,      1,      2,      3,      1,      2,      3,      4,      2,      3,      4,	5],     #CD1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      0,      3,      4,      2,      3,      4,      5,      3,      4,      5,	6],     #HD1
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      3,      0,      1,      3,      4,      1,      2,      2,      3,      4,	5],     #CD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      4,      1,      0,      4,      5,      2,      3,      3,      4,      5,	6],     #HD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      2,      3,      4,      0,      1,      2,      3,      1,      2,      3,	6],     #CE1
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      4,      5,      1,      0,      3,      4,      2,      3,      4,	7],     #HE1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      4,      1,      2,      2,      3,      0,      1,      1,      2,      3,	6],     #CE2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      4,      5,      2,      3,      3,      4,      1,      0,      2,      3,      4,	7],     #HE2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      2,      3,      1,      2,      1,      2,      0,      1,      2,	7],     #CZ
                [7,     8,      7,      8,      6,      7,      5,      6,      6,      4,      3,      4,      3,      4,      2,      3,      2,      3,      1,      0,      1,	8],     #OH
                [8,     9,      8,      9,      7,      8,      6,      7,      7,      5,      4,      5,      4,      5,      3,      4,      3,      4,      2,      1,      0,	9],     #HH
                [1,     2,      3,      4,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,      0]     #OXT
        ]
}

