# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from string import Template

ResolveAtoms = True

def indent(n, s):
	return '\n'.join(['\t'*n + l for l in s.split('\n')])

n_sym = 0
def gensym():
	global n_sym

	n_sym = n_sym+1
	return '_rs_%d' % n_sym

def CreateFormatter(_atom_order):
	global atom_order, atom_set
	atom_order = _atom_order
	
	atom_set = set()
	for lst in atom_order.values():
		atom_set = atom_set.union(set(lst))

	return Format

def _GetIndex(aa, atom):
	return atom_order[aa].index(atom)

def _SimpleFormatter(aa, data):
	repr = '%s ' % str(data)
	reader = '''
		istream >> $field;
	'''
	
	return repr, reader

def _StringFormatter(aa, data):
	repr = '%d!%s ' % (len(str(data)), str(data))
	reader = '''
		$field = ReadString(istream);
	'''

	return repr, reader

def _BoolFormatter(aa, data):
	if data: repr = '1 '
	else:	 repr = '0 '

	# we need to handle bools in a slightly roundabout
	# way because istreams can't read into vectors of
	# booleans...

	tmpsym = gensym()
	reader = Template('''
		int $tmp;
		istream >> $tmp;
		$field = (bool)$tmp;''').safe_substitute(tmp=tmpsym)

	return repr, reader

def _AtomFormatter(aa, data):
	return _SimpleFormatter(aa, _GetIndex(aa, data))

def _VectorFormatter(aa, data):
	repr = '%s %s %s ' % (str(data[0]), str(data[1]), str(data[2]))
	reader = '''
		$field = ReadVector3(istream);
	'''

	return repr, reader

def _ListFormatter(aa, data):
	reprs, readers = [], []
	if not data: return '', ''

	for i in xrange(len(data)):
		f = _GeneralFormatter(aa, data[i])
		reprs.append(f[0])
		readers.append(f[1])

	repr = '%d %s ' % (len(reprs), ' '.join(reprs))
	
	isym = gensym()
	nsym = gensym()
	el_read = Template(readers[0]).substitute(field='$field[%s]' % isym)

	reader = Template('''
		{
			int $n;
			istream >> $n;

			for (int $i = 0; $i < $n; $i++)
			{
				$el_read
			}
		}
	''').substitute(n=nsym, i=isym, el_read=indent(2, el_read))

	return repr, reader

def _DictFormatter(aa, data):	
	reprs, readers = [], []
	if not data: return '', ''

	for key, val in data.items():
		# we support only ints, atom indices and strings as keys
		if isinstance(key, str) and key in atom_set:
			key = _GetIndex(aa, key)
			tmp_type = 'int'

		elif isinstance(key, int):
			tmp_type = 'int'

		else:
			key = str(key)
			tmp_type = 'std::string'

		key_f = _GeneralFormatter(aa, key)
		data_f = _GeneralFormatter(aa, val)

		reprs.append((key_f[0], data_f[0]))
		readers.append((key_f[1], data_f[1]))

	reprs = ['%s %s' % r for r in reprs]
	repr = '%d %s ' % (len(reprs), ' '.join(reprs))

	tmpsym = gensym()
	nsym = gensym()
	isym = gensym()

	tmp_decl = tmp_type + ' ' + tmpsym + ';' 
	key_read = Template(readers[0][0]).substitute(field=tmpsym)
	val_read = Template(readers[0][1]).substitute(field='$field[%s]'%tmpsym)
	
	reader = Template('''
		{
			int $n;
			istream >> $n;

			for (int $i = 0; $i < $n; $i++)
			{
				$tmp_decl
				$key_read
				$val_read
			}
		}
	''').substitute(n=nsym, i=isym, tmp_decl=tmp_decl, 
			key_read=indent(2, key_read), 
			val_read=indent(2, val_read))

	return repr, reader

def _GeneralFormatter(aa, data):
	if isinstance(data, dict):
		return _DictFormatter(aa, data)
	
	elif isinstance(data, list):
		return _ListFormatter(aa, data)

	elif isinstance(data, str) and ResolveAtoms and data in atom_set:
		return _AtomFormatter(aa, data)

	elif isinstance(data, str):
		return _StringFormatter(aa, data)

	elif isinstance(data, tuple) and len(data) == 3 and isinstance(data[0], float):
		return _VectorFormatter(aa, data)

	elif isinstance(data, tuple):
		return _ListFormatter(aa, data)

	elif isinstance(data, bool):
		return _BoolFormatter(aa, data)

	else:
		return _SimpleFormatter(aa, data)

def Format(aa, item, data, resolve_atoms = True):
	global ResolveAtoms
	ResolveAtoms = resolve_atoms

	repr, reader = _GeneralFormatter(aa, data)
	return '"%s"\n' % repr, Template(reader).substitute(field=item)

