# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

# The pKa for deprotonation of free proline is 10.64 (25); 
# thus, proline is expected to be fully protonated at physiological pH 
# 25. Martell, A. E., and Smith, R. M. (1974) 
# Critical Stability Constants, p 37, Plenum Press, New York.  

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',   'N',  'CA',  'HA', 'CB', '1HB', '2HB', 'CG', 
		          '1HG', '2HG', 'CD', '1HD', '2HD', 'H1', 'H2'],

		'PDB'  : ['N',   'CA',  'C',   'O',  'CB', 'CG', 'CD', '1HD', '2HD', 
                  '1HG', '2HG', '1HB', '2HB', 'HA', 'H1', 'H2']
	},

    'AA_class' : 12,

	'polar' : False,  

	'acceptors' : { 'O' : 'C' },

	'templateAtom' : 
	{
		'1HD' : [ 'CD', 'N',  'CG' ],
		'2HD' : [ 'CD', 'N',  'CG' ],
		'1HG' : [ 'CG', 'CB', 'CD' ],
		'2HG' : [ 'CG', 'CB', 'CD' ],
		'1HB' : [ 'CB', 'CA', 'CG' ],
		'2HB' : [ 'CB', 'CA', 'CG' ],
		'HA'  : [ 'CA', 'N',  'C'  ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB', '1HG', '2HG' ]
	],

	'chiAtoms' :
	[
		[ 'N', 'CA', 'CB', 'CG' ]
	],

	'templateCoords' :
	{
		'N'   :  ( 13.895 , 1.827 , 9.451  )  , 
		'CA'  :  ( 15.259 , 2.242 , 9.192  )  , 
		'C'   :  ( 15.959 , 1.321 , 8.222  )  , 
		'O'   :  ( 15.702 , 0.132 , 8.143  )  , 
		'CB'  :  ( 15.927 , 2.223 , 10.577 )  , 
		'CG'  :  ( 15.069 , 1.310 , 11.409 )  , 
		'CD'  :  ( 13.672 , 1.471 , 10.862 )  , 
		'1HD' :  ( 13.189 , 0.650 , 10.821 )  , 
		'2HD' :  ( 13.219 , 2.235 , 11.206 )  , 
		'1HG' :  ( 15.229 , 0.510 , 11.182 )  , 
		'2HG' :  ( 14.957 , 1.715 , 12.247 )  , 
		'1HB' :  ( 16.695 , 1.763 , 10.389 )  , 
		'2HB' :  ( 15.782 , 3.034 , 10.843 )  , 
		'HA'  :  ( 15.270 , 3.160 , 8.805  )  ,
		'H1'  :  ( 13.273 , 2.584 , 9.205  )  ,
		'H2'  :  ( 13.714 , 1.003 , 8.895  )   
	},

	'centroidAtoms' : [ 'CB', 'CG', 'CD' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       CA      HA      CB      1HB     2HB     CG      1HG     2HG     CD      1HD     2HD     H1     H2
                [0,     1,      2,      1,      2,      2,      3,      3,      3,      4,      4,      3,      4,      4,      3,      3],     #C
                [1,     0,      3,      2,      3,      3,      4,      4,      4,      5,      5,      4,      5,      5,      4,      4],     #O
                [2,     3,      0,      1,      2,      2,      3,      3,      2,      3,      3,      1,      2,      2,      1,      1],     #N
                [1,     2,      1,      0,      1,      1,      2,      2,      2,      3,      3,      2,      3,      3,      2,      2],     #CA
                [2,     3,      2,      1,      0,      2,      3,      3,      3,      4,      4,      3,      4,      4,      3,      3],     #HA
                [2,     3,      2,      1,      2,      0,      1,      1,      1,      2,      2,      2,      3,      3,      3,      3],     #CB
                [3,     4,      3,      2,      3,      1,      0,      2,      2,      3,      3,      3,      4,      4,      4,      4],     #1HB
                [3,     4,      3,      2,      3,      1,      2,      0,      2,      3,      3,      3,      4,      4,      4,      4],     #2HB
                [3,     4,      2,      2,      3,      1,      2,      2,      0,      1,      1,      1,      2,      2,      3,      3],     #CG
                [4,     5,      3,      3,      4,      2,      3,      3,      1,      0,      2,      2,      3,      3,      4,      4],     #1HG
                [4,     5,      3,      3,      4,      2,      3,      3,      1,      2,      0,      2,      3,      3,      4,      4],     #2HG
                [3,     4,      1,      2,      3,      2,      3,      3,      1,      2,      2,      0,      1,      1,      2,      2],     #CD
                [4,     5,      2,      3,      4,      3,      4,      4,      2,      3,      3,      1,      0,      2,      3,      3],     #1HD
                [4,     5,      2,      3,      4,      3,      4,      4,      2,      3,      3,      1,      2,      0,      3,      3],     #2HD
                [3,     4,      1,      2,      3,      3,      4,      4,      3,      4,      4,      2,      3,      3,      0,      2],     #H1
                [3,     4,      1,      2,      3,      3,      4,      4,      3,      4,      4,      2,      3,      3,      2,      0]      #H2
        ]

}

