# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C', 'O',  'N', 'H1', 'CA', 'HA', 'CB', '1HB', '2HB', 'OG',  'HG', 'H2', 'H3'],
		'PDB'  : ['N', 'CA', 'C', 'O', 'CB', 'OG', 'H1',  'HG',  'HA',  '1HB', '2HB', 'H2', 'H3']                       
	},

        'AA_class' : 15,

	'polar' : True,  

	# note that the base for OG is at a *deeper* branch depth
	# (in the chiDependent array) than the OG itself! This requires
	# special handling for rotamer trie generation.
	'acceptors' : { 'O' : 'C', 'OG' : 'HG' },

	'templateAtom' : 
	{
		'HG'  : [ 'OG', 'CB', 'CA' ],
		'HA'  : [ 'CA', 'N',  'C'  ],
		'1HB' : [ 'CB', 'CA', 'OG' ],
		'2HB' : [ 'CB', 'CA', 'OG' ]
	},

	'chiDependent' :
	[
		[ 'OG', '1HB', '2HB' ],
		[ 'HG' ]
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB', 'OG' ],
		[ 'CA', 'CB', 'OG', 'HG' ]
	],

	'templateCoords' :
	{

		'N'   :  ( -0.621 , -1.240 , -0.688 )  , 
		'CA'  :  ( 0.009  , 0.077  , -0.688 )  , 
		'C'   :  ( 1.515  , -0.044 , -0.688 )  , 
		'O'   :  ( 2.081  , -1.137 , -0.688 )  , 
		'CB'  :  ( -0.494 ,  0.929 ,  0.504 )  , 
		'OG'  :  ( -0.029 ,  0.446 ,  1.769 )  , 
		'H1'  :  ( -0.767 , -1.544 ,  0.264 )  , 
		'HG'  :  ( -0.393 , -0.434 ,  1.892 )  , 
		'HA'  :  ( -0.271 ,  0.589 , -1.627 )  , 
		'1HB' :  ( -1.599 ,  0.989 ,  0.515 )  , 
		'2HB' :  ( -0.153 ,  1.976 ,  0.384 )  ,
                'H2'  :  ( -0.024 , -1.900 , -1.165 )  ,
                'H3'  :  ( -1.511 , -1.187 , -1.163 )
	},

	'centroidAtoms' : [ 'OG' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H1      CA      HA      CB      1HB     2HB     OG      HG	H2	H3
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,	3,	3],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,	4,	4],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,	1,	1],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,	2,	2],     #H1
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,	2,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,	3,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,	3,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,	4,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,	4,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,	4,	4],     #OG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,	5,	5],     #HG
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      0,      2],     #H2
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      2,      0]      #H3
        ]
}

