# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',    'N',    'H1',    'CA',  'HA',  'CB', 'HB', 
		          'CG2', '1HG2', '2HG2', '3HG2', 'OG1', 'HG1', 'H2', 'H3'],

		'PDB'  : ['N',   'CA', 'C',  'O',    'CB',   'OG1',  'CG2',  'H1', 
		          'HG1', 'HA', 'HB', '1HG2', '2HG2', '3HG2', 'H2', 'H3']                             
	},

        'AA_class' : 16,

	'polar' : True,   

	# note that the base for OG1 is at a *deeper* branch depth
	# (in the chiDependent array) than the OG1 itself! This requires
	# special handling for rotamer trie generation.
	'acceptors' : { 'O' : 'C', 'OG1' : 'HG1' },

	'templateAtom' : 
	{
		'HG1' :  [ 'OG1', 'CB', 'CA'  ],
		'HA'  :  [ 'CA',  'N',  'C'   ],
		'HB'  :  [ 'CB',  'CA', 'OG1' ],
		'1HG2' : [ 'CG2', 'CB', 'CA'  ],
		'2HG2' : [ 'CG2', 'CB', 'CA'  ],
		'3HG2' : [ 'CG2', 'CB', 'CA'  ]
	},

	'chiDependent' :
	[
		[ 'OG1', 'CG2', 'HB', '1HG2', '2HG2', '3HG2' ],  
		[ 'HG1' ] 
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB',  'OG1' ],
		[ 'CA', 'CB', 'OG1', 'HG1' ]
	],

	'templateCoords' :
	{

		'N'    :  ( -0.521 , -2.002 ,  -0.796 )  , 
		'CA'   :  ( 0.097  , -0.680 ,  -0.796 )  , 
		'C'    :  ( 1.603  , -0.786 ,  -0.796 )  , 
		'O'    :  ( 2.172  , -1.859 ,  -0.796 )  , 
		'CB'   :  ( -0.399 ,  0.159 ,  0.429  )  , 
		'OG1'  :  ( -1.806 ,  0.353 ,  0.365  )  , 
		'CG2'  :  (  0.182 ,  1.583 ,  0.564  )  , 
		'H1'   :  ( -0.664 , -2.308 ,  0.156  )  , 
		'HG1'  :  ( -2.050 ,  0.808 ,  1.177  )  , 
		'HA'   :  ( -0.197 , -0.158 , -1.725  )  , 
		'HB'   :  ( -0.161 , -0.405 ,  1.358  )  , 
		'1HG2' :  ( -0.231 ,  2.120 ,  1.438  )  , 
		'2HG2' :  ( 1.279  , 1.581  ,  0.706  )  , 
		'3HG2' :  ( -0.030 ,  2.199 , -0.329  )  ,
                'H2'   :  ( 0.082  , -2.657 , -1.274  )  ,
                'H3'   :  ( -1.411 , -1.957 , -1.272  )
	},

	'centroidAtoms' : [ 'OG1' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H1      CA      HA      CB      HB      CG2     1HG2    2HG2    3HG2    OG1     HG1	H2	H3
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      4,      4,      4,      3,      4,	3,	3],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      5,      5,      5,      4,      5,	4,	4],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      4,      4,      4,      3,      4,	1,	1],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      5,      5,      5,      4,      5,	2,	2],     #H1
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      3,      3,      3,      2,      3,	2,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      4,      4,      4,      3,      4,	3,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      2,      2,      2,      1,      2,	3,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      3,      3,      3,      2,      3,	4,	4],     #HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      1,      1,      1,      2,      3,	4,	4],     #CG2
                [4,     5,      4,      5,      3,      4,      2,      3,      1,      0,      2,      2,      3,      4,	5,	5],     #1HG2
                [4,     5,      4,      5,      3,      4,      2,      3,      1,      2,      0,      2,      3,      4,	5,	5],     #2HG2
                [4,     5,      4,      5,      3,      4,      2,      3,      1,      2,      2,      0,      3,      4,	5,	5],     #3HG2
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      3,      3,      3,      0,      1,	4,	4],     #OG1
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      4,      4,      4,      1,      0,	5,	5],     #HG1
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      5,      5,      5,      4,      5,      0,      2],     #H2
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      5,      5,      5,      4,      5,      2,      0]      #H3
        ]

}

