# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',   'N',   'H1',   'CA',  'HA',  'CB',  '1HB', '2HB', 'CG', 'CD1', 'HD1', 'CD2', 'NE1', 
		          'HE1', 'CE2', 'CE3', 'HE3', 'CZ2', 'HZ2', 'CZ3', 'HZ3', 'CH2', 'HH2', 'H2', 'H3'],

		'PDB'  : ['N', 'CA',  'C',   'O',   'CB',  'CG',  'CD1', 'CD2', 'NE1', 'CE2', 'CE3', 'CZ2', 'CZ3', 'CH2', 
		          'H1', 'HE1', 'HD1', 'HZ2', 'HH2', 'HZ3', 'HE3', 'HA',  '1HB', '2HB', 'H2', 'H3']                                                 
	},

        'AA_class' : 18,

	'polar' : False, 

	'acceptors' : { 'O' : 'C' },

	'templateAtom' : 
	{
		'HE1' : [ 'NE1', 'CD1', 'CE2' ],
		'HD1' : [ 'CD1', 'CG',  'NE1' ],
		'HZ2' : [ 'CZ2', 'CE2', 'CH2' ],
		'HH2' : [ 'CH2', 'CZ2', 'CZ3' ],
		'HZ3' : [ 'CZ3', 'CE3', 'CH2' ],
		'HE3' : [ 'CE3', 'CD2', 'CZ3' ],
		'HA'  : [ 'CA',  'N',   'C'   ],
		'1HB' : [ 'CB',  'CA',  'CG'  ],
		'2HB' : [ 'CB',  'CA',  'CG'  ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'CD1', 'CD2', 'NE1', 'CE2', 'CE3', 'CZ2', 'CZ3', 
		  'CH2', 'HE1', 'HD1', 'HZ2', 'HH2', 'HZ3', 'HE3' ]
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB', 'CG'  ],
		[ 'CA', 'CB', 'CG', 'CD1' ]
	],

	'templateCoords' :
        {
		'N'   :  ( 1.630 , -2.708 ,  -0.400 )  , 
		'CA'  :  ( 2.192 , -1.361 , -0.429  )  , 
		'C'   :  ( 3.702 , -1.404 , -0.428  )  , 
		'O'   :  ( 4.323 , -2.465 , -0.405  )  , 
		'CB'  :  ( 1.688 , -0.573 ,  0.812  )  , 
		'CG'  :  ( 0.182 , -0.294 ,  0.826  )  , 
		'CD1' :  ( -0.790,  -1.060,   1.502 )  , 
		'CD2' :  ( -0.515,   0.672,   0.130 )  , 
		'NE1' :  ( -2.092,  -0.588,   1.248 )  , 
		'CE2' :  ( -1.895,   0.486,   0.393 )  , 
		'CE3' :  ( -0.079,   1.700,  -0.745 )  , 
		'CZ2' :  ( -2.849,   1.335,  -0.209 )  , 
		'CZ3' :  ( -1.040,   2.530,  -1.322 )  , 
		'CH2' :  ( -2.405,   2.353,  -1.057 )  , 
		'H1'  :  ( 1.503,  -3.000 ,  0.558  )  , 
		'HE1' :  ( -2.984,  -0.976,   1.575 )  , 
		'HD1' :  ( -0.561,  -1.932,   2.101 )  , 
		'HZ2' :  ( -3.902,   1.197,  -0.013 )  , 
		'HH2' :  ( -3.125,   3.014,  -1.515 )  , 
		'HZ3' :  ( -0.724,   3.323,  -1.985 )  , 
		'HE3' :  ( 0.971 ,  1.839 ,  -0.961 )  , 
		'HA'  :  ( 1.873 , -0.855 , -1.360  )  , 
		'1HB' :  (  2.208,   0.401,   0.884 )  , 
		'2HB' :  ( 1.972 , -1.104 ,  1.742  )  ,
                'H2'  :  (2.260  , -3.346 ,  -0.866 )  ,
                'H3'  :  (0.738  , -2.711 ,  -0.873 )
	},

	'centroidAtoms' : [ 'CD2', 'CE2' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H1      CA      HA      CB      1HB     2HB     CG      CD1     HD1     CD2     NE1     HE1     CE2     CE3     HE3     CZ2     HZ2     CZ3     HZ3     CH2     HH2	H2	H3
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      6,      5,      5,      6,      6,      7,      6,      7,      7,      8,	3,	3],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      7,      6,      6,      7,      7,      8,      7,      8,      8,      9,	4,	4],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      6,      5,      5,      6,      6,      7,      6,      7,      7,      8,	1,	1],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      7,      6,      6,      7,      7,      8,      7,      8,      8,      9,	2,	2],     #H1
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,      4,      3,      4,      5,      4,      4,      5,      5,      6,      5,      6,      6,      7,	2,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,      5,      4,      5,      6,      5,      5,      6,      6,      7,      6,      7,      7,      8,	3,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,      3,      2,      3,      4,      3,      3,      4,      4,      5,      4,      5,      5,      6,	3,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,      4,      3,      4,      5,      4,      4,      5,      5,      6,      5,      6,      6,      7,	4,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,      4,      3,      4,      5,      4,      4,      5,      5,      6,      5,      6,      6,      7,	4,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,      2,      1,      2,      3,      2,      2,      3,      3,      4,      3,      4,      4,      5,	4,	4],     #CG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,      1,      2,      1,      2,      2,      3,      4,      3,      4,      4,      5,      4,      5,	5,	5],     #CD1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      0,      3,      2,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,	6,	6],     #HD1
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      3,      0,      2,      3,      1,      1,      2,      2,      3,      2,      3,      3,      4,	5,	5],     #CD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      2,      2,      0,      1,      1,      3,      4,      2,      3,      4,      5,      3,      4,	6,	6],     #NE1
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      3,      1,      0,      2,      4,      5,      3,      4,      5,      6,      4,      5,	7,	7],     #HE1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      2,      3,      1,      1,      2,      0,      2,      3,      1,      2,      3,      4,      2,      3,	6,	6],     #CE2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      4,      1,      3,      4,      2,      0,      1,      3,      4,      1,      2,      2,      3,	6,	6],     #CE3
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      4,      5,      2,      4,      5,      3,      1,      0,      4,      5,      2,      3,      3,      4,	7,	7],     #HE3
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      3,      4,      2,      2,      3,      1,      3,      4,      0,      1,      2,      3,      1,      2,	7,	7],     #CZ2
                [7,     8,      7,      8,      6,      7,      5,      6,      6,      4,      4,      5,      3,      3,      4,      2,      4,      5,      1,      0,      3,      4,      2,      3,	8,	8],     #HZ2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      4,      5,      2,      4,      5,      3,      1,      2,      2,      3,      0,      1,      1,      2,	7,	7],     #CZ3
                [7,     8,      7,      8,      6,      7,      5,      6,      6,      4,      5,      6,      3,      5,      6,      4,      2,      3,      3,      4,      1,      0,      2,      3,	8,	8],     #HZ3
                [7,     8,      7,      8,      6,      7,      5,      6,      6,      4,      5,      6,      3,      3,      4,      2,      2,      3,      1,      2,      1,      2,      0,      1,	8,	8],     #CH2
                [8,     9,      8,      9,      7,      8,      6,      7,      7,      5,      6,      7,      4,      4,      5,      3,      3,      4,      2,      3,      2,      3,      1,      0,	9,	9],     #HH2
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      7,      6,      6,      7,      7,      8,      7,      8,      8,      9,      0,      2],     #H2
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      7,      6,      6,      7,      7,      8,      7,      8,      8,      9,      2,      0]      #H3
        ]


}

