# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Ram Kandasamy, Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',   'N',   'H1',   'CA',  'HA',  'CB',  '1HB', '2HB', 'CG', 'CD1', 
		          'HD1', 'CD2', 'HD2', 'CE1', 'HE1', 'CE2', 'HE2', 'CZ',  'OH',  'HH', 'H2', 'H3'],

		'PDB'  : ['N',  'CA', 'C',  'O',   'CB',  'CG',  'CD1', 'CD2', 'CE1', 'CE2', 'CZ', 
		          'OH', 'H1',  'HH', 'HD1', 'HE1', 'HE2', 'HD2', 'HA',  '1HB', '2HB', 'H2', 'H3']                                           
	},

        'AA_class' : 19,

	'polar' : False, 

	# note that the base for OH is at a *deeper* branch depth
	# (in the chiDependent array) than the OH itself! This requires
	# special handling for rotamer trie generation.
	'acceptors' : { 'O' : 'C', 'OH' : 'HH' },

	'templateAtom' : 
	{
		'HH'  : [ 'OH',  'CZ',  'CE1' ],
		'HD1' : [ 'CD1', 'CG',  'CE1' ],
		'HE1' : [ 'CE1', 'CD1', 'CZ'  ],
		'HE2' : [ 'CE2', 'CD2', 'CZ'  ],
		'HD2' : [ 'CD2', 'CG',  'CE2' ],
		'HA'  : [ 'CA',  'N',   'C'   ],
		'1HB' : [ 'CB',  'CA',  'CG'  ],
		'2HB' : [ 'CB',  'CA',  'CG'  ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'CD1', 'CD2', 'CE1', 'CE2', 'CZ', 'OH', 'HD1', 'HD2', 'HE1', 'HE2' ],
		[ 'HH' ]
	],

	'chiAtoms' :
	[
		[ 'N',   'CA', 'CB', 'CG'  ],
		[ 'CA',  'CB', 'CG', 'CD1' ],
		[ 'CE2', 'CZ', 'OH', 'HH'  ]
	],

	'templateCoords' :
	{
		'N'   :  ( -0.813  , -2.199  ,  1.423  )  , 
		'CA'  :  ( 0.612   , -2.082  ,  1.127  )  , 
		'C'   :  ( 1.441   , -2.790  ,  2.171  )  , 
		'O'   :  ( 0.927   , -3.375  ,  3.128  )  , 
		'CB'  :  ( 1.052   , -0.589  ,  1.096  )  , 
		'CG'  :  ( 0.390   , 0.302   ,  0.038  )  , 
		'CD1' :  ( 0.807   , 0.254   , -1.296  )  , 
		'CD2' :  ( -0.647  , 1.164   , 0.406   )  , 
		'CE1' :  ( 0.188   , 1.057   , -2.250  )  , 
		'CE2' :  ( -1.264  , 1.966   , -0.549  )  , 
		'CZ'  :  ( -0.846  , 1.912   , -1.877  )  , 
		'OH'  :  ( -1.452  , 2.696   , -2.817  )  , 
		'H1'  :  ( -1.102  , -1.426  ,  2.005  )  , 
		'HH'  :  ( -2.136  ,  3.213  , -2.386  )  , 
		'HD1' :  ( 1.605   , -0.411  , -1.596  )  , 
		'HE1' :  ( 0.504   , 1.017   , -3.282  )  , 
		'HE2' :  ( -2.068  , 2.623   , -0.253  )  , 
		'HD2' :  ( -0.984  , 1.203   , 1.433   )  , 
		'HA'  :  (  0.816  , -2.559  , 0.150   )  , 
		'1HB' :  ( 2.148   , -0.527  , 0.945   )  , 
		'2HB' :  ( 0.905   , -0.142  , 2.101   )  ,
                'H2'  :  ( -0.987  , -3.068  , 1.907   )  ,
                'H3'  :  ( -1.338  , -2.184  , 0.560   )

	},

	'centroidAtoms' : [ 'CG', 'CZ' ],

	'neighborAtom' : 'CB',

        'bondLengths' :
        [       #C      O       N       H1      CA      HA      CB      1HB     2HB     CG      CD1     HD1     CD2     HD2     CE1     HE1     CE2     HE2     CZ      OH      HH	H2	H3
                [0,     1,      2,      3,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	3,	3],     #C
                [1,     0,      3,      4,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,	4,	4],     #O
                [2,     3,      0,      1,      1,      2,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	1,	1],     #N
                [3,     4,      1,      0,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,	2,	2],     #H1
                [1,     2,      1,      2,      0,      1,      1,      2,      2,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	2,	2],     #CA
                [2,     3,      2,      3,      1,      0,      2,      3,      3,      3,      4,      5,      4,      5,      5,      6,      5,      6,      6,      7,      8,	3,	3],     #HA
                [2,     3,      2,      3,      1,      2,      0,      1,      1,      1,      2,      3,      2,      3,      3,      4,      3,      4,      4,      5,      6,	3,	3],     #CB
                [3,     4,      3,      4,      2,      3,      1,      0,      2,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	4,	4],     #1HB
                [3,     4,      3,      4,      2,      3,      1,      2,      0,      2,      3,      4,      3,      4,      4,      5,      4,      5,      5,      6,      7,	4,	4],     #2HB
                [3,     4,      3,      4,      2,      3,      1,      2,      2,      0,      1,      2,      1,      2,      2,      3,      2,      3,      3,      4,      5,	4,	4],     #CG
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      0,      1,      2,      3,      1,      2,      3,      4,      2,      3,      4,	5,	5],     #CD1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      0,      3,      4,      2,      3,      4,      5,      3,      4,      5,	6,	6],     #HD1
                [4,     5,      4,      5,      3,      4,      2,      3,      3,      1,      2,      3,      0,      1,      3,      4,      1,      2,      2,      3,      4,	5,	5],     #CD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      4,      1,      0,      4,      5,      2,      3,      3,      4,      5,	6,	6],     #HD2
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      1,      2,      3,      4,      0,      1,      2,      3,      1,      2,      3,	6,	6],     #CE1
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      4,      5,      1,      0,      3,      4,      2,      3,      4,	7,	7],     #HE1
                [5,     6,      5,      6,      4,      5,      3,      4,      4,      2,      3,      4,      1,      2,      2,      3,      0,      1,      1,      2,      3,	6,	6],     #CE2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      4,      5,      2,      3,      3,      4,      1,      0,      2,      3,      4,	7,	7],     #HE2
                [6,     7,      6,      7,      5,      6,      4,      5,      5,      3,      2,      3,      2,      3,      1,      2,      1,      2,      0,      1,      2,	7,	7],     #CZ
                [7,     8,      7,      8,      6,      7,      5,      6,      6,      4,      3,      4,      3,      4,      2,      3,      2,      3,      1,      0,      1,	8,	8],     #OH
                [8,     9,      8,      9,      7,      8,      6,      7,      7,      5,      4,      5,      4,      5,      3,      4,      3,      4,      2,      1,      0,	9,	9],     #HH
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,      0,      2],     #H2
                [3,     4,      1,      2,      2,      3,      3,      4,      4,      4,      5,      6,      5,      6,      6,      7,      6,      7,      7,      8,      9,      2,      0]      #H3
        ]
}

