# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',   'N',   'H',   'CA',  'HA',  'CB',  '1HB', '2HB', 'CG', 
		          'CD1', 'HD1', 'CD2', 'HD2', 'CE1', 'HE1', 'CE2', 'HE2', 'CZ',  'HZ'],

		'PDB'  : ['N',  'CA', 'C',   'O',   'CB', 'CG',  'CD1', 'CD2', 'CE1', 'CE2', 
		          'CZ', 'H',  'HD1', 'HE1', 'HZ', 'HE2', 'HD2', 'HA',  '1HB', '2HB']                                         
	},

        'AA_class' : 4,

	'polar' : False,  

	'acceptors' : { 'O' : 'C' },
	
	'templateAtom' : 
	{
		'HD1' : [ 'CD1', 'CG',  'CE1' ],
		'HE1' : [ 'CE1', 'CD1', 'CZ'  ],
		'HZ'  : [ 'CZ',  'CE1', 'CE2' ],
		'HE2' : [ 'CE2', 'CD2', 'CZ'  ],
		'HD2' : [ 'CD2', 'CG',  'CE2' ],
		'HA'  : [ 'CA',  'N',   'C'   ],
		'1HB' : [ 'CB',  'CA',  'CG'  ],
		'2HB' : [ 'CB',  'CA',  'CG'  ]
	},

	'chiDependent' :
	[
		[ 'CG', '1HB', '2HB' ],
		[ 'CD1', 'CD2', 'CE1', 'CE2', 'CZ', 'HD1', 'HE1', 'HZ', 'HE2', 'HD2' ]
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB', 'CG'  ],
		[ 'CA', 'CB', 'CG', 'CD1' ]
	],

	'templateCoords' :
	{
		'N'   :  ( 0.0000  , 8.6220  , 16.0890 )  , 
		'CA'  :  ( 0.0000  , 9.8810  , 16.8260 )  , 
		'C'   :  ( 0.0000  , 9.6390  , 18.3300 )  , 
		'O'   :  ( 0.0020  , 8.4940  , 18.7830 )  , 
		'CB'  :  ( -1.2080 , 10.7320 , 16.4300 )  , 
		'CG'  :  ( -1.2820 , 12.0500 , 17.1470 )  , 
		'CD1' :  ( -2.3200 , 12.9350 , 16.8960 )  , 
		'CD2' :  ( -0.3140 , 12.4080 , 18.0730 )  , 
		'CE1' :  ( -2.3890 , 14.1480 , 17.5550 )  , 
		'CE2' :  ( -0.3810 , 13.6190 , 18.7340 )  , 
		'CZ'  :  ( -1.4200 , 14.4900 , 18.4740 )  , 
		'H'   :  ( -0.0400 , 7.7600  , 16.6140 )  , 
		'HD1' :  ( -3.0870 , 12.6640 , 16.1700 )  , 
		'HE1' :  ( -3.2100 , 14.8340 , 17.3480 )  , 
		'HZ'  :  ( -1.4730 , 15.4460 , 18.9930 )  , 
		'HE2' :  ( 0.3860  , 13.8880 , 19.4600 )  , 
		'HD2' :  ( 0.5070  , 11.7200 , 18.2780 )  , 
		'HA'  :  ( 0.9090  , 10.4410 , 16.6020 )  , 
		'1HB' :  ( -1.1750 , 10.9580 , 15.3650 )  , 
		'2HB' :  ( -2.1310 , 10.2010 , 16.6590 )   
	},

	'centroidAtoms' : [ 'CG', 'CZ' ],

	'neighborAtom' : 'CB',

	'bondLengths' : 
	[	#C	O	N	H	CA	HA	CB	1HB	2HB	CG	CD1	HD1	CD2	HD2	CE1	HE1	CE2	HE2	CZ	HZ
		[0,	1,	2,	3,	1,	2,	2,	3,	3,	3,	4,	5,	4,	5,	5,	6,	5,	6,	6,	7],	#C
		[1,	0,	3,	4,	2,	3,	3,	4,	4,	4,	5,	6,	5,	6,	6,	7,	6,	7,	7,	8],	#O
		[2,	3,	0,	1,	1,	2,	2,	3,	3,	3,	4,	5,	4,	5,	5,	6,	5,	6,	6,	7],	#N
		[3,	4,	1,	0,	2,	3,	3,	4,	4,	4,	5,	6,	5,	6,	6,	7,	6,	7,	7,	8],	#H
		[1,	2,	1,	2,	0,	1,	1,	2,	2,	2,	3,	4,	3,	4,	4,	5,	4,	5,	5,	6],	#CA
		[2,	3,	2,	3,	1,	0,	2,	3,	3,	3,	4,	5,	4,	5,	5,	6,	5,	6,	6,	7],	#HA
		[2,	3,	2,	3,	1,	2,	0,	1,	1,	1,	2,	3,	2,	3,	3,	4,	3,	4,	4,	5],	#CB
		[3,	4,	3,	4,	2,	3,	1,	0,	2,	2,	3,	4,	3,	4,	4,	5,	4,	5,	5,	6],	#1HB
		[3,	4,	3,	4,	2,	3,	1,	2,	0,	2,	3,	4,	3,	4,	4,	5,	4,	5,	5,	6],	#2HB
		[3,	4,	3,	4,	2,	3,	1,	2,	2,	0,	1,	2,	1,	2,	2,	3,	2,	3,	3,	4],	#CG
		[4,	5,	4,	5,	3,	4,	2,	3,	3,	1,	0,	1,	2,	3,	1,	2,	3,	4,	2,	3],	#CD1
		[5,	6,	5,	6,	4,	5,	3,	4,	4,	2,	1,	0,	3,	4,	2,	3,	4,	5,	3,	4],	#HD1
		[4,	5,	4,	5,	3,	4,	2,	3,	3,	1,	2,	3,	0,	1,	3,	4,	1,	2,	2,	3],	#CD2
		[5,	6,	5,	6,	4,	5,	3,	4,	4,	2,	3,	4,	1,	0,	4,	5,	2,	3,	3,	4],	#HD2
		[5,	6,	5,	6,	4,	5,	3,	4,	4,	2,	1,	2,	3,	4,	0,	1,	2,	3,	1,	2],	#CE1
		[6,	7,	6,	7,	5,	6,	4,	5,	5,	3,	2,	3,	4,	5,	1,	0,	3,	4,	2,	3],	#HE1
		[5,	6,	5,	6,	4,	5,	3,	4,	4,	2,	3,	4,	1,	2,	2,	3,	0,	1,	1,	2],	#CE2
		[6,	7,	6,	7,	5,	6,	4,	5,	5,	3,	4,	5,	2,	3,	3,	4,	1,	0,	2,	3],	#HE2
		[6,	7,	6,	7,	5,	6,	4,	5,	5,	3,	2,	3,	2,	3,	1,	2,	1,	2,	0,	1],	#CZ
		[7,	8,	7,	8,	6,	7,	5,	6,	6,	4,	3,	4,	3,	4,	2,	3,	2,	3,	1,	0]	#HZ
	]
}


