# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from AminoAcid import AminoAcidInfo

class AAData(AminoAcidInfo):
	def __init__(self):
		self.data = data
		AminoAcidInfo.__init__(self, data['atomOrders']['trie'])

data = 		\
{
	'atomOrders' : 
	{
		'trie' : ['C',   'O',    'N',    'H',    'CA',  'HA',  'CB', 'HB', 
		          'CG2', '1HG2', '2HG2', '3HG2', 'OG1', 'HG1'],

		'PDB'  : ['N',   'CA', 'C',  'O',    'CB',   'OG1',  'CG2',  'H', 
		          'HG1', 'HA', 'HB', '1HG2', '2HG2', '3HG2']                             
	},

        'AA_class' : 16,

	'polar' : True,   

	# note that the base for OG1 is at a *deeper* branch depth
	# (in the chiDependent array) than the OG1 itself! This requires
	# special handling for rotamer trie generation.
	'acceptors' : { 'O' : 'C', 'OG1' : 'HG1' },

	'templateAtom' : 
	{
		'HG1' :  [ 'OG1', 'CB', 'CA'  ],
		'HA'  :  [ 'CA',  'N',  'C'   ],
		'HB'  :  [ 'CB',  'CA', 'OG1' ],
		'1HG2' : [ 'CG2', 'CB', 'CA'  ],
		'2HG2' : [ 'CG2', 'CB', 'CA'  ],
		'3HG2' : [ 'CG2', 'CB', 'CA'  ]
	},

	'chiDependent' :
	[
		[ 'OG1', 'CG2', 'HB', '1HG2', '2HG2', '3HG2' ],  
		[ 'HG1' ] 
	],

	'chiAtoms' :
	[
		[ 'N',  'CA', 'CB',  'OG1' ],
		[ 'CA', 'CB', 'OG1', 'HG1' ]
	],

	'templateCoords' :
	{
		'N'    :  ( 0.0000  , 16.9170 , 58.2740 )  , 
		'CA'   :  ( 0.0000  , 17.7300 , 59.4840 )  , 
		'C'    :  ( 0.0000  , 16.8580 , 60.7340 )  , 
		'O'    :  ( 0.0030  , 15.6300 , 60.6450 )  , 
		'CB'   :  ( -1.2150 , 18.6750 , 59.5300 )  , 
		'OG1'  :  ( -1.1740 , 19.4520 , 60.7340 )  , 
		'CG2'  :  ( -2.5110 , 17.8800 , 59.4880 )  , 
		'H'    :  ( -0.0310 , 15.9120 , 58.3720 )  , 
		'HG1'  :  ( -1.9320 , 20.0410 , 60.7600 )  , 
		'HA'   :  ( 0.9100  , 18.3290 , 59.5260 )  , 
		'HB'   :  ( -1.1740 , 19.3470 , 58.6730 )  , 
		'1HG2' :  ( -3.3580 , 18.5650 , 59.5210 )  , 
		'2HG2' :  ( -2.5500 , 17.2970 , 58.5680 )  , 
		'3HG2' :  ( -2.5530 , 17.2090 , 60.3450 )   
	},

	'centroidAtoms' : [ 'OG1' ],

	'neighborAtom' : 'CB',

	'bondLengths' : 
	[	#C	O	N	H	CA	HA	CB	HB	CG2	1HG2	2HG2	3HG2	OG1	HG1	
		[0,	1,	2,	3,	1,	2,	2,	3,	3,	4,	4,	4,	3,	4],	#C
		[1,	0,	3,	4,	2,	3,	3,	4,	4,	5,	5,	5,	4,	5],	#O
		[2,	3,	0,	1,	1,	2,	2,	3,	3,	4,	4,	4,	3,	4],	#N
		[3,	4,	1,	0,	2,	3,	3,	4,	4,	5,	5,	5,	4,	5],	#H
		[1,	2,	1,	2,	0,	1,	1,	2,	2,	3,	3,	3,	2,	3],	#CA
		[2,	3,	2,	3,	1,	0,	2,	3,	3,	4,	4,	4,	3,	4],	#HA
		[2,	3,	2,	3,	1,	2,	0,	1,	1,	2,	2,	2,	1,	2],	#CB
		[3,	4,	3,	4,	2,	3,	1,	0,	2,	3,	3,	3,	2,	3],	#HB
		[3,	4,	3,	4,	2,	3,	1,	2,	0,	1,	1,	1,	2,	3],	#CG2
		[4,	5,	4,	5,	3,	4,	2,	3,	1,	0,	2,	2,	3,	4],	#1HG2
		[4,	5,	4,	5,	3,	4,	2,	3,	1,	2,	0,	2,	3,	4],	#2HG2
		[4,	5,	4,	5,	3,	4,	2,	3,	1,	2,	2,	0,	3,	4],	#3HG2
		[3,	4,	3,	4,	2,	3,	1,	2,	2,	3,	3,	3,	0,	1],	#OG1
		[4,	5,	4,	5,	3,	4,	2,	3,	3,	4,	4,	4,	1,	0]	#HG1
	]
}


