# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

import rosetta

class AtomType:
	def __init__(self, type_lst):
		# required fields (see rosetta.py for details)
		self.name = 'CountPair'
		self.enumFn = lambda x: x
		self.enumName = 'COUNT_PAIR_TYPE'
		self.types = dict( [(type, []) for type in type_list] )
		self.typeList = type_list

		# make a deep copy of the rosetta type data
		type_data = dict( [ (key, val.copy()) for (key, val) in rosetta.type_data.items() ] )

		# prepare a map from rosetta type to countpair type for the usual cases
		map = { 'BB_N' : 'N', 'BB_CO' : 'C', 'BB_OC' : 'O', 'BB_CA' : 'CA', 
			'PRO_N' : 'N', 'BB_HN' : 'HN' }
	
		for aa, types in type_data.items():
			for atom in types.keys():
				try:	types[atom] = map[types[atom]]
				except: types[atom] = 'SC'

				if atom == 'CB': types[atom] = 'CB'		# beta carbons
				if atom.find('HA') != -1: types[atom] = 'HA'	# alpha carbon hydrogens
				
				if aa == 'PRO':					# proline gets special handling
										# due to its weirdness
					if atom == 'CD': types[atom] = 'PRO_CD'
					if atom in ['1HD', '2HD']: types[atom] = 'PRO_HD'
		
		for aa, data in type_data.items():
			for atom, type in data.items():
				self.types[type].append(aa + ':' + atom)

type_list =			\
[
	'N',		# backbone nitrogen
	'CA',		# alpha carbon
	'C',		# backbone carbon	
	'O',		# backbone oxygen
	'CB',		# beta carbon
	'SC',		# miscellaneous side chain atom
	'HN',		# backbone nitrogen's hydrogen
	'HA',		# alpha carbon's hydrogen
	'PRO_CD',	# Proline's delta carbon
	'PRO_HD'	# Proline's delta carbon's hydrogen
]


