# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

#generated by a python script
class AtomType:
	def __init__(self, type_lst):
		# the following are the required fields

		# the name of the type system (must be a valid C++ symbol)
		self.name = 'OPLSaa'

		# if the names of the types are not valid C++ symbols, this must be
		# a function that turns them into valid ones
		self.enumFn = lambda x: x

		# the name of the type of system in a form that looks like a constant
		# in an enumeration
		self.enumName = 'OPLSaa_TYPE'

		# a map that goes from each type in type_list, to a list of raw types
		# (the raw types being strings in the form AA:ATOM)
		self.types = dict( [(type,[]) for type in type_list] )

		# the list of types in this system. This defines the order in which
		# data tables and such have the types
		self.typeList = type_list

		# this snippet turns type_data into the format required by self.types
		for aa, data in type_data.items():
			for atom, type in data.items():
				self.types[type].append(aa + ':' + atom)

type_list =	\
[
	'atom_1',	# "RCH3 Alkane"
	'atom_2',	# "R2CH2 Alkane"
	'atom_3',	# "R3CH Alkane"
	'atom_6',	# "HR Alkane"
	'atom_11',	# "CH Benzene"
	'atom_12',	# "H Benzene"
	'atom_15',	# "CH2 Ethyl Benzene"
	'atom_16',	# "OH Alcohol"
	'atom_17',	# "HO Alcohol"
	'atom_19',	# "CH2 & CH3 Alcohol"
	'atom_20',	# "CH i-Pr Alcohol"
	'atom_28',	# "COH Phenol"
	'atom_29',	# "OH Phenol"
	'atom_30',	# "HO Phenol"
	'atom_52',	# "SH Thiol"
	'atom_54',	# "S Sulfide"
	'atom_56',	# "HS Thiol"
	'atom_59',	# "RCH2SH Et Thiol"
	'atom_62',	# "CH3SR Me Sulfide"
	'atom_63',	# "RCH2SR Et Sulfide"
	'atom_73',	# "RCH2NH2 & GLY CA"
	'atom_74',	# "R2CHNH2 & ALA CA"
	'atom_82',	# "NC=O Amide"
	'atom_83',	# "NC=O Amide"
	'atom_84',	# "H2NC=O Amide"
	'atom_85',	# "RHNC=O Amide"
	'atom_86',	# "R2NC=O Amide"
	'atom_87',	# "H2NC=O Amide"
	'atom_88',	# "RHNC=O Amide"
	'atom_93',	# "O=CNR-CHR2 Amide"
	'atom_111',	# "COO- Carboxylate"
	'atom_112',	# "COO- Carboxylate"
	'atom_114',	# "RCH2COO- Carboxylate"
	'atom_146',	# "RNH3+ Ammonium"
	'atom_147',	# "R2NH2+ Ammonium"
	'atom_151',	# "RNH3+ Ammonium"
	'atom_152',	# "R2NH2+ Ammonium"
	'atom_155',	# "RCH2NH3+ Ammonium"
	'atom_156',	# "R2CHNH3+ Ammonium"
	'atom_158',	# "NH2 Guanidinium"
	'atom_159',	# "NHR Guanidinium"
	'atom_160',	# "NH2 Guanidinium"
	'atom_161',	# "NHR Guanidinium"
	'atom_162',	# "C+ Guanidinium"
	'atom_165',	# "CH2 Et Guanidinium"
	'atom_166',	# "CH2 ARG CG"
	'atom_167',	# "C TRP CG"
	'atom_168',	# "C TRP CD2"
	'atom_169',	# "C TRP CE2"
	'atom_170',	# "NH TRP NE1 & HID/E"
	'atom_171',	# "HN TRP HE1 & HID/E"
	'atom_172',	# "CH2 HIS CB"
	'atom_173',	# "CH HID/HIE CE1"
	'atom_174',	# "C HID CD2 & HIE CG"
	'atom_175',	# "C HID CG & HIE CD2"
	'atom_178',	# "N HID NE2 & HIE ND1"
	'atom_181',	# "CH2 PRO CD"
	'atom_182',	# "CH2 C-term GLY CA"
	'atom_183',	# "CH C-term ALA CA"
	'atom_185'	# "CH C-term PRO CA"
]

type_data =	\
{
'ALA':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'3HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83'},

'ARG':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'1HH1': 'atom_160',
	'1HH2': 'atom_160',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'2HH1': 'atom_160',
	'2HH2': 'atom_160',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD': 'atom_165',
	'CG': 'atom_166',
	'CZ': 'atom_162',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HE': 'atom_161',
	'N': 'atom_85',
	'NE': 'atom_159',
	'NH1': 'atom_158',
	'NH2': 'atom_158',
	'O': 'atom_83'},

'ASN':{	'1HB': 'atom_6',
	'1HD2': 'atom_87',
	'2HB': 'atom_6',
	'2HD2': 'atom_87',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CG': 'atom_82',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'ND2': 'atom_84',
	'O': 'atom_83',
	'OD1': 'atom_83'},

'ASP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_114',
	'CG': 'atom_111',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83',
	'OD1': 'atom_112',
	'OD2': 'atom_112'},

'CALA':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'3HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CARG':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'1HH1': 'atom_160',
	'1HH2': 'atom_160',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'2HH1': 'atom_160',
	'2HH2': 'atom_160',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD': 'atom_165',
	'CG': 'atom_166',
	'CZ': 'atom_162',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HE': 'atom_161',
	'N': 'atom_85',
	'NE': 'atom_159',
	'NH1': 'atom_158',
	'NH2': 'atom_158',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CASN':{	'1HB': 'atom_6',
	'1HD2': 'atom_87',
	'2HB': 'atom_6',
	'2HD2': 'atom_87',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CG': 'atom_82',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'ND2': 'atom_84',
	'O': 'atom_112',
	'OD1': 'atom_83',
	'OXT': 'atom_112'},

'CASP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_114',
	'CG': 'atom_111',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OD1': 'atom_112',
	'OD2': 'atom_112',
	'OXT': 'atom_112'},

'CCYS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_59',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_56',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112',
	'SG': 'atom_52'},

'CGLN':{	'1HB': 'atom_6',
	'1HE2': 'atom_87',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE2': 'atom_87',
	'2HG': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD': 'atom_82',
	'CG': 'atom_2',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'NE2': 'atom_84',
	'O': 'atom_112',
	'OE1': 'atom_83',
	'OXT': 'atom_112'},

'CGLU':{	'1HB': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HG': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD': 'atom_111',
	'CG': 'atom_114',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OE1': 'atom_112',
	'OE2': 'atom_112',
	'OXT': 'atom_112'},

'CGLY':{	'1HA': 'atom_6',
	'2HA': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_182',
	'H': 'atom_88',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CHIS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_172',
	'CD2': 'atom_175',
	'CE1': 'atom_173',
	'CG': 'atom_174',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_171',
	'N': 'atom_85',
	'ND1': 'atom_178',
	'NE2': 'atom_170',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CILE':{	'1HD1': 'atom_6',
	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HD1': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HD1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_3',
	'CD1': 'atom_1',
	'CG1': 'atom_2',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CLEU':{	'1HB': 'atom_6',
	'1HD1': 'atom_6',
	'1HD2': 'atom_6',
	'2HB': 'atom_6',
	'2HD1': 'atom_6',
	'2HD2': 'atom_6',
	'3HD1': 'atom_6',
	'3HD2': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD1': 'atom_1',
	'CD2': 'atom_1',
	'CG': 'atom_3',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CLYS':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'1HZ': 'atom_151',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'2HZ': 'atom_151',
	'3HZ': 'atom_151',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD': 'atom_2',
	'CE': 'atom_155',
	'CG': 'atom_2',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'NZ': 'atom_146',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CMET':{	'1HB': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'3HE': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CE': 'atom_62',
	'CG': 'atom_63',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112',
	'SD': 'atom_54'},

'CPHE':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_11',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HZ': 'atom_12',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CPRO':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_185',
	'CB': 'atom_2',
	'CD': 'atom_181',
	'CG': 'atom_2',
	'HA': 'atom_6',
	'N': 'atom_86',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CSER':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_19',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_17',
	'N': 'atom_85',
	'O': 'atom_112',
	'OG': 'atom_16',
	'OXT': 'atom_112'},

'CTHR':{	'1HG2': 'atom_6',
	'2HG2': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_20',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'HG1': 'atom_17',
	'N': 'atom_85',
	'O': 'atom_112',
	'OG1': 'atom_16',
	'OXT': 'atom_112'},

'CTRP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_2',
	'CD1': 'atom_11',
	'CD2': 'atom_168',
	'CE2': 'atom_169',
	'CE3': 'atom_11',
	'CG': 'atom_167',
	'CH2': 'atom_11',
	'CZ2': 'atom_11',
	'CZ3': 'atom_11',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HE1': 'atom_171',
	'HE3': 'atom_12',
	'HH2': 'atom_12',
	'HZ2': 'atom_12',
	'HZ3': 'atom_12',
	'N': 'atom_85',
	'NE1': 'atom_170',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CTYR':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_28',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HH': 'atom_30',
	'N': 'atom_85',
	'O': 'atom_112',
	'OH': 'atom_29',
	'OXT': 'atom_112'},

'CVAL':{	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HG1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_111',
	'CA': 'atom_183',
	'CB': 'atom_3',
	'CG1': 'atom_1',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_112',
	'OXT': 'atom_112'},

'CYS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_59',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_56',
	'N': 'atom_85',
	'O': 'atom_83',
	'SG': 'atom_52'},

'GLN':{	'1HB': 'atom_6',
	'1HE2': 'atom_87',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE2': 'atom_87',
	'2HG': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD': 'atom_82',
	'CG': 'atom_2',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'NE2': 'atom_84',
	'O': 'atom_83',
	'OE1': 'atom_83'},

'GLU':{	'1HB': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HG': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD': 'atom_111',
	'CG': 'atom_114',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83',
	'OE1': 'atom_112',
	'OE2': 'atom_112'},

'GLY':{	'1HA': 'atom_6',
	'2HA': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_73',
	'H': 'atom_88',
	'N': 'atom_85',
	'O': 'atom_83'},

'HIS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_172',
	'CD2': 'atom_175',
	'CE1': 'atom_173',
	'CG': 'atom_174',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_171',
	'N': 'atom_85',
	'ND1': 'atom_178',
	'NE2': 'atom_170',
	'O': 'atom_83'},

'ILE':{	'1HD1': 'atom_6',
	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HD1': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HD1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_3',
	'CD1': 'atom_1',
	'CG1': 'atom_2',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83'},

'LEU':{	'1HB': 'atom_6',
	'1HD1': 'atom_6',
	'1HD2': 'atom_6',
	'2HB': 'atom_6',
	'2HD1': 'atom_6',
	'2HD2': 'atom_6',
	'3HD1': 'atom_6',
	'3HD2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD1': 'atom_1',
	'CD2': 'atom_1',
	'CG': 'atom_3',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83'},

'LYS':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'1HZ': 'atom_151',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'2HZ': 'atom_151',
	'3HZ': 'atom_151',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD': 'atom_2',
	'CE': 'atom_155',
	'CG': 'atom_2',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'NZ': 'atom_146',
	'O': 'atom_83'},

'MET':{	'1HB': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'3HE': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CE': 'atom_62',
	'CG': 'atom_63',
	'H': 'atom_88',
	'HA': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83',
	'SD': 'atom_54'},

'NALA':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'3HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_1',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83'},

'NARG':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'1HH1': 'atom_160',
	'1HH2': 'atom_160',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'2HH1': 'atom_160',
	'2HH2': 'atom_160',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD': 'atom_165',
	'CG': 'atom_166',
	'CZ': 'atom_162',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HE': 'atom_161',
	'N': 'atom_146',
	'NE': 'atom_159',
	'NH1': 'atom_158',
	'NH2': 'atom_158',
	'O': 'atom_83'},

'NASN':{	'1HB': 'atom_6',
	'1HD2': 'atom_87',
	'2HB': 'atom_6',
	'2HD2': 'atom_87',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CG': 'atom_82',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'ND2': 'atom_84',
	'O': 'atom_83',
	'OD1': 'atom_83'},

'NASP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_114',
	'CG': 'atom_111',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83',
	'OD1': 'atom_112',
	'OD2': 'atom_112'},

'NCYS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_59',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HG': 'atom_56',
	'N': 'atom_146',
	'O': 'atom_83',
	'SG': 'atom_52'},

'NGLN':{	'1HB': 'atom_6',
	'1HE2': 'atom_87',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE2': 'atom_87',
	'2HG': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD': 'atom_82',
	'CG': 'atom_2',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'NE2': 'atom_84',
	'O': 'atom_83',
	'OE1': 'atom_83'},

'NGLU':{	'1HB': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HG': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD': 'atom_111',
	'CG': 'atom_114',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83',
	'OE1': 'atom_112',
	'OE2': 'atom_112'},

'NGLY':{	'1HA': 'atom_6',
	'2HA': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_155',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'N': 'atom_146',
	'O': 'atom_83'},

'NHIS':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_172',
	'CD2': 'atom_175',
	'CE1': 'atom_173',
	'CG': 'atom_174',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_171',
	'N': 'atom_146',
	'ND1': 'atom_178',
	'NE2': 'atom_170',
	'O': 'atom_83'},

'NILE':{	'1HD1': 'atom_6',
	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HD1': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HD1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_3',
	'CD1': 'atom_1',
	'CG1': 'atom_2',
	'CG2': 'atom_1',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83'},

'NLEU':{	'1HB': 'atom_6',
	'1HD1': 'atom_6',
	'1HD2': 'atom_6',
	'2HB': 'atom_6',
	'2HD1': 'atom_6',
	'2HD2': 'atom_6',
	'3HD1': 'atom_6',
	'3HD2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD1': 'atom_1',
	'CD2': 'atom_1',
	'CG': 'atom_3',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HG': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83'},

'NLYS':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'1HZ': 'atom_151',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'2HZ': 'atom_151',
	'3HZ': 'atom_151',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD': 'atom_2',
	'CE': 'atom_155',
	'CG': 'atom_2',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'NZ': 'atom_146',
	'O': 'atom_83'},

'NMET':{	'1HB': 'atom_6',
	'1HE': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HE': 'atom_6',
	'2HG': 'atom_6',
	'3HE': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CE': 'atom_62',
	'CG': 'atom_63',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83',
	'SD': 'atom_54'},

'NPHE':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_11',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HZ': 'atom_12',
	'N': 'atom_146',
	'O': 'atom_83'},

'NPRO':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'H1': 'atom_152',
	'H2': 'atom_152',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD': 'atom_155',
	'CG': 'atom_2',
	'HA': 'atom_6',
	'N': 'atom_147',
	'O': 'atom_83'},

'NSER':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_19',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HG': 'atom_17',
	'N': 'atom_146',
	'O': 'atom_83',
	'OG': 'atom_16'},

'NTHR':{	'1HG2': 'atom_6',
	'2HG2': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_20',
	'CG2': 'atom_1',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'HG1': 'atom_17',
	'N': 'atom_146',
	'O': 'atom_83',
	'OG1': 'atom_16'},

'NTRP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_2',
	'CD1': 'atom_11',
	'CD2': 'atom_168',
	'CE2': 'atom_169',
	'CE3': 'atom_11',
	'CG': 'atom_167',
	'CH2': 'atom_11',
	'CZ2': 'atom_11',
	'CZ3': 'atom_11',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HE1': 'atom_171',
	'HE3': 'atom_12',
	'HH2': 'atom_12',
	'HZ2': 'atom_12',
	'HZ3': 'atom_12',
	'N': 'atom_146',
	'NE1': 'atom_170',
	'O': 'atom_83'},

'NTYR':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_28',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HH': 'atom_30',
	'N': 'atom_146',
	'O': 'atom_83',
	'OH': 'atom_29'},

'NVAL':{	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HG1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_156',
	'CB': 'atom_3',
	'CG1': 'atom_1',
	'CG2': 'atom_1',
	'H1': 'atom_151',
	'H2': 'atom_151',
	'H3': 'atom_151',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_146',
	'O': 'atom_83'},

'PHE':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_11',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HZ': 'atom_12',
	'N': 'atom_85',
	'O': 'atom_83'},

'PRO':{	'1HB': 'atom_6',
	'1HD': 'atom_6',
	'1HG': 'atom_6',
	'2HB': 'atom_6',
	'2HD': 'atom_6',
	'2HG': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_93',
	'CB': 'atom_2',
	'CD': 'atom_181',
	'CG': 'atom_2',
	'HA': 'atom_6',
	'N': 'atom_86',
	'O': 'atom_83'},

'SER':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_19',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HG': 'atom_17',
	'N': 'atom_85',
	'O': 'atom_83',
	'OG': 'atom_16'},

'THR':{	'1HG2': 'atom_6',
	'2HG2': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_20',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'HG1': 'atom_17',
	'N': 'atom_85',
	'O': 'atom_83',
	'OG1': 'atom_16'},

'TRP':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_2',
	'CD1': 'atom_11',
	'CD2': 'atom_168',
	'CE2': 'atom_169',
	'CE3': 'atom_11',
	'CG': 'atom_167',
	'CH2': 'atom_11',
	'CZ2': 'atom_11',
	'CZ3': 'atom_11',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HE1': 'atom_171',
	'HE3': 'atom_12',
	'HH2': 'atom_12',
	'HZ2': 'atom_12',
	'HZ3': 'atom_12',
	'N': 'atom_85',
	'NE1': 'atom_170',
	'O': 'atom_83'},

'TYR':{	'1HB': 'atom_6',
	'2HB': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_15',
	'CD1': 'atom_11',
	'CD2': 'atom_11',
	'CE1': 'atom_11',
	'CE2': 'atom_11',
	'CG': 'atom_11',
	'CZ': 'atom_28',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HD1': 'atom_12',
	'HD2': 'atom_12',
	'HE1': 'atom_12',
	'HE2': 'atom_12',
	'HH': 'atom_30',
	'N': 'atom_85',
	'O': 'atom_83',
	'OH': 'atom_29'},

'VAL':{	'1HG1': 'atom_6',
	'1HG2': 'atom_6',
	'2HG1': 'atom_6',
	'2HG2': 'atom_6',
	'3HG1': 'atom_6',
	'3HG2': 'atom_6',
	'C': 'atom_82',
	'CA': 'atom_74',
	'CB': 'atom_3',
	'CG1': 'atom_1',
	'CG2': 'atom_1',
	'H': 'atom_88',
	'HA': 'atom_6',
	'HB': 'atom_6',
	'N': 'atom_85',
	'O': 'atom_83'}
}

