# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Contributor(s): Albert Ng 
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

#generated by a python script

import oplsaa

class AtomType:
	def __init__(self, type_lst):
		# the following are the required fields

		# the name of the type system (must be a valid C++ symbol)
		self.name = 'OPLSaa_Interaction'

		# if the names of the types are not valid C++ symbols, this must be
		# a function that turns them into valid ones
		self.enumFn = lambda x: x

		# the name of the type of system in a form that looks like a constant
		# in an enumeration
		self.enumName = 'OPLSaa_INTERACTION_TYPE'

		# a map that goes from each type in type_list, to a list of raw types
		# (the raw types being strings in the form AA:ATOM)
		self.types = dict( [(type,[]) for type in type_list] )

		# the list of types in this system. This defines the order in which
		# data tables and such have the types
		self.typeList = type_list

		# this snippet turns type_data into the format required by self.types
		for aa, data in oplsaa.type_data.items():
			for atom, type in data.items():
				self.types[type_data[type]].append(aa + ':' + atom)

type_list =	\
[
	'interaction_type_1',
	'interaction_type_2',
	'interaction_type_5',
	'interaction_type_6',
	'interaction_type_7',
	'interaction_type_8',
	'interaction_type_10',
	'interaction_type_15',
	'interaction_type_17',
	'interaction_type_18',
	'interaction_type_21',
	'interaction_type_22',
	'interaction_type_23',
	'interaction_type_24',
	'interaction_type_27',
	'interaction_type_28',
	'interaction_type_37',
	'interaction_type_38',
	'interaction_type_39',
	'interaction_type_40',
	'interaction_type_41',
	'interaction_type_42',
	'interaction_type_43',
	'interaction_type_44',
	'interaction_type_45',
	'interaction_type_46',
	'interaction_type_47',
	'interaction_type_48',
	'interaction_type_49'
]

type_data =       \
{
	'atom_1':	'interaction_type_1',
	'atom_2':	'interaction_type_1',
	'atom_3':	'interaction_type_1',
	'atom_6':	'interaction_type_2',
	'atom_11':	'interaction_type_5',
	'atom_12':	'interaction_type_6',
	'atom_15':	'interaction_type_1',
	'atom_16':	'interaction_type_7',
	'atom_17':	'interaction_type_8',
	'atom_19':	'interaction_type_1',
	'atom_20':	'interaction_type_1',
	'atom_28':	'interaction_type_5',
	'atom_29':	'interaction_type_10',
	'atom_30':	'interaction_type_8',
	'atom_52':	'interaction_type_15',
	'atom_54':	'interaction_type_17',
	'atom_56':	'interaction_type_18',
	'atom_59':	'interaction_type_1',
	'atom_62':	'interaction_type_1',
	'atom_63':	'interaction_type_1',
	'atom_73':	'interaction_type_1',
	'atom_74':	'interaction_type_1',
	'atom_82':	'interaction_type_21',
	'atom_83':	'interaction_type_22',
	'atom_84':	'interaction_type_23',
	'atom_85':	'interaction_type_23',
	'atom_86':	'interaction_type_23',
	'atom_87':	'interaction_type_24',
	'atom_88':	'interaction_type_24',
	'atom_93':	'interaction_type_1',
	'atom_111':	'interaction_type_27',
	'atom_112':	'interaction_type_28',
	'atom_114':	'interaction_type_1',
	'atom_146':	'interaction_type_37',
	'atom_147':	'interaction_type_37',
	'atom_151':	'interaction_type_38',
	'atom_152':	'interaction_type_38',
	'atom_155':	'interaction_type_1',
	'atom_156':	'interaction_type_1',
	'atom_158':	'interaction_type_39',
	'atom_159':	'interaction_type_39',
	'atom_160':	'interaction_type_38',
	'atom_161':	'interaction_type_38',
	'atom_162':	'interaction_type_40',
	'atom_165':	'interaction_type_1',
	'atom_166':	'interaction_type_1',
	'atom_167':	'interaction_type_41',
	'atom_168':	'interaction_type_42',
	'atom_169':	'interaction_type_43',
	'atom_170':	'interaction_type_44',
	'atom_171':	'interaction_type_24',
	'atom_172':	'interaction_type_1',
	'atom_173':	'interaction_type_45',
	'atom_174':	'interaction_type_46',
	'atom_175':	'interaction_type_47',
	'atom_178':	'interaction_type_48',
	'atom_181':	'interaction_type_49',
	'atom_182':	'interaction_type_1',
	'atom_183':	'interaction_type_1',
	'atom_185':	'interaction_type_1'
}

