# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 


class AtomType:
	def __init__(self, type_lst):
		# the following are the required fields

		# the name of the type system (must be a valid C++ symbol)
		self.name = 'Rosetta'

		# if the names of the types are not valid C++ symbols, this must be
		# a function that turns them into valid ones
		self.enumFn = lambda x: x

		# the name of the type system in a form that looks like a constant
		# in an enumeration
		self.enumName = 'ROSETTA_TYPE'

		# a map that goes from each type in type_list, to a list of raw types
		# (the raw types being strings in the form AA:ATOM)
		self.types = dict( [(type, []) for type in type_list] )

		# the list of types in this system. This defines the order in which
		# data tables and such have the types
		self.typeList = type_list

		# this snippet turns type_data into the format required by self.types
		for aa, data in type_data.items():
			for atom, type in data.items():
				self.types[type].append(aa + ':' + atom)

type_list = 		\
[
	'CNH2',
	'COO',
	'CH1',
	'CH2',
	'CH3',
	'ARO_C',
	'TRP_N',
	'HIS_N',
	'H2O_N',
	'LYS_N',
	'ARG_N',
	'PRO_N',
	'OH',
	'ONH2',
	'OOC',
	'S',
	'BB_N',
	'BB_CA',
	'BB_CO',
	'BB_OC',
	'PHOS',
	'POL_H',
	'APO_H',
	'ARO_H',
	'BB_HN',
	'H2O',
        'FE3P'
]

#The type data all appear to be in alphabetical order.
type_data = 		\
{
 'ALA': {'1HB':		'APO_H',
         '2HB': 	'APO_H',
         '3HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH3',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC'},

 'NALA': {'1HB':        'APO_H',
         '2HB':         'APO_H',
         '3HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH3',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

 'CALA': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         '3HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH3',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'ARG': {'1HB': 	'APO_H',
         '1HD': 	'APO_H',
         '1HG': 	'APO_H',
         '1HH1':	'POL_H',
         '1HH2':	'POL_H',
         '2HB': 	'APO_H',
         '2HD': 	'APO_H',
         '2HG': 	'APO_H',
         '2HH1':	'POL_H',
         '2HH2':	'POL_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD': 		'CH2',
         'CG': 		'CH2',
         'CZ': 		'ARO_C',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HE': 		'POL_H',
         'N': 		'BB_N',
         'NE': 		'ARG_N',
         'NH1': 	'ARG_N',
         'NH2': 	'ARG_N',
         'O': 		'BB_OC'},

 'NARG': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HG':         'APO_H',
         '1HH1':        'POL_H',
         '1HH2':        'POL_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HG':         'APO_H',
         '2HH1':        'POL_H',
         '2HH2':        'POL_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CG':          'CH2',
         'CZ':          'ARO_C',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HE':          'POL_H',
         'N':           'LYS_N',
         'NE':          'ARG_N',
         'NH1':         'ARG_N',
         'NH2':         'ARG_N',
         'O':           'BB_OC'},

 'CARG': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HG':         'APO_H',
         '1HH1':        'POL_H',
         '1HH2':        'POL_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HG':         'APO_H',
         '2HH1':        'POL_H',
         '2HH2':        'POL_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CG':          'CH2',
         'CZ':          'ARO_C',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HE':          'POL_H',
         'N':           'BB_N',
         'NE':          'ARG_N',
         'NH1':         'ARG_N',
         'NH2':         'ARG_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'ASN': {'1HB': 	'APO_H',
         '1HD2':	'POL_H',
         '2HB': 	'APO_H',
         '2HD2':	'POL_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CG': 		'CNH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'ND2': 	'H2O_N',
         'O': 		'BB_OC',
         'OD1': 	'ONH2'},

 'NASN': {'1HB':         'APO_H',
         '1HD2':        'POL_H',
         '2HB':         'APO_H',
         '2HD2':        'POL_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CG':          'CNH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'ND2':         'H2O_N',
         'O':           'BB_OC',
         'OD1':         'ONH2'},

 'CASN': {'1HB':         'APO_H',
         '1HD2':        'POL_H',
         '2HB':         'APO_H',
         '2HD2':        'POL_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CG':          'CNH2',
         'H' :          'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'ND2':         'H2O_N',
         'O':           'OOC',
         'OD1':         'ONH2',
         'OXT':         'OOC'},

 'ASP': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CG': 		'COO',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'OD1': 	'OOC',
         'OD2': 	'OOC'},

 'NASP': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CG':          'COO',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'OD1':         'OOC',
         'OD2':         'OOC'},

 'CASP': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CG':          'COO',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OD1':         'OOC',
         'OD2':         'OOC',
         'OXT':         'OOC'},

 'CYS': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HG': 		'POL_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'SG': 		'S'},

 'NCYS': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HG':          'POL_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'SG':          'S'},

 'CCYS': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HG':          'POL_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC',
         'SG':          'S'},

 'GLN': {'1HB': 	'APO_H',
         '1HE2':	'POL_H',
         '1HG': 	'APO_H',
         '2HB': 	'APO_H',
         '2HE2':	'POL_H',
         '2HG': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD': 		'CNH2',
         'CG': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'NE2': 	'H2O_N',
         'O': 		'BB_OC',
         'OE1': 	'ONH2'},

 'NGLN': {'1HB':         'APO_H',
         '1HE2':        'POL_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HE2':        'POL_H',
         '2HG':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CNH2',
         'CG':          'CH2',         
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'NE2':         'H2O_N',
         'O':           'BB_OC',
         'OE1':         'ONH2'},

 'CGLN': {'1HB':         'APO_H',
         '1HE2':        'POL_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HE2':        'POL_H',
         '2HG':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CNH2',
         'CG':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'NE2':         'H2O_N',
         'O':           'OOC',
         'OE1':         'ONH2',
         'OXT':         'OOC'},

 'GLU': {'1HB': 	'APO_H',
         '1HG': 	'APO_H',
         '2HB': 	'APO_H',
         '2HG': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD': 		'COO',
         'CG': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'OE1': 	'OOC',
         'OE2': 	'OOC'},

 'NGLU': {'1HB':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HG':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'COO',
         'CG':          'CH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'OE1':         'OOC',
         'OE2':         'OOC'},

 'CGLU': {'1HB':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HG':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'COO',
         'CG':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OE1':         'OOC',
         'OE2':         'OOC',
         'OXT':         'OOC'},

 'GLY': {'1HA': 	'APO_H',
         '2HA': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'H': 		'BB_HN',
         'N': 		'BB_N',
         'O': 		'BB_OC'},

 'NGLY': {'1HA':         'APO_H',
         '2HA':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

 'CGLY': {'1HA':         'APO_H',
         '2HA':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'H':           'BB_HN',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'HIS': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD2': 	'ARO_C',
         'CE1': 	'ARO_C',
         'CG': 		'ARO_C',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HD2': 	'APO_H',
         'HE1': 	'APO_H',
         'HE2': 	'POL_H',
         'N': 		'BB_N',
         'ND1': 	'HIS_N',
         'NE2': 	'TRP_N',
         'O': 		'BB_OC'},

 'NHIS': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CG':          'ARO_C',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HD2':         'APO_H',
         'HE1':         'APO_H',
         'HE2':         'POL_H',
         'N':           'LYS_N',
         'ND1':         'HIS_N',
         'NE2':         'TRP_N',
         'O':           'BB_OC'},

 'CHIS': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CG':          'ARO_C',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HD2':         'APO_H',
         'HE1':         'APO_H',
         'HE2':         'POL_H',
         'N':           'BB_N',
         'ND1':         'HIS_N',
         'NE2':         'TRP_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'ILE': {'1HD1':	'APO_H',
         '1HG1':	'APO_H',
         '1HG2':	'APO_H',
         '2HD1':	'APO_H',
         '2HG1':	'APO_H',
         '2HG2':	'APO_H',
         '3HD1':	'APO_H',
         '3HG2':	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH1',
         'CD1': 	'CH3',
         'CG1': 	'CH2',
         'CG2': 	'CH3',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HB': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC'},

 'NILE': {'1HD1':        'APO_H',
         '1HG1':        'APO_H',
         '1HG2':        'APO_H',
         '2HD1':        'APO_H',
         '2HG1':        'APO_H',
         '2HG2':        'APO_H',
         '3HD1':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CD1':         'CH3',
         'CG1':         'CH2',
         'CG2':         'CH3',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

 'CILE': {'1HD1':        'APO_H',
         '1HG1':        'APO_H',
         '1HG2':        'APO_H',
         '2HD1':        'APO_H',
         '2HG1':        'APO_H',
         '2HG2':        'APO_H',
         '3HD1':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CD1':         'CH3',
         'CG1':         'CH2',
         'CG2':         'CH3',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'LEU': {'1HB': 	'APO_H',
         '1HD1':	'APO_H',
         '1HD2':	'APO_H',
         '2HB': 	'APO_H',
         '2HD1':	'APO_H',
         '2HD2':	'APO_H',
         '3HD1':	'APO_H',
         '3HD2':	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD1': 	'CH3',
         'CD2': 	'CH3',
         'CG': 		'CH1',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HG': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC'},

 'NLEU': {'1HB':         'APO_H',
         '1HD1':        'APO_H',
         '1HD2':        'APO_H',
         '2HB':         'APO_H',
         '2HD1':        'APO_H',
         '2HD2':        'APO_H',
         '3HD1':        'APO_H',
         '3HD2':        'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'CH3',
         'CD2':         'CH3',
         'CG':          'CH1',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HG':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

 'CLEU': {'1HB':         'APO_H',
         '1HD1':        'APO_H',
         '1HD2':        'APO_H',
         '2HB':         'APO_H',
         '2HD1':        'APO_H',
         '2HD2':        'APO_H',
         '3HD1':        'APO_H',
         '3HD2':        'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'CH3',
         'CD2':         'CH3',
         'CG':          'CH1',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HG':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'LYS': {'1HB': 	'APO_H',
         '1HD': 	'APO_H',
         '1HE': 	'APO_H',
         '1HG': 	'APO_H',
         '1HZ': 	'POL_H',
         '2HB': 	'APO_H',
         '2HD': 	'APO_H',
         '2HE': 	'APO_H',
         '2HG': 	'APO_H',
         '2HZ': 	'POL_H',
         '3HZ': 	'POL_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD': 		'CH2',
         'CE': 		'CH2',
         'CG': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'NZ': 		'LYS_N',
         'O': 		'BB_OC'},

 'NLYS': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HE':         'APO_H',
         '1HG':         'APO_H',
         '1HZ':         'POL_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HE':         'APO_H',
         '2HG':         'APO_H',
         '2HZ':         'POL_H',
         '3HZ':         'POL_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CE':          'CH2',
         'CG':          'CH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'NZ':          'LYS_N',
         'O':           'BB_OC'},

 'CLYS': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HE':         'APO_H',
         '1HG':         'APO_H',
         '1HZ':         'POL_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HE':         'APO_H',
         '2HG':         'APO_H',
         '2HZ':         'POL_H',
         '3HZ':         'POL_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CE':          'CH2',
         'CG':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'NZ':          'LYS_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'MET': {'1HB': 	'APO_H',
         '1HE': 	'APO_H',
         '1HG': 	'APO_H',
         '2HB': 	'APO_H',
         '2HE': 	'APO_H',
         '2HG': 	'APO_H',
         '3HE': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CE': 		'CH3',
         'CG': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'SD': 		'S'},

 'NMET': {'1HB':         'APO_H',
         '1HE':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HE':         'APO_H',
         '2HG':         'APO_H',
         '3HE':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CE':          'CH3',
         'CG':          'CH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'SD':          'S'},

 'CMET': {'1HB':         'APO_H',
         '1HE':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HE':         'APO_H',
         '2HG':         'APO_H',
         '3HE':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CE':          'CH3',
         'CG':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC',
         'SD':          'S'},

 'PHE': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD1': 	'ARO_C',
         'CD2': 	'ARO_C',
         'CE1': 	'ARO_C',
         'CE2': 	'ARO_C',
         'CG': 		'ARO_C',
         'CZ': 		'ARO_C',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HD1': 	'ARO_H',
         'HD2': 	'ARO_H',
         'HE1': 	'ARO_H',
         'HE2': 	'ARO_H',
         'HZ': 		'ARO_H',
         'N': 		'BB_N',
         'O': 		'BB_OC'},

 'NPHE': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CE2':         'ARO_C',
         'CG':          'ARO_C',
         'CZ':          'ARO_C',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HD2':         'ARO_H',
         'HE1':         'ARO_H',
         'HE2':         'ARO_H',
         'HZ':          'ARO_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

 'CPHE': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CE2':         'ARO_C',
         'CG':          'ARO_C',
         'CZ':          'ARO_C',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HD2':         'ARO_H',
         'HE1':         'ARO_H',
         'HE2':         'ARO_H',
         'HZ':          'ARO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'PRO': {'1HB': 	'APO_H',
         '1HD': 	'APO_H',
         '1HG': 	'APO_H',
         '2HB': 	'APO_H',
         '2HD': 	'APO_H',
         '2HG': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD': 		'CH2',
         'CG': 		'CH2',
         'HA': 		'APO_H',
         'N': 		'PRO_N',
         'O': 		'BB_OC'},

 'NPRO': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HG':         'APO_H',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CG':          'CH2',
         'HA':          'APO_H',
         'N':           'PRO_N',
         'O':           'BB_OC'},

 'CPRO': {'1HB':         'APO_H',
         '1HD':         'APO_H',
         '1HG':         'APO_H',
         '2HB':         'APO_H',
         '2HD':         'APO_H',
         '2HG':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD':          'CH2',
         'CG':          'CH2',
         'HA':          'APO_H',
         'N':           'PRO_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'SER': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HG': 		'POL_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'OG': 		'OH'},

 'NSER': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HG':          'POL_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'OG':          'OH'},

 'CSER': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HG':          'POL_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OG':          'OH',
         'OXT':         'OOC'},

 'THR': {'1HG2':	'APO_H',
         '2HG2':	'APO_H',
         '3HG2':	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH1',
         'CG2': 	'CH3',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HB': 		'APO_H',
         'HG1': 	'POL_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'OG1': 	'OH'},

 'NTHR': {'1HG2':        'APO_H',
         '2HG2':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CG2':         'CH3',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'HG1':         'POL_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'OG1':         'OH'},

 'CTHR': {'1HG2':        'APO_H',
         '2HG2':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CG2':         'CH3',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'HG1':         'POL_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OG1':         'OH',
         'OXT':         'OOC'},

'TRP': {'1HB': 		'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD1': 	'ARO_C',
         'CD2': 	'ARO_C',
         'CE2': 	'ARO_C',
         'CE3': 	'ARO_C',
         'CG': 		'ARO_C',
         'CH2': 	'ARO_C',
         'CZ2': 	'ARO_C',
         'CZ3': 	'ARO_C',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HD1': 	'ARO_H',
         'HE1': 	'POL_H',
         'HE3': 	'ARO_H',
         'HH2': 	'ARO_H',
         'HZ2': 	'ARO_H',
         'HZ3': 	'ARO_H',
         'N': 		'BB_N',
         'NE1': 	'TRP_N',
         'O': 		'BB_OC'},

'NTRP': {'1HB':          'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE2':         'ARO_C',
         'CE3':         'ARO_C',
         'CG':          'ARO_C',
         'CH2':         'ARO_C',
         'CZ2':         'ARO_C',
         'CZ3':         'ARO_C',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HE1':         'POL_H',
         'HE3':         'ARO_H',
         'HH2':         'ARO_H',
         'HZ2':         'ARO_H',
         'HZ3':         'ARO_H',
         'N':           'LYS_N',
         'NE1':         'TRP_N',
         'O':           'BB_OC'},

'CTRP': {'1HB':          'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE2':         'ARO_C',
         'CE3':         'ARO_C',
         'CG':          'ARO_C',
         'CH2':         'ARO_C',
         'CZ2':         'ARO_C',
         'CZ3':         'ARO_C',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HE1':         'POL_H',
         'HE3':         'ARO_H',
         'HH2':         'ARO_H',
         'HZ2':         'ARO_H',
         'HZ3':         'ARO_H',
         'N':           'BB_N',
         'NE1':         'TRP_N',
         'O':           'OOC',
         'OXT':         'OOC'},

 'TYR': {'1HB': 	'APO_H',
         '2HB': 	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH2',
         'CD1': 	'ARO_C',
         'CD2': 	'ARO_C',
         'CE1': 	'ARO_C',
         'CE2': 	'ARO_C',
         'CG': 		'ARO_C',
         'CZ': 		'ARO_C',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HD1': 	'ARO_H',
         'HD2': 	'ARO_H',
         'HE1': 	'ARO_H',
         'HE2': 	'ARO_H',
         'HH': 		'POL_H',
         'N': 		'BB_N',
         'O': 		'BB_OC',
         'OH': 		'OH'},

 'NTYR': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CE2':         'ARO_C',
         'CG':          'ARO_C',
         'CZ':          'ARO_C',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HD2':         'ARO_H',
         'HE1':         'ARO_H',
         'HE2':         'ARO_H',
         'HH':          'POL_H',
         'N':           'LYS_N',
         'O':           'BB_OC',
         'OH':          'OH'},

 'CTYR': {'1HB':         'APO_H',
         '2HB':         'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH2',
         'CD1':         'ARO_C',
         'CD2':         'ARO_C',
         'CE1':         'ARO_C',
         'CE2':         'ARO_C',
         'CG':          'ARO_C',
         'CZ':          'ARO_C',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HD1':         'ARO_H',
         'HD2':         'ARO_H',
         'HE1':         'ARO_H',
         'HE2':         'ARO_H',
         'HH':          'POL_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OH':          'OH',
         'OXT':         'OOC'},

'VAL': {'1HG1':		'APO_H',
         '1HG2':	'APO_H',
         '2HG1':	'APO_H',
         '2HG2':	'APO_H',
         '3HG1':	'APO_H',
         '3HG2':	'APO_H',
         'C': 		'BB_CO',
         'CA': 		'BB_CA',
         'CB': 		'CH1',
         'CG1': 	'CH3',
         'CG2': 	'CH3',
         'H': 		'BB_HN',
         'HA': 		'APO_H',
         'HB': 		'APO_H',
         'N': 		'BB_N',
	 'O': 		'BB_OC'},

'NVAL': {'1HG1':         'APO_H',
         '1HG2':        'APO_H',
         '2HG1':        'APO_H',
         '2HG2':        'APO_H',
         '3HG1':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'BB_CO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CG1':         'CH3',
         'CG2':         'CH3',
         'H1':          'POL_H',
         'H2':          'POL_H',
         'H3':          'POL_H',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'N':           'LYS_N',
         'O':           'BB_OC'},

'CVAL': {'1HG1':         'APO_H',
         '1HG2':        'APO_H',
         '2HG1':        'APO_H',
         '2HG2':        'APO_H',
         '3HG1':        'APO_H',
         '3HG2':        'APO_H',
         'C':           'COO',
         'CA':          'BB_CA',
         'CB':          'CH1',
         'CG1':         'CH3',
         'CG2':         'CH3',
         'H':           'BB_HN',
         'HA':          'APO_H',
         'HB':          'APO_H',
         'N':           'BB_N',
         'O':           'OOC',
         'OXT':         'OOC'}

}

