// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#define N_RAW_ATOM_TYPES 1032
#define N_ATOM_TYPE_SYSTEMS 6

namespace AtomType {

enum TypeSystem {
	RAW_TYPE,
	HYBRIDIZATION,
	ROSETTA_TYPE,
	COUNT_PAIR_TYPE,
	OPLSaa_INTERACTION_TYPE,
	OPLSaa_TYPE,
};
struct Raw {

enum {
	ALA_C,
	ALA_O,
	ALA_N,
	ALA_H,
	ALA_CA,
	ALA_HA,
	ALA_CB,
	ALA_1HB,
	ALA_2HB,
	ALA_3HB,
	CYS_C,
	CYS_O,
	CYS_N,
	CYS_H,
	CYS_CA,
	CYS_HA,
	CYS_CB,
	CYS_1HB,
	CYS_2HB,
	CYS_SG,
	CYS_HG,
	ASP_C,
	ASP_O,
	ASP_N,
	ASP_H,
	ASP_CA,
	ASP_HA,
	ASP_CB,
	ASP_1HB,
	ASP_2HB,
	ASP_CG,
	ASP_OD1,
	ASP_OD2,
	GLU_C,
	GLU_O,
	GLU_N,
	GLU_H,
	GLU_CA,
	GLU_HA,
	GLU_CB,
	GLU_1HB,
	GLU_2HB,
	GLU_CG,
	GLU_1HG,
	GLU_2HG,
	GLU_CD,
	GLU_OE1,
	GLU_OE2,
	PHE_C,
	PHE_O,
	PHE_N,
	PHE_H,
	PHE_CA,
	PHE_HA,
	PHE_CB,
	PHE_1HB,
	PHE_2HB,
	PHE_CG,
	PHE_CD1,
	PHE_HD1,
	PHE_CD2,
	PHE_HD2,
	PHE_CE1,
	PHE_HE1,
	PHE_CE2,
	PHE_HE2,
	PHE_CZ,
	PHE_HZ,
	GLY_C,
	GLY_O,
	GLY_N,
	GLY_H,
	GLY_CA,
	GLY_1HA,
	GLY_2HA,
	HIS_C,
	HIS_O,
	HIS_N,
	HIS_H,
	HIS_CA,
	HIS_HA,
	HIS_CB,
	HIS_1HB,
	HIS_2HB,
	HIS_CG,
	HIS_ND1,
	HIS_CD2,
	HIS_HD2,
	HIS_CE1,
	HIS_HE1,
	HIS_NE2,
	HIS_HE2,
	ILE_C,
	ILE_O,
	ILE_N,
	ILE_H,
	ILE_CA,
	ILE_HA,
	ILE_CB,
	ILE_HB,
	ILE_CG2,
	ILE_1HG2,
	ILE_2HG2,
	ILE_3HG2,
	ILE_CG1,
	ILE_1HG1,
	ILE_2HG1,
	ILE_CD1,
	ILE_1HD1,
	ILE_2HD1,
	ILE_3HD1,
	LYS_C,
	LYS_O,
	LYS_N,
	LYS_H,
	LYS_CA,
	LYS_HA,
	LYS_CB,
	LYS_1HB,
	LYS_2HB,
	LYS_CG,
	LYS_1HG,
	LYS_2HG,
	LYS_CD,
	LYS_1HD,
	LYS_2HD,
	LYS_CE,
	LYS_1HE,
	LYS_2HE,
	LYS_NZ,
	LYS_1HZ,
	LYS_2HZ,
	LYS_3HZ,
	LEU_C,
	LEU_O,
	LEU_N,
	LEU_H,
	LEU_CA,
	LEU_HA,
	LEU_CB,
	LEU_1HB,
	LEU_2HB,
	LEU_CG,
	LEU_HG,
	LEU_CD1,
	LEU_1HD1,
	LEU_2HD1,
	LEU_3HD1,
	LEU_CD2,
	LEU_1HD2,
	LEU_2HD2,
	LEU_3HD2,
	MET_C,
	MET_O,
	MET_N,
	MET_H,
	MET_CA,
	MET_HA,
	MET_CB,
	MET_1HB,
	MET_2HB,
	MET_CG,
	MET_1HG,
	MET_2HG,
	MET_SD,
	MET_CE,
	MET_1HE,
	MET_2HE,
	MET_3HE,
	ASN_C,
	ASN_O,
	ASN_N,
	ASN_H,
	ASN_CA,
	ASN_HA,
	ASN_CB,
	ASN_1HB,
	ASN_2HB,
	ASN_CG,
	ASN_OD1,
	ASN_ND2,
	ASN_1HD2,
	ASN_2HD2,
	PRO_C,
	PRO_O,
	PRO_N,
	PRO_CA,
	PRO_HA,
	PRO_CB,
	PRO_1HB,
	PRO_2HB,
	PRO_CG,
	PRO_1HG,
	PRO_2HG,
	PRO_CD,
	PRO_1HD,
	PRO_2HD,
	GLN_C,
	GLN_O,
	GLN_N,
	GLN_H,
	GLN_CA,
	GLN_HA,
	GLN_CB,
	GLN_1HB,
	GLN_2HB,
	GLN_CG,
	GLN_1HG,
	GLN_2HG,
	GLN_CD,
	GLN_OE1,
	GLN_NE2,
	GLN_1HE2,
	GLN_2HE2,
	ARG_C,
	ARG_O,
	ARG_N,
	ARG_H,
	ARG_CA,
	ARG_HA,
	ARG_CB,
	ARG_1HB,
	ARG_2HB,
	ARG_CG,
	ARG_1HG,
	ARG_2HG,
	ARG_CD,
	ARG_1HD,
	ARG_2HD,
	ARG_NE,
	ARG_HE,
	ARG_CZ,
	ARG_NH1,
	ARG_1HH1,
	ARG_2HH1,
	ARG_NH2,
	ARG_1HH2,
	ARG_2HH2,
	SER_C,
	SER_O,
	SER_N,
	SER_H,
	SER_CA,
	SER_HA,
	SER_CB,
	SER_1HB,
	SER_2HB,
	SER_OG,
	SER_HG,
	THR_C,
	THR_O,
	THR_N,
	THR_H,
	THR_CA,
	THR_HA,
	THR_CB,
	THR_HB,
	THR_CG2,
	THR_1HG2,
	THR_2HG2,
	THR_3HG2,
	THR_OG1,
	THR_HG1,
	VAL_C,
	VAL_O,
	VAL_N,
	VAL_H,
	VAL_CA,
	VAL_HA,
	VAL_CB,
	VAL_HB,
	VAL_CG1,
	VAL_1HG1,
	VAL_2HG1,
	VAL_3HG1,
	VAL_CG2,
	VAL_1HG2,
	VAL_2HG2,
	VAL_3HG2,
	TRP_C,
	TRP_O,
	TRP_N,
	TRP_H,
	TRP_CA,
	TRP_HA,
	TRP_CB,
	TRP_1HB,
	TRP_2HB,
	TRP_CG,
	TRP_CD1,
	TRP_HD1,
	TRP_CD2,
	TRP_NE1,
	TRP_HE1,
	TRP_CE2,
	TRP_CE3,
	TRP_HE3,
	TRP_CZ2,
	TRP_HZ2,
	TRP_CZ3,
	TRP_HZ3,
	TRP_CH2,
	TRP_HH2,
	TYR_C,
	TYR_O,
	TYR_N,
	TYR_H,
	TYR_CA,
	TYR_HA,
	TYR_CB,
	TYR_1HB,
	TYR_2HB,
	TYR_CG,
	TYR_CD1,
	TYR_HD1,
	TYR_CD2,
	TYR_HD2,
	TYR_CE1,
	TYR_HE1,
	TYR_CE2,
	TYR_HE2,
	TYR_CZ,
	TYR_OH,
	TYR_HH,
	NALA_C,
	NALA_O,
	NALA_N,
	NALA_H1,
	NALA_CA,
	NALA_HA,
	NALA_CB,
	NALA_1HB,
	NALA_2HB,
	NALA_3HB,
	NALA_H2,
	NALA_H3,
	NCYS_C,
	NCYS_O,
	NCYS_N,
	NCYS_H1,
	NCYS_CA,
	NCYS_HA,
	NCYS_CB,
	NCYS_1HB,
	NCYS_2HB,
	NCYS_SG,
	NCYS_HG,
	NCYS_H2,
	NCYS_H3,
	NASP_C,
	NASP_O,
	NASP_N,
	NASP_H1,
	NASP_CA,
	NASP_HA,
	NASP_CB,
	NASP_1HB,
	NASP_2HB,
	NASP_CG,
	NASP_OD1,
	NASP_OD2,
	NASP_H2,
	NASP_H3,
	NGLU_C,
	NGLU_O,
	NGLU_N,
	NGLU_H1,
	NGLU_CA,
	NGLU_HA,
	NGLU_CB,
	NGLU_1HB,
	NGLU_2HB,
	NGLU_CG,
	NGLU_1HG,
	NGLU_2HG,
	NGLU_CD,
	NGLU_OE1,
	NGLU_OE2,
	NGLU_H2,
	NGLU_H3,
	NPHE_C,
	NPHE_O,
	NPHE_N,
	NPHE_H1,
	NPHE_CA,
	NPHE_HA,
	NPHE_CB,
	NPHE_1HB,
	NPHE_2HB,
	NPHE_CG,
	NPHE_CD1,
	NPHE_HD1,
	NPHE_CD2,
	NPHE_HD2,
	NPHE_CE1,
	NPHE_HE1,
	NPHE_CE2,
	NPHE_HE2,
	NPHE_CZ,
	NPHE_HZ,
	NPHE_H2,
	NPHE_H3,
	NGLY_C,
	NGLY_O,
	NGLY_N,
	NGLY_H1,
	NGLY_CA,
	NGLY_1HA,
	NGLY_2HA,
	NGLY_H2,
	NGLY_H3,
	NHIS_C,
	NHIS_O,
	NHIS_N,
	NHIS_H1,
	NHIS_CA,
	NHIS_HA,
	NHIS_CB,
	NHIS_1HB,
	NHIS_2HB,
	NHIS_CG,
	NHIS_ND1,
	NHIS_CD2,
	NHIS_HD2,
	NHIS_CE1,
	NHIS_HE1,
	NHIS_NE2,
	NHIS_HE2,
	NHIS_H2,
	NHIS_H3,
	NILE_C,
	NILE_O,
	NILE_N,
	NILE_H1,
	NILE_CA,
	NILE_HA,
	NILE_CB,
	NILE_HB,
	NILE_CG2,
	NILE_1HG2,
	NILE_2HG2,
	NILE_3HG2,
	NILE_CG1,
	NILE_1HG1,
	NILE_2HG1,
	NILE_CD1,
	NILE_1HD1,
	NILE_2HD1,
	NILE_3HD1,
	NILE_H2,
	NILE_H3,
	NLYS_C,
	NLYS_O,
	NLYS_N,
	NLYS_H1,
	NLYS_CA,
	NLYS_HA,
	NLYS_CB,
	NLYS_1HB,
	NLYS_2HB,
	NLYS_CG,
	NLYS_1HG,
	NLYS_2HG,
	NLYS_CD,
	NLYS_1HD,
	NLYS_2HD,
	NLYS_CE,
	NLYS_1HE,
	NLYS_2HE,
	NLYS_NZ,
	NLYS_1HZ,
	NLYS_2HZ,
	NLYS_3HZ,
	NLYS_H2,
	NLYS_H3,
	NLEU_C,
	NLEU_O,
	NLEU_N,
	NLEU_H1,
	NLEU_CA,
	NLEU_HA,
	NLEU_CB,
	NLEU_1HB,
	NLEU_2HB,
	NLEU_CG,
	NLEU_HG,
	NLEU_CD1,
	NLEU_1HD1,
	NLEU_2HD1,
	NLEU_3HD1,
	NLEU_CD2,
	NLEU_1HD2,
	NLEU_2HD2,
	NLEU_3HD2,
	NLEU_H2,
	NLEU_H3,
	NMET_C,
	NMET_O,
	NMET_N,
	NMET_H1,
	NMET_CA,
	NMET_HA,
	NMET_CB,
	NMET_1HB,
	NMET_2HB,
	NMET_CG,
	NMET_1HG,
	NMET_2HG,
	NMET_SD,
	NMET_CE,
	NMET_1HE,
	NMET_2HE,
	NMET_3HE,
	NMET_H2,
	NMET_H3,
	NASN_C,
	NASN_O,
	NASN_N,
	NASN_H1,
	NASN_CA,
	NASN_HA,
	NASN_CB,
	NASN_1HB,
	NASN_2HB,
	NASN_CG,
	NASN_OD1,
	NASN_ND2,
	NASN_1HD2,
	NASN_2HD2,
	NASN_H2,
	NASN_H3,
	NPRO_C,
	NPRO_O,
	NPRO_N,
	NPRO_CA,
	NPRO_HA,
	NPRO_CB,
	NPRO_1HB,
	NPRO_2HB,
	NPRO_CG,
	NPRO_1HG,
	NPRO_2HG,
	NPRO_CD,
	NPRO_1HD,
	NPRO_2HD,
	NPRO_H1,
	NPRO_H2,
	NGLN_C,
	NGLN_O,
	NGLN_N,
	NGLN_H1,
	NGLN_CA,
	NGLN_HA,
	NGLN_CB,
	NGLN_1HB,
	NGLN_2HB,
	NGLN_CG,
	NGLN_1HG,
	NGLN_2HG,
	NGLN_CD,
	NGLN_OE1,
	NGLN_NE2,
	NGLN_1HE2,
	NGLN_2HE2,
	NGLN_H2,
	NGLN_H3,
	NARG_C,
	NARG_O,
	NARG_N,
	NARG_H1,
	NARG_CA,
	NARG_HA,
	NARG_CB,
	NARG_1HB,
	NARG_2HB,
	NARG_CG,
	NARG_1HG,
	NARG_2HG,
	NARG_CD,
	NARG_1HD,
	NARG_2HD,
	NARG_NE,
	NARG_HE,
	NARG_CZ,
	NARG_NH1,
	NARG_1HH1,
	NARG_2HH1,
	NARG_NH2,
	NARG_1HH2,
	NARG_2HH2,
	NARG_H2,
	NARG_H3,
	NSER_C,
	NSER_O,
	NSER_N,
	NSER_H1,
	NSER_CA,
	NSER_HA,
	NSER_CB,
	NSER_1HB,
	NSER_2HB,
	NSER_OG,
	NSER_HG,
	NSER_H2,
	NSER_H3,
	NTHR_C,
	NTHR_O,
	NTHR_N,
	NTHR_H1,
	NTHR_CA,
	NTHR_HA,
	NTHR_CB,
	NTHR_HB,
	NTHR_CG2,
	NTHR_1HG2,
	NTHR_2HG2,
	NTHR_3HG2,
	NTHR_OG1,
	NTHR_HG1,
	NTHR_H2,
	NTHR_H3,
	NVAL_C,
	NVAL_O,
	NVAL_N,
	NVAL_H1,
	NVAL_CA,
	NVAL_HA,
	NVAL_CB,
	NVAL_HB,
	NVAL_CG1,
	NVAL_1HG1,
	NVAL_2HG1,
	NVAL_3HG1,
	NVAL_CG2,
	NVAL_1HG2,
	NVAL_2HG2,
	NVAL_3HG2,
	NVAL_H2,
	NVAL_H3,
	NTRP_C,
	NTRP_O,
	NTRP_N,
	NTRP_H1,
	NTRP_CA,
	NTRP_HA,
	NTRP_CB,
	NTRP_1HB,
	NTRP_2HB,
	NTRP_CG,
	NTRP_CD1,
	NTRP_HD1,
	NTRP_CD2,
	NTRP_NE1,
	NTRP_HE1,
	NTRP_CE2,
	NTRP_CE3,
	NTRP_HE3,
	NTRP_CZ2,
	NTRP_HZ2,
	NTRP_CZ3,
	NTRP_HZ3,
	NTRP_CH2,
	NTRP_HH2,
	NTRP_H2,
	NTRP_H3,
	NTYR_C,
	NTYR_O,
	NTYR_N,
	NTYR_H1,
	NTYR_CA,
	NTYR_HA,
	NTYR_CB,
	NTYR_1HB,
	NTYR_2HB,
	NTYR_CG,
	NTYR_CD1,
	NTYR_HD1,
	NTYR_CD2,
	NTYR_HD2,
	NTYR_CE1,
	NTYR_HE1,
	NTYR_CE2,
	NTYR_HE2,
	NTYR_CZ,
	NTYR_OH,
	NTYR_HH,
	NTYR_H2,
	NTYR_H3,
	CALA_C,
	CALA_O,
	CALA_N,
	CALA_H,
	CALA_CA,
	CALA_HA,
	CALA_CB,
	CALA_1HB,
	CALA_2HB,
	CALA_3HB,
	CALA_OXT,
	CCYS_C,
	CCYS_O,
	CCYS_N,
	CCYS_H,
	CCYS_CA,
	CCYS_HA,
	CCYS_CB,
	CCYS_1HB,
	CCYS_2HB,
	CCYS_SG,
	CCYS_HG,
	CCYS_OXT,
	CASP_C,
	CASP_O,
	CASP_N,
	CASP_H,
	CASP_CA,
	CASP_HA,
	CASP_CB,
	CASP_1HB,
	CASP_2HB,
	CASP_CG,
	CASP_OD1,
	CASP_OD2,
	CASP_OXT,
	CGLU_C,
	CGLU_O,
	CGLU_N,
	CGLU_H,
	CGLU_CA,
	CGLU_HA,
	CGLU_CB,
	CGLU_1HB,
	CGLU_2HB,
	CGLU_CG,
	CGLU_1HG,
	CGLU_2HG,
	CGLU_CD,
	CGLU_OE1,
	CGLU_OE2,
	CGLU_OXT,
	CPHE_C,
	CPHE_O,
	CPHE_N,
	CPHE_H,
	CPHE_CA,
	CPHE_HA,
	CPHE_CB,
	CPHE_1HB,
	CPHE_2HB,
	CPHE_CG,
	CPHE_CD1,
	CPHE_HD1,
	CPHE_CD2,
	CPHE_HD2,
	CPHE_CE1,
	CPHE_HE1,
	CPHE_CE2,
	CPHE_HE2,
	CPHE_CZ,
	CPHE_HZ,
	CPHE_OXT,
	CGLY_C,
	CGLY_O,
	CGLY_N,
	CGLY_H,
	CGLY_CA,
	CGLY_1HA,
	CGLY_2HA,
	CGLY_OXT,
	CHIS_C,
	CHIS_O,
	CHIS_N,
	CHIS_H,
	CHIS_CA,
	CHIS_HA,
	CHIS_CB,
	CHIS_1HB,
	CHIS_2HB,
	CHIS_CG,
	CHIS_ND1,
	CHIS_CD2,
	CHIS_HD2,
	CHIS_CE1,
	CHIS_HE1,
	CHIS_NE2,
	CHIS_HE2,
	CHIS_OXT,
	CILE_C,
	CILE_O,
	CILE_N,
	CILE_H,
	CILE_CA,
	CILE_HA,
	CILE_CB,
	CILE_HB,
	CILE_CG2,
	CILE_1HG2,
	CILE_2HG2,
	CILE_3HG2,
	CILE_CG1,
	CILE_1HG1,
	CILE_2HG1,
	CILE_CD1,
	CILE_1HD1,
	CILE_2HD1,
	CILE_3HD1,
	CILE_OXT,
	CLYS_C,
	CLYS_O,
	CLYS_N,
	CLYS_H,
	CLYS_CA,
	CLYS_HA,
	CLYS_CB,
	CLYS_1HB,
	CLYS_2HB,
	CLYS_CG,
	CLYS_1HG,
	CLYS_2HG,
	CLYS_CD,
	CLYS_1HD,
	CLYS_2HD,
	CLYS_CE,
	CLYS_1HE,
	CLYS_2HE,
	CLYS_NZ,
	CLYS_1HZ,
	CLYS_2HZ,
	CLYS_3HZ,
	CLYS_OXT,
	CLEU_C,
	CLEU_O,
	CLEU_N,
	CLEU_H,
	CLEU_CA,
	CLEU_HA,
	CLEU_CB,
	CLEU_1HB,
	CLEU_2HB,
	CLEU_CG,
	CLEU_HG,
	CLEU_CD1,
	CLEU_1HD1,
	CLEU_2HD1,
	CLEU_3HD1,
	CLEU_CD2,
	CLEU_1HD2,
	CLEU_2HD2,
	CLEU_3HD2,
	CLEU_OXT,
	CMET_C,
	CMET_O,
	CMET_N,
	CMET_H,
	CMET_CA,
	CMET_HA,
	CMET_CB,
	CMET_1HB,
	CMET_2HB,
	CMET_CG,
	CMET_1HG,
	CMET_2HG,
	CMET_SD,
	CMET_CE,
	CMET_1HE,
	CMET_2HE,
	CMET_3HE,
	CMET_OXT,
	CASN_C,
	CASN_O,
	CASN_N,
	CASN_H,
	CASN_CA,
	CASN_HA,
	CASN_CB,
	CASN_1HB,
	CASN_2HB,
	CASN_CG,
	CASN_OD1,
	CASN_ND2,
	CASN_1HD2,
	CASN_2HD2,
	CASN_OXT,
	CPRO_C,
	CPRO_O,
	CPRO_N,
	CPRO_CA,
	CPRO_HA,
	CPRO_CB,
	CPRO_1HB,
	CPRO_2HB,
	CPRO_CG,
	CPRO_1HG,
	CPRO_2HG,
	CPRO_CD,
	CPRO_1HD,
	CPRO_2HD,
	CPRO_OXT,
	CGLN_C,
	CGLN_O,
	CGLN_N,
	CGLN_H,
	CGLN_CA,
	CGLN_HA,
	CGLN_CB,
	CGLN_1HB,
	CGLN_2HB,
	CGLN_CG,
	CGLN_1HG,
	CGLN_2HG,
	CGLN_CD,
	CGLN_OE1,
	CGLN_NE2,
	CGLN_1HE2,
	CGLN_2HE2,
	CGLN_OXT,
	CARG_C,
	CARG_O,
	CARG_N,
	CARG_H,
	CARG_CA,
	CARG_HA,
	CARG_CB,
	CARG_1HB,
	CARG_2HB,
	CARG_CG,
	CARG_1HG,
	CARG_2HG,
	CARG_CD,
	CARG_1HD,
	CARG_2HD,
	CARG_NE,
	CARG_HE,
	CARG_CZ,
	CARG_NH1,
	CARG_1HH1,
	CARG_2HH1,
	CARG_NH2,
	CARG_1HH2,
	CARG_2HH2,
	CARG_OXT,
	CSER_C,
	CSER_O,
	CSER_N,
	CSER_H,
	CSER_CA,
	CSER_HA,
	CSER_CB,
	CSER_1HB,
	CSER_2HB,
	CSER_OG,
	CSER_HG,
	CSER_OXT,
	CTHR_C,
	CTHR_O,
	CTHR_N,
	CTHR_H,
	CTHR_CA,
	CTHR_HA,
	CTHR_CB,
	CTHR_HB,
	CTHR_CG2,
	CTHR_1HG2,
	CTHR_2HG2,
	CTHR_3HG2,
	CTHR_OG1,
	CTHR_HG1,
	CTHR_OXT,
	CVAL_C,
	CVAL_O,
	CVAL_N,
	CVAL_H,
	CVAL_CA,
	CVAL_HA,
	CVAL_CB,
	CVAL_HB,
	CVAL_CG1,
	CVAL_1HG1,
	CVAL_2HG1,
	CVAL_3HG1,
	CVAL_CG2,
	CVAL_1HG2,
	CVAL_2HG2,
	CVAL_3HG2,
	CVAL_OXT,
	CTRP_C,
	CTRP_O,
	CTRP_N,
	CTRP_H,
	CTRP_CA,
	CTRP_HA,
	CTRP_CB,
	CTRP_1HB,
	CTRP_2HB,
	CTRP_CG,
	CTRP_CD1,
	CTRP_HD1,
	CTRP_CD2,
	CTRP_NE1,
	CTRP_HE1,
	CTRP_CE2,
	CTRP_CE3,
	CTRP_HE3,
	CTRP_CZ2,
	CTRP_HZ2,
	CTRP_CZ3,
	CTRP_HZ3,
	CTRP_CH2,
	CTRP_HH2,
	CTRP_OXT,
	CTYR_C,
	CTYR_O,
	CTYR_N,
	CTYR_H,
	CTYR_CA,
	CTYR_HA,
	CTYR_CB,
	CTYR_1HB,
	CTYR_2HB,
	CTYR_CG,
	CTYR_CD1,
	CTYR_HD1,
	CTYR_CD2,
	CTYR_HD2,
	CTYR_CE1,
	CTYR_HE1,
	CTYR_CE2,
	CTYR_HE2,
	CTYR_CZ,
	CTYR_OH,
	CTYR_HH,
	CTYR_OXT,
};
};

struct Hybridization {

enum {
	SP2,
	SP3,
	RING,
	H,
};
};

struct Rosetta {

enum {
	CNH2,
	COO,
	CH1,
	CH2,
	CH3,
	ARO_C,
	TRP_N,
	HIS_N,
	H2O_N,
	LYS_N,
	ARG_N,
	PRO_N,
	OH,
	ONH2,
	OOC,
	S,
	BB_N,
	BB_CA,
	BB_CO,
	BB_OC,
	PHOS,
	POL_H,
	APO_H,
	ARO_H,
	BB_HN,
	H2O,
	FE3P,
};
};

struct CountPair {

enum {
	N,
	CA,
	C,
	O,
	CB,
	SC,
	HN,
	HA,
	PRO_CD,
	PRO_HD,
};
};

struct OPLSaa_Interaction {

enum {
	interaction_type_1,
	interaction_type_2,
	interaction_type_5,
	interaction_type_6,
	interaction_type_7,
	interaction_type_8,
	interaction_type_10,
	interaction_type_15,
	interaction_type_17,
	interaction_type_18,
	interaction_type_21,
	interaction_type_22,
	interaction_type_23,
	interaction_type_24,
	interaction_type_27,
	interaction_type_28,
	interaction_type_37,
	interaction_type_38,
	interaction_type_39,
	interaction_type_40,
	interaction_type_41,
	interaction_type_42,
	interaction_type_43,
	interaction_type_44,
	interaction_type_45,
	interaction_type_46,
	interaction_type_47,
	interaction_type_48,
	interaction_type_49,
};
};

struct OPLSaa {

enum {
	atom_1,
	atom_2,
	atom_3,
	atom_6,
	atom_11,
	atom_12,
	atom_15,
	atom_16,
	atom_17,
	atom_19,
	atom_20,
	atom_28,
	atom_29,
	atom_30,
	atom_52,
	atom_54,
	atom_56,
	atom_59,
	atom_62,
	atom_63,
	atom_73,
	atom_74,
	atom_82,
	atom_83,
	atom_84,
	atom_85,
	atom_86,
	atom_87,
	atom_88,
	atom_93,
	atom_111,
	atom_112,
	atom_114,
	atom_146,
	atom_147,
	atom_151,
	atom_152,
	atom_155,
	atom_156,
	atom_158,
	atom_159,
	atom_160,
	atom_161,
	atom_162,
	atom_165,
	atom_166,
	atom_167,
	atom_168,
	atom_169,
	atom_170,
	atom_171,
	atom_172,
	atom_173,
	atom_174,
	atom_175,
	atom_178,
	atom_181,
	atom_182,
	atom_183,
	atom_185,
};
};



}

