// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "energy_function.h"

int EnergyFunction::efcount_ = 0;
std::map<std::string, int> EnergyFunction::loaded_efs_;
std::string EnergyFunction::ef_dirname_;
std::string EnergyFunction::ef_path_ = ".";

bool atexit_set = false;

///////////////////////////////////////////////////////////////////////////
//////                         HELPER FNS                           ///////
///////////////////////////////////////////////////////////////////////////

void EnergyFunction::UnloadAll() 
{
#ifndef __APPLE__
#ifdef __GNUC__
	lt_dlexit();
#endif
#endif
	DeleteLoadedModuleDir();
}

void EnergyFunction::SetPluginPath(std::string const& path)
{
	// clear out the current path, if necessary
	if (efcount_ == 0)
		DeleteLoadedModuleDir();

	ef_path_ = path;

	// clear out the new path, if necessary
	if (efcount_ == 0)
		DeleteLoadedModuleDir();

	CreateLoadedModuleDir();
}

std::string EnergyFunction::GetPluginPath()
{
	return ef_path_;
}

void EnergyFunction::CreateLoadedModuleDir()
{
	char dirn[256], cmd[256];
#ifdef _MSC_VER
	sprintf(dirn, "%s\\.eflibs_p%d", ef_path_.c_str(), GetCurrentProcessId());
#else
	sprintf(dirn, "%s/.eflibs_p%d", ef_path_.c_str(), getpid());
#endif
	sprintf(cmd, "mkdir %s", dirn);
	system(cmd);

	ef_dirname_ = dirn;
}

void EnergyFunction::DeleteLoadedModuleDir()
{
	char cmd[256];
#ifdef _MSC_VER
	sprintf(cmd, "rd /S /Q %s", ef_dirname_.c_str());
#else
	sprintf(cmd, "rm -rf %s", ef_dirname_.c_str());
#endif
	system(cmd);
}

void EnergyFunction::LoadModule(std::string const& libName)
{
	char filename[256], cmd[256];
	int load_count = loaded_efs_[libName];

#ifdef _MSC_VER
	sprintf(filename, "%s\\inst%d_%s", ef_dirname_.c_str(), load_count, libName.c_str());

	// try copying over the annoying files for now
	sprintf(cmd, "copy Microsoft.VC90.CRT.manifest %s", ef_dirname_.c_str());
	system(cmd);
	sprintf(cmd, "copy msvcm90.dll %s", ef_dirname_.c_str());
	system(cmd);
	sprintf(cmd, "copy msvcp90.dll %s", ef_dirname_.c_str());
	system(cmd);
	sprintf(cmd, "copy msvcr90.dll %s", ef_dirname_.c_str());
	system(cmd);

	sprintf(cmd, "copy %s\\%s %s", ef_path_.c_str(), libName.c_str(), filename);
#else
	sprintf(filename, "%s/inst%d_%s", ef_dirname_.c_str(), load_count, libName.c_str());
	sprintf(cmd, "cp %s/%s %s", ef_path_.c_str(), libName.c_str(), filename);
#endif
	system(cmd);

#ifdef __APPLE__
        module_ = dlopen(filename, RTLD_LOCAL|RTLD_LAZY);
#else
#ifdef __GNUC__
	module_ = lt_dlopen(filename);
#else
#ifdef _MSC_VER
	module_ = LoadLibrary(filename);
#endif
#endif
#endif
	loaded_efs_[libName]++;
}

///////////////////////////////////////////////////////////////////////////
//////                         CONSTRUCTORS                         ///////
///////////////////////////////////////////////////////////////////////////


EnergyFunction::EnergyFunction(std::string const& libName, AtomTypes::ConstPointer atypes, 
			       DataDirectory::ConstPointer directory, DunbrackLibrary::ConstPointer dunLib,
			       RuntimeStats::Pointer rstats)
		: atypes_(atypes), dataDir_(directory), dunLib_(dunLib)
{
	if (efcount_ == 0)
	{
#ifdef __APPLE__
                dlclose(module_);
#else
#ifdef __GNUC__
		lt_dlinit();
		//lt_dlsetsearchpath("../../..");
#endif
#endif
		DeleteLoadedModuleDir();
		CreateLoadedModuleDir();
	}

	if (!atexit_set)
	{
		atexit(UnloadAll);
		atexit_set = true;
	}

	LoadModule(libName);	

	scoreFn_ = (ScoreFn)DLL_ACCESS(module_, "ComputeScore");
	graphFn_ = (GraphFn)DLL_ACCESS(module_, "MakeEnergyGraph");
	initFn_  = (InitFn)DLL_ACCESS(module_, "Initialize");
	pairwiseFn_ = (PWFn)DLL_ACCESS(module_, "ComputePairwiseEnergies");
	onebodyFn_ = (SBFn)DLL_ACCESS(module_, "ComputeOneBodyEnergies");

	if (!module_ || !scoreFn_ || !graphFn_ || !initFn_ || !onebodyFn_ || !pairwiseFn_)
	{
		#ifdef __APPLE__
                        dlclose(module_);
                #else
                #ifdef __GNUC__
			lt_dlclose(module_);
		#else
		#ifdef _MSC_VER
			FreeLibrary(module_);
		#endif
		#endif
                #endif

		std::cout << (void*)(onebodyFn_) << " " << (void*)(pairwiseFn_) << std::endl;
		throw std::string("Failed to load energy function " + libName);
		
	}

	++efcount_;

	Reinitialize(atypes, directory, dunLib, rstats);
}

EnergyFunction::~EnergyFunction()
{
#ifdef __APPLE__
        dlclose(module_); 
#else
#ifdef __GNUC__
/*
	For some reason, the code below causes memory corruption when multiple
	modules are being shutdown. Instead of doing it this way, we postpone
	all module unloading until the application exits, at which point we
	just call lt_dlexit() in the atexit function.

	The memory corruption involves deallocating global blitz arrays defined
	in the energy function modules. Note that the corruption *still happens*
	with this workaround, but since it occurs in the atexit routine that's
	late enough in the game to not cause any problems (I think).

	Nevertheless, this is obviously something that should be investigated
	further. Write a script that creates two energy function objects of the
	same module, run it through valgrind, and look a the 'Invalid write's.
*/
/*
	lt_dlclose(module_);

	if (--efcount_ == 0)
		lt_dlexit();
*/
#else
#ifdef _MSC_VER
	FreeLibrary(module_);
#endif
#endif
#endif
}

///////////////////////////////////////////////////////////////////////////
//////                      STANDARD INTERFACE                      ///////
///////////////////////////////////////////////////////////////////////////

void EnergyFunction::Reinitialize(AtomTypes::ConstPointer atypes, 
	       	  		  DataDirectory::ConstPointer directory, 
	       	  		  DunbrackLibrary::ConstPointer dunLib,
				  RuntimeStats::Pointer rstats)
{
	initFn_(directory, dunLib, atypes, rstats);
}

FLOAT EnergyFunction::ScoreSystem(System::ConstPointer residues, NeighborMap::ConstPointer nmap)
{
 	return scoreFn_(residues, nmap);
}

void EnergyFunction::FillEnergyGraph(RotamerLibrary::ConstPointer rotamers, NeighborMap::ConstPointer nmap, 
				     RotamerIterationMap::ConstPointer rmap, EnergyGraph* graphOut)
{
	graphFn_(rotamers, nmap, rmap, graphOut);
}

blitz::Array<FLOAT, 3> EnergyFunction::GetPairwiseEnergies(System::ConstPointer residues, NeighborMap::ConstPointer nmap)
{
	blitz::Array<FLOAT, 3> result;
	pairwiseFn_(residues, nmap, result);
	return result;
}

blitz::Array<FLOAT, 2> EnergyFunction::GetOneBodyEnergies(System::ConstPointer residues, NeighborMap::ConstPointer nmap)
{
	blitz::Array<FLOAT, 2> result;
	onebodyFn_(residues, nmap, result);
	return result;
}

