// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Contributor(s): Chris Kennelly 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "opt_base.h"
#include <ctime>

typedef boost::variate_generator<boost::mt19937&, boost::uniform_real<> > RealGenerator;
typedef boost::variate_generator<boost::mt19937&, boost::uniform_int<> > IntGenerator;

///////////////////////////////////////////////////////////////////////////
//////                        IMPLEMENTATION                        ///////
///////////////////////////////////////////////////////////////////////////

OptimizationBase::OptimizationBase()
	: mtrand_(42u)
{
	SeedWithTime();
}

void OptimizationBase::InitState(EnergyGraph& graph, std::vector<int> const& initialState)
{
	stateVector_.clear();
	stateIDStart_.clear();
	freeDimList_.clear();
	
	stateVector_.resize(searchSpace_->NumDimensions());
	stateIDStart_.resize(searchSpace_->NumDimensions());

	std::vector<int>::const_iterator initialID = initialState.begin();

	for (int d = 0; d < searchSpace_->NumDimensions(); ++d)
	{
		std::pair<EnergyGraph::vertex_iterator, 
			  EnergyGraph::vertex_iterator> vi = boost::vertices(graph);
	
		std::pair<int, int> IDRange = searchSpace_->DimensionIDRange(d);

		for (; vi.first != vi.second; ++vi.first)
		{
			if ( graph[*vi.first].elementID == IDRange.first ) 
			{
				stateIDStart_[d] = vi.first;
				break;
			}
		}

		int newIDOffset;
		if (initialID != initialState.end())
		{
			if (*initialID < IDRange.first ||
			    *initialID > IDRange.second)
			    throw std::string("Invalid initial element ID.");

			newIDOffset = *initialID - IDRange.first;
			++initialID;
		}
		else
		{
			IntGenerator igen(mtrand_, boost::uniform_int<>(0, IDRange.second - IDRange.first));
			newIDOffset = igen();
		}

		stateVector_[d] = stateIDStart_[d];
		for (int i = 0; i < newIDOffset; ++i)
			++stateVector_[d];

		if (IDRange.second > IDRange.first)
			freeDimList_.push_back(d);
	}

	if (freeDimList_.empty())
		throw std::string("OptimizationBase error: No free dimensions.");
}

void OptimizationBase::Seed( unsigned int seed )
{
	mtrand_.seed( (boost::mt19937::result_type)seed );
}

void OptimizationBase::SeedWithTime()
{
	mtrand_.seed( (unsigned int)time(0) );
}


void OptimizationBase::RecordRuntimeStats(RuntimeStats::Pointer stats) {
	stats_	= stats;
}

