// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Contributor(s): Chris Kennelly 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "opt_base_montecarlo.h"
#include <ctime>

typedef boost::variate_generator<boost::mt19937&, boost::uniform_real<> > RealGenerator;
typedef boost::variate_generator<boost::mt19937&, boost::uniform_int<> > IntGenerator;

///////////////////////////////////////////////////////////////////////////
//////                      PARAMETER ACCESSORS                      //////
///////////////////////////////////////////////////////////////////////////

FLOAT OptimizationBaseMonteCarlo::GetIterationMultiplier() const
{
	return iterMultiplier_;
}

void OptimizationBaseMonteCarlo::SetIterationMultiplier(FLOAT multiplier)
{
	// FIXME: add validation here
	iterMultiplier_ = multiplier;
}

FLOAT OptimizationBaseMonteCarlo::GetTemperature()
{
	return temperature_;
}

void OptimizationBaseMonteCarlo::SetTemperature(FLOAT temperature)
{
	temperature_ = temperature;
}

///////////////////////////////////////////////////////////////////////////
//////                        IMPLEMENTATION                        ///////
///////////////////////////////////////////////////////////////////////////

OptimizationBaseMonteCarlo::OptimizationBaseMonteCarlo()
	: OptimizationBase()
{
	temperature_	= 1.0;
	iterMultiplier_ = 5.0;
}

std::pair<int, int> OptimizationBaseMonteCarlo::PickMove()
{
	IntGenerator dimGen(mtrand_, boost::uniform_int<>(0, (unsigned int)freeDimList_.size() - 1));
	int randomDim = dimGen();
		
	std::pair<int, int> IDRange = searchSpace_->DimensionIDRange( freeDimList_[randomDim] );

	IntGenerator idGen(mtrand_, boost::uniform_int<>(0, IDRange.second - IDRange.first));
	return std::make_pair( freeDimList_[randomDim], idGen() );
}

bool OptimizationBaseMonteCarlo::KeepChange(FLOAT prevEnergy, FLOAT diff, FLOAT temperature)
{
	if (diff < 0.0)
		return true;

	if (temperature < 0.001)
		return false;

	FLOAT lnprob = 0.0;
	FLOAT beta = 1.0/temperature;
	lnprob = beta * diff;
	
	if (lnprob < 10.0)
	{
		FLOAT prob = std::exp(-lnprob);

		RealGenerator pgen(mtrand_, boost::uniform_real<>(0, 1));
		if (pgen() < prob)
			return true;
	}

	return false;
}

