// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Contributor(s): Chris Kennelly 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "opt_bruteforce.h"
#include <ctime>
#include <stdio.h>

///////////////////////////////////////////////////////////////////////////
//////                      PARAMETER ACCESSORS                      //////
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
//////                        IMPLEMENTATION                        ///////
///////////////////////////////////////////////////////////////////////////

BruteForcePacker::BruteForcePacker()
	: OptimizationBase()
{

}

std::pair< std::vector<int>, FLOAT >
	BruteForcePacker::Pack(
                    DiscreteSearchSpace::ConstPointer searchSpace,
                    EnergyGraph* graph, 
		    std::vector<int> const* initialState,
                    EnergyCombList::Pointer energyList, int listSize)
{
        // Check on the energy list - should be in reverse order
        if ((energyList.get() != NULL) && (!(*energyList).isDescOrder()))
            (*energyList).reverse();

	searchSpace_ = searchSpace;
	energies_.clear();
	
	InitState(*graph, initialState? *initialState : std::vector<int>());
	energies_.push_back(graph->ComputeEnergy(stateVector_));

        // Keep the best solution seen so far
	std::vector<EnergyGraph::vertex_iterator> bestState = stateVector_;
	FLOAT bestEnergy = energies_.back();

        // vector for storing temporary states
        std::vector<int> tempVec;

        // Add the first solution to the list if necceary
        if (energyList.get() != NULL) {
            // populate the vector
            foreach(EnergyGraph::vertex_iterator i, stateVector_)
                tempVec.push_back((*graph)[*i].elementID);

            // add the solution to the list
            (*energyList).push(EnergyComb(tempVec, energies_.back()));
        }

	std::vector<int> incrementingVector = std::vector<int>();
	uint totalResidues = freeDimList_.size();

	for (uint numberOfFreeResidues = 0; numberOfFreeResidues < freeDimList_.size(); numberOfFreeResidues++) {
		int rotamerOffset = 0;
		incrementingVector.push_back(rotamerOffset);
	}

	do {
		// Increment rotamer of first residue
		incrementingVector [0]++;

		// Check for "overflows" which require rolling over to the next rotamer of the next residue
		bool bolDone = false;

		for (uint currentResidue = 0; currentResidue < totalResidues; currentResidue++) {
			std::pair<int, int> IDRange = searchSpace_->DimensionIDRange(freeDimList_[currentResidue]);
			int rotamerRange = IDRange.second - IDRange.first;

			if (incrementingVector [currentResidue] >= rotamerRange) {
				if (currentResidue == totalResidues - 1) {
					bolDone = true;
					break;
				}

				incrementingVector [currentResidue] = 0;
				incrementingVector [currentResidue + 1]++;
			}
		}

		if (bolDone) {
			break;
		}

		if (stats_.get() != NULL) {
			stats_->Increment(RSTATS_BRUTEFORCE_ITER, 1);
		}

		std::vector<EnergyGraph::vertex_iterator> tempVector = std::vector<EnergyGraph::vertex_iterator>();

		for (uint residue = 0; residue < freeDimList_.size(); ++residue)
		{
			std::pair<int, int> move = std::make_pair(freeDimList_[residue], incrementingVector [residue]);

			EnergyGraph::vertex_iterator newVertex = stateIDStart_[move.first];
			EnergyGraph::vertex_iterator oldVertex = stateVector_[move.first];

			for (int j = 0; j < move.second; ++j) {
				++newVertex;
			}

			tempVector.push_back(newVertex);
		}

		uint vectorSize	= stateVector_.size();

		FLOAT tempEnergy = graph->ComputeEnergy(tempVector, true);

		if (tempEnergy < bestEnergy)
		{
			energies_.back() = tempEnergy;
		
			for (uint currentVertex = 0; currentVertex < freeDimList_.size(); currentVertex++) {
				EnergyGraph::vertex_iterator newVertex = tempVector [currentVertex];
				EnergyGraph::vertex_iterator oldVertex = stateVector_ [freeDimList_ [currentVertex]];
				
				(*graph)[*oldVertex].active = false;
				(*graph)[*newVertex].active = true;

				stateVector_ [freeDimList_ [currentVertex]] = tempVector [currentVertex];
			}
	
                       	// Add to the list if necessary
                   	if (energyList.get() != NULL) {
				// remove old state
                               	tempVec.clear();

                               	// format the vector
                               	foreach(EnergyGraph::vertex_iterator i, stateVector_) {
                              		tempVec.push_back((*graph)[*i].elementID);
				}

                	        // push the solution
        	                (*energyList).push(EnergyComb(tempVec, energies_.back()));

	                        // pop if we're over the limit
                        	if ((*energyList).size() > listSize) {
        				(*energyList).pop();
				}
        	        }

			// keep the new best energy
			bestEnergy = energies_.back();
			bestState = stateVector_;
		}
	} while (true);

	energies_.push_back( energies_.back() );
       
        // format the best solution for output
	std::vector<int> result;
	foreach(EnergyGraph::vertex_iterator i, bestState) {
		result.push_back((*graph)[*i].elementID);
	}

        // reverse the list if necessary
        if (energyList.get() != NULL) {
            (*energyList).reverse();
	}

	return std::make_pair(result, bestEnergy);
}

