// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "poses.h"

//////////////////////////////////////////////////////////////////////////
/////                          CONSTRUCTORS                          /////
//////////////////////////////////////////////////////////////////////////

PoseLibrary::Pointer PoseLibrary::Create(int nBlocks)
{	
	return PoseLibrary::Pointer( new PoseLibrary(nBlocks) );
}

PoseLibrary::PoseLibrary(int nBlocks)
	: nBlocks_(nBlocks), nPoses_(0), idIndex_(0) 
{
	/// A PoseLibrary is analogous to a RotamerLibrary and exists solely to help index
	/// a set of discrete possible combinations. Here it starts uninitialized.
	/// It consists of a list of pairs of integers denoting first and last nodes
	/// corresponding to one choice

	for (int i = 0; i < nBlocks; ++i)
		idRanges_.push_back( std::pair<int, int>(-1, -1) );
}

//////////////////////////////////////////////////////////////////////////
/////                        PUBLIC INTERFACE                        /////
//////////////////////////////////////////////////////////////////////////

std::pair<int, int> PoseLibrary::GetBlockIDRange(int block) const
{
	/// Lookup the first,last index for a particular block 
	/// if the block is in range, and the indices have been initialized

	if (block < 0 || block >= nBlocks_)
		throw std::string("GetBlockIDRange: Block index out of range.");

	if (idRanges_[block].first < 0 || idRanges_[block].second < 0)
		throw std::string("GetBlockIDRange: Block indices uninitialized.");

	return idRanges_[block];
}

std::pair<int, int> PoseLibrary::GeneratePoseIDs(int block, int nPoses)
{
	std::pair<int, int> result(idIndex_, idIndex_ + nPoses - 1);

	/// A PoseLibrary is analogous to a RotamerLibrary and exists solely to help index
	/// a set of discrete possible combinations. Here real indices are assigned to a block.

	if (idRanges_[block].first >= 0 && idRanges_[block].second >= 0)
		nPoses_ -= ( idRanges_[block].second - idRanges_[block].first + 1 );

	idIndex_ += nPoses;
	nPoses_  += nPoses;
	idRanges_[block] = result;

	return result;
}

//////////////////////////////////////////////////////////////////////////
/////                 DISCRETE SEARCH SPACE INTERFACE                /////
//////////////////////////////////////////////////////////////////////////

int PoseLibrary::NumDimensions() const
{
	return nBlocks_;
}

int PoseLibrary::NumElements() const
{
	return nPoses_;
}

std::pair<int, int> PoseLibrary::DimensionIDRange(int dimension) const
{
	return idRanges_[dimension];
}


