// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "rotamer_iteration_map.h"

RotamerIterationMap::Pointer RotamerIterationMap::Create(RotamerLibrary::ConstPointer rlib)
{
	return RotamerIterationMap::Pointer(new RotamerIterationMap(rlib));
}

FLOAT RotamerIterationMap::CountPair(Residue const& res1, Residue const& res2) const
{
	System::ConstPointer s1 = res1.GetSystem(), 
			     s2 = res2.GetSystem();

	return pairData_(s1, s2, s1->GetStorageIndex(res1), 
				 s2->GetStorageIndex(res2));
}

FLOAT RotamerIterationMap::CountSingle(Residue const& res) const
{
	System::ConstPointer s = res.GetSystem();

	return singleData_(s, s->GetStorageIndex(res));
}

// Initialize to 1.0
void RotamerIterationMap::EnableAll()
{
	std::vector<System::ConstPointer> const& systems = rlib_->GetSystems();
	std::vector<System::ConstPointer>::const_iterator sys1, sys2;

	for (sys1 = systems.begin(); sys1 != systems.end(); ++sys1)
	{
		blitz::Array<FLOAT, 1>& singleData = singleData_(*sys1);
		singleData = 1.0;
	
		for (sys2 = sys1; sys2 != systems.end(); ++sys2)
		{
			blitz::Array<FLOAT, 2>& pairData = pairData_(*sys1, *sys2);
			pairData = 1.0;
		}
	}
}

// Set to 0
void RotamerIterationMap::DisableAll()
{
	std::vector<System::ConstPointer> const& systems = rlib_->GetSystems();
	std::vector<System::ConstPointer>::const_iterator sys1, sys2;

	for (sys1 = systems.begin(); sys1 != systems.end(); ++sys1)
	{
		blitz::Array<FLOAT, 1>& singleData = singleData_(*sys1);
		singleData = 0;
	
		for (sys2 = sys1; sys2 != systems.end(); ++sys2)
		{
			blitz::Array<FLOAT, 2>& pairData = pairData_(*sys1, *sys2);
			pairData = 0;
		}
	}
}

void RotamerIterationMap::SetSystemPairWeight(System::ConstPointer sys1, System::ConstPointer sys2, FLOAT weight)
{
	pairData_(sys1, sys2) = weight;
}

void RotamerIterationMap::SetResiduePairWeight(Residue const& res1, Residue const& res2, FLOAT weight)
{
	System::ConstPointer s1 = res1.GetSystem(), 
			     s2 = res2.GetSystem();

	pairData_(s1, s2, s1->GetStorageIndex(res1), 
		          s2->GetStorageIndex(res2)) = weight;
}

void RotamerIterationMap::SetSystemSingleWeight(System::ConstPointer sys, FLOAT weight)
{
	singleData_(sys) = weight;
}

void RotamerIterationMap::SetResidueSingleWeight(Residue const& res, FLOAT weight)
{
	System::ConstPointer s = res.GetSystem();

	singleData_(s, s->GetStorageIndex(res)) = weight;
}

RotamerLibrary::ConstPointer RotamerIterationMap::GetRotamerLibrary() const
{
	return rlib_;
}


