# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from CHOMP import *
import sys,os,random,time


def SPR(originalcombo,s,eg,rl,SPRrounds=1,v=0,randomize=True,cutoff=0):
    combo = originalcombo[:]
    comboE = eg.ComputeEnergy(IntVector(combo))
    if v>0: print 'Beginning SPR from:',combo,comboE

    numbatches,numaccepted=0,0
    resids = [res.ID for res in s]

    for SPRround in range(SPRrounds):
        if v>0: print 'SPR Round',SPRround
        nodes = eg.GetVertexIDs()
        if randomize: random.shuffle(nodes)

        # Fix one rotamer (node) at a time    
        for n in nodes:
            resn = rl.GetRotamerResidue(n).ID  # The residue number in the system

            if v>1:  #More verbose
                if len(set(combo).difference(originalcombo))>0:
                    print 'Proceeding with altered combination:',len(set(combo).difference(originalcombo))
            
            perturb = combo[:]  # Make a copy of the current combination to perturb by fixing a rotamer
            perturbindex = resids.index(resn) # Find the position in the combo to replace
            perturb[perturbindex] = n  # Make the perturbation
            if v>0: perturbE = eg.ComputeEnergy(IntVector(perturb))  # For display purposes

            if cutoff>0:  # Only relax neighboring residues which have at least one rotamer where the interaction magnitude exceeds the cutoff
                edges = [(rl.GetRotamerResidue(x.first).ID,x.second) for x in eg.GetVertexEdges(n)]
                edges = sorted(edges, key=lambda x:(abs(x[1])))
                edges.reverse()
                if v>2: print 'edges:',edges
                
                neighbors = sorted(list(set([x[0] for x in edges])))
                if v>1: print 'neighbors:',neighbors

                partners = []
                for nres,nE in edges:
                    if abs(nE)>cutoff and nres not in partners: partners.append(nres)
                    if nE < cutoff: break
                partners.sort()
                if v>1: print 'partners:',partners
            else:
                #which unique residues interact with the current rotamer?
                partners = [x.first for x in eg.GetVertexEdges(n)]
                partners = list(set([rl.GetRotamerResidue(x).ID for x in partners])) 

            if randomize: random.shuffle(partners)
            if v>0: print '\nWorking on',resn,'rot',n,'Interaction partners:',partners
            
            batchmove = perturb[:]   # A copy to store batch changes
            if v>0: old,new,sres,sdiff = [],[],[],[]

            # Walk through a list of interacting partner residues
            for partner in partners:
                trialcombo = perturb[:]    # A working copy of the combination, the partners can see the perturbation but not pending relaxation moves
                #trialcombo = batchmove[:]  # A working copy of the combination, this is iterative relaxation not batch relaxation

                # The alternatives for this partner
                rotIDs = range(rl.GetRotamerIDRange(s.GetResidue(partner)).first,rl.GetRotamerIDRange(s.GetResidue(partner)).second+1)
                replaceindex = resids.index(partner)  ## Find the position in the combo to replace
                pickedbest,bestE = False,0
                if v>0: oldrotID = perturb[replaceindex]
                    
                ## Go through rotamers for this interaction partner and find the best alternative
                oldrotID = trialcombo[replaceindex]
                for rotID in rotIDs:
                    trialcombo[replaceindex] = rotID
                    # It would be nifty if we could directly call eg.DiffEnergy from python, but I don't think DiffEnergy is nicely exposed
                    trialE = eg.ComputeEnergy(IntVector(trialcombo))
                    if not pickedbest or trialE < bestE:  # First rotamer or best rotamer
                        #if v>0: print 'Improvement: res',rl.GetRotamerResidue(rotID).ID,'rot',oldrotID,'to',rotID,trialE,'vs',bestE
                        pickedbest = True
                        bestE = trialE
                        batchmove[replaceindex] = rotID

                ## If the partner chooses a new rotamer, add to a list for display purposes
                if v>0 and batchmove[replaceindex]!=oldrotID:
                    old.append(oldrotID)
                    new.append(batchmove[replaceindex])
                    sres.append(partner)
                    sdiff.append(trialE-perturbE)

            ## Compute the energy of making the batch change        
            batchE = eg.ComputeEnergy(IntVector(batchmove))
            if v>0: 
                print 'Perturb res %4d to rot %4d: E = %9.2f => %9.2f (best E = %9.2f) relaxing %s' % (resn,n,perturbE,
                                                                    batchE,comboE,[rl.GetRotamerResidue(x).ID for x in old])
                for switch in range(len(old)):
                    print 'Switching res %4d from rotamer %4d to %4d: diffE = %9.2f' % (sres[switch],old[switch],new[switch],sdiff[switch])


            numbatches += 1
            if batchE < comboE:
                numaccepted += 1
                if v>1: print 'Accepting batch move',comboE,' => ',batchE
                #if v>0: print 'Switching',old,' => ',new
                combo = batchmove[:]
                comboE = batchE

    if v>0: print '% accepted perturbations:',100*(float(numaccepted)/float(numbatches)),numaccepted,numbatches


if __name__=='__main__':
    try:
        inputpdb = sys.argv[1]
    except:
        print 'Provide pdb'
        sys.exit()

    # load global data
    atomtypes = AtomTypes()
    amino_dir = AminoAcidDirectory(atomtypes)
    data_dir = DataDirectory(os.environ['CHOMPDATA'])
    dunbrack_lib = DunbrackLibrary(amino_dir,os.environ['DUNBRACK'])
    EnergyFunction.SetPluginPath(os.environ['CHOMPDIST'])
    energy_function = EnergyFunction('libCHOMP_ef_rosetta.so', atomtypes, data_dir, dunbrack_lib)
 
    # load a system and score it
    S = System(atomtypes, amino_dir)
    S.Parse(open(inputpdb).read())

    print 'generating rotamers...'
    n = NeighborMap(S, data_dir.PairCutoffTable)
    rl = RotamerLibrary(S)
    rl2 = RotamerLibrary(S)
    rg = RotamerGenerator(n, dunbrack_lib)
    rl2.GenerateRotamers(rg)
    ri = RotamerIterationMap(rl)
    original_rotamers = rl2.TranslateRotamerIDs(rl, IntVector(range(S.size)))

    print 'filling energy graph'
    eg = EnergyGraph()
    energy_function.FillEnergyGraph(rl2, n, ri, eg)

    ## random starting combination
    random.seed(0)
    combo = [random.choice(range(rl2.GetRotamerIDRange(res).first,rl2.GetRotamerIDRange(res).second+1)) for res in S]
    comboE = eg.ComputeEnergy(IntVector(combo))
    print combo

    ## start with brief MC
    #mc = MonteCarloPacker()
    #mc.SetIterationMultiplier(1)
    #soln = mc.Pack(rl2, eg, IntVector())
    #print list(soln.first),soln.second
    #combo = soln.first
    #comboE = soln.second
    before = time.time()
    SPR(combo,S,eg,rl2,v=1,cutoff=0.2,SPRrounds=2,randomize=False)
    print time.time()-before,'seconds'


