# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from CHOMP import *
import sys,os

try:
    inputpdb = sys.argv[1]
except:
    print 'Provide pdb'
    sys.exit()

# load global data
atomtypes = AtomTypes()
amino_dir = AminoAcidDirectory(atomtypes)
data_dir = DataDirectory(os.environ['CHOMPDATA'])
dunbrack_lib = DunbrackLibrary(amino_dir,os.environ['DUNBRACK'])
EnergyFunction.SetPluginPath(os.environ['CHOMPDIST'])
energy_function = EnergyFunction('libCHOMP_ef_rosetta.so', atomtypes, data_dir, dunbrack_lib)
#energy_function = EnergyFunction(os.environ['CHOMP_EF'], atomtypes, data_dir, dunbrack_lib)
 
# load a system and score it
S = System(atomtypes, amino_dir)
S.Parse(open(inputpdb).read())

print 'generating rotamers...'
n = NeighborMap(S, data_dir.PairCutoffTable)
rl = RotamerLibrary(S)
rg = RotamerGenerator(n, dunbrack_lib)
rl.GenerateRotamers(rg)
ri = RotamerIterationMap(rl)


print 'filling energy graph'
eg = EnergyGraph()
energy_function.FillEnergyGraph(rl, n, ri, eg)
sa = SimulatedAnnealingPacker()

print 'repacking'
soln = sa.Pack(rl, eg)
print 'E =', soln.second
print 'writing output'
newsys = rl.CreateSystem(S, soln.first)
open('repacked.%s'%inputpdb, 'w').write(str(newsys))


