# SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
# (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
# furnished to do so, subject to the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
# THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
# 

from CHOMP import *
import sys,os
import random

try:
    mypdb = sys.argv[1]
except:
    print 'Provide pdb'
    sys.exit()
 
def REPACK(input,label='',discardinputrot=False,initialstate=IntVector(),iter=5,verbose=False):
    """Given a system this function runs rotamer optimization"""
    rotlib = RotamerLibrary(input)
    neighbormap = NeighborMap(input, data_dir.PairCutoffTable)
    print 'Before repack:', energy_function.ScoreSystem(input, neighbormap)
    rotgen = RotamerGenerator(neighbormap, dunbrack_lib)
    rotlib.GenerateRotamers(rotgen, discardinputrot) ## Discard rotamers from input pdbs
    rotiter = RotamerIterationMap(rotlib)
    energy_graph = EnergyGraph()
    energy_function.FillEnergyGraph(rotlib, neighbormap, rotiter, energy_graph)
    sim_anneal.SetIterationMultiplier(iter)
    pack_result = sim_anneal.Pack(rotlib, energy_graph, initialstate)
    solution = pack_result.first
    packed = rotlib.CreateSystem(input, solution)
    afterE = energy_function.ScoreSystem(packed,NeighborMap(packed,data_dir.PairCutoffTable))
    if verbose: print 'Repacked',label,'E =',afterE
    return packed,solution
 
# load global data
atom_types = AtomTypes()
amino_dir = AminoAcidDirectory(atom_types)
data_dir = DataDirectory(os.environ['CHOMPDATA'])
dunbrack_lib = DunbrackLibrary(amino_dir,os.environ['DUNBRACK'])
EnergyFunction.SetPluginPath(os.environ['CHOMPDIST'])
energy_function = EnergyFunction('libCHOMP_ef_rosetta.so', atom_types, data_dir, dunbrack_lib)
 
# load a system
S = System(atom_types, amino_dir)
S.Parse(open(mypdb).read())
n = NeighborMap(S, data_dir.PairCutoffTable)
#print 'Original score:', energy_function.ScoreSystem(S, n)
sim_anneal = SimulatedAnnealingPacker()
repackedWT,rotcombo = REPACK(S,label='WT',iter=10,verbose=True)

palette = ['ALA','CYS','ASP','GLU','PHE','GLY','HIS','ILE','LYS','LEU','MET','ASN','GLN','ARG','SER','THR','VAL','TRP','TYR']

residues = [r.ID for r in S]
print 'Residues',residues
mutant = repackedWT.Clone
for ires,res in enumerate(residues):
    mutation = random.choice(palette)
    name = '%s%d to %s' % (S[ires].AminoAcid.name3Char,res,mutation)
    print name
    mutant.MutateResidue(res,amino_dir[mutation])
    #REPACK(mutant,label=name,iter=1,initialstate=IntVector(rotcombo),verbose=True)
packed,soln = REPACK(mutant,label='many mutations',iter=1,verbose=True)
open('redesigned.pdb','w').write(str(packed))



