// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_ATOM_PAIRWISE_H
#define IVL_ATOM_PAIRWISE_H

#include <blitz/array.h>

#include "general.h"
#include "system.h"
#include "rotamers.h"
#include "rotamer_trie.h"
#include "neighbor_map.h"

void Atom_Pairwise_Initialize();

bool Atom_Pairwise_TrieTrie(RotamerLibrary::ConstPointer rotlib,
			    RotamerSet const & trie1, 
		    	    RotamerSet const & trie2,
			    blitz::Array<FLOAT, 2>& output,
			    bool bbOnly);

FLOAT Atom_Pairwise_ResidueResidue(System::ConstPointer system,
				   Residue const& residue1, 
			       	   Residue const& residue2,
				   bool backboneOnly,
				   blitz::Array<FLOAT, 1>& energiesOut);

bool Atom_Pairwise_IntraRes_Trie(RotamerLibrary::ConstPointer rotlib,
				 RotamerSet const & trie, 
				 blitz::Array<FLOAT, 1>& output);

FLOAT Atom_Pairwise_IntraRes_Residue(System::ConstPointer system,
				     Residue const& res,
				     blitz::Array<FLOAT, 1>& energiesOut);

#endif


