// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#define DIST_BINS_PER_A2 20.0  ///< The number of bins in the data tables per angstrom squared of distance between atoms

/** \brief Looks up an interpolated value from a data table indexed by [Distance^2, Atom Type, Atom Type].
 *
 * \param dist2 The distance squared between two atoms
 * \param type1 The type of the first atom (in whatever type system table is using)
 * \param type2 The type of the second atom (in whatever type system table is using)
 * \param table The 3D data table to look up the value in
 *
 * \return FLOAT An interpolated value from table
 */
inline FLOAT Atom_DistType2_Term(int bin1, FLOAT frac, int type1, int type2, blitz::Array<FLOAT, 3> const& table)
{
//	return lerp(dist2, DIST_BINS_PER_A2, 0.0, table(type1, type2, blitz::Range::all()));

	const FLOAT* bin1_ptr = &table(type1, type2, bin1);

	if (bin1 >= table.extent(2) - 1)
		return *bin1_ptr;

	const FLOAT bin1_val = *bin1_ptr;
	const FLOAT bin2_val = bin1_ptr[1];

        return bin1_val + frac * ( bin2_val - bin1_val );
}

inline void Atom_Dist2Interp(FLOAT dist2, int& bin1, FLOAT& frac)
{
	const FLOAT f_bin1 = (dist2 - 0.0) * DIST_BINS_PER_A2;
	bin1    = static_cast<int>(f_bin1);
	frac   = f_bin1 - static_cast<float>(bin1);
}

inline FLOAT Atom_FloatInt_Term(FLOAT val1, uint val2, blitz::Array<FLOAT, 2> const& table,
				FLOAT binsPerVar, FLOAT minVar)
{
	return CHOMPMath::LinearInterpolate(val1, binsPerVar, minVar, table(val2, blitz::Range::all()));
}


