// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_ATOM_ENERGY_TERMS_H
#define IVL_ATOM_ENERGY_TERMS_H

#define N_PAIRWISE_ENERGY_TERMS 8
#define N_ONEBODY_ENERGY_TERMS  4

#include "general.h"

enum PairwiseEnergyTerm
{
	// pairwise atomic terms
	PTERM_AP_ATR,			// vdw attractive	
	PTERM_AP_REP,			// vdw repulsive
	PTERM_AP_SOLV,			// solvation

	// hydrogen bonding terms
	PTERM_HB_SC_SC,			// hbond sidechain-sidechain
	PTERM_HB_SC_BB,			// hbond sidechain-backbone
	PTERM_HB_BB_BB_SR,		// hbond short-range bb-bb
	PTERM_HB_BB_BB_LR,		// hbond long-range bb-bb

	// rotamer pairwise terms
	PTERM_RP_STAT			// statistical rotamer pairwise term
};

enum OneBodyEnergyTerm
{
	// fullatom terms
	STERM_AP_REP,			// inter-residue vdw repulsive

	// statistical terms
	STERM_DUNBRACK,			// bonus for solns. close to dunbrack rotamers
	STERM_PAAPPE,			// Paa/ppE
	STERM_STRAIN			// 'backbone-dependent strain'
};

#endif

