// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

//////////////////////////////////////////////////////////////////////////
/////                           LOCAL DATA                           /////
//////////////////////////////////////////////////////////////////////////

static bool scInterpolate = false;

//////////////////////////////////////////////////////////////////////////
/////                           PARAMETERS                           /////
//////////////////////////////////////////////////////////////////////////

extern int   parm_HBond_srCutoff;

extern FLOAT parm_HBond_scInterpMin;
extern FLOAT parm_HBond_scInterpMax;
extern FLOAT parm_HBond_RInterpEdge;
extern FLOAT parm_HBond_AngInterpEdge;

extern FLOAT parm_HBond_RMax;
extern FLOAT parm_HBond_RMin;
extern FLOAT parm_HBond_xDMin;
extern FLOAT parm_HBond_xDMax;
extern FLOAT parm_HBond_xHMin;
extern FLOAT parm_HBond_xHMax;

extern FLOAT parm_HB_SC_SC_Weight;
extern FLOAT parm_HB_SC_BB_Weight;
extern FLOAT parm_HB_Long_BB_Weight;
extern FLOAT parm_HB_Short_BB_Weight;

extern unsigned long long atomPairHBCount;

#include "hbond_helpers.h"

//////////////////////////////////////////////////////////////////////////
/////                           INTERFACE                            /////
//////////////////////////////////////////////////////////////////////////

template < void AddTerm(FLOAT, FLOAT*, FLOAT*, PairwiseEnergyTerm) >
inline void HBondTerm(FLOAT* total, FLOAT* output)
{
	int resDist = GetResidueSequenceDist();
	int hbType  = GetHBondType(resDist);

	if (resDist <= 2 && hbType != HBOND_SC_SC)
		return;

	HBondComponents params, energies;
	int polyType;

	FLOAT hbWeight, hb_total;

	if (hbType == HBOND_SC_BB)
	{
		if (Hydrogen_IsBackbone() && ResidueIsAlreadyDonor())
			goto hb_end;
		if (Acceptor_IsBackbone() && ResidueIsAlreadyAcceptor())
			goto hb_end;
	}

	atomPairHBCount++;

	params = GetHBondParameters();

	if (params.r   < parm_HBond_RMin  || params.r   > parm_HBond_RMax  ||
	    params.xD  < parm_HBond_xDMin || params.xD  > parm_HBond_xDMax ||
	    params.xH  < parm_HBond_xHMin || params.xH  > parm_HBond_xHMax)  return;

	polyType = GetHBondPolynomial(hbType, resDist, params.r);

	energies.r  = Atom_FloatInt_Term(params.r,  polyType, directory_->GetHB_AHTable(),
					 1000.0/(parm_HBond_RMax - parm_HBond_RMin), parm_HBond_RMin);
	energies.xD = Atom_FloatInt_Term(params.xD, polyType, directory_->GetHB_XDTable(),
		    			 1000.0/(parm_HBond_xDMax - parm_HBond_xDMin), parm_HBond_xDMin);
	energies.xH = Atom_FloatInt_Term(params.xH, polyType, directory_->GetHB_XHTable(),
		    			 1000.0/(parm_HBond_xHMax - parm_HBond_xHMin), parm_HBond_xHMin);

	if (scInterpolate)
		InterpolateSC(polyType, params, &energies);
	else if (params.r > parm_HBond_RInterpEdge)
		FadeOut(params.r, parm_HBond_RMax, hbType < 2 ? parm_HBond_scInterpMax : parm_HBond_RInterpEdge, &energies.xD, &energies.xH);
	if (params.xD < parm_HBond_AngInterpEdge)
		FadeOut(params.xD, parm_HBond_xDMin, parm_HBond_AngInterpEdge, &energies.r, &energies.xH);
	if (params.xH < parm_HBond_AngInterpEdge)
		FadeOut(params.xH, parm_HBond_xHMin, parm_HBond_AngInterpEdge, &energies.r, &energies.xD);
		
	hbWeight = 1.0;
	if (hbType == HBOND_SC_SC || hbType == HBOND_SC_BB)
		hbWeight = get_hb_env_weight_lin(nmap_->NumNeighbors(*_hres), nmap_->NumNeighbors(*_ares));

	hb_total = (energies.r + energies.xD + energies.xH) * hbWeight;

	if (hb_total <= 0.0)
	{
		switch (hbType)
		{
			case HBOND_SC_SC: 
				hb_total *= parm_HB_SC_SC_Weight; 
				AddTerm(hb_total, total, output, PTERM_HB_SC_SC);
				break;
			case HBOND_SC_BB: 
				hb_total *= parm_HB_SC_BB_Weight; 
				AddTerm(hb_total, total, output, PTERM_HB_SC_BB);
				break;
			case HBOND_BB_BB_SR: 
				hb_total *= parm_HB_Short_BB_Weight; 
				AddTerm(hb_total, total, output, PTERM_HB_BB_BB_SR);
				break;
			case HBOND_BB_BB_LR: 
				hb_total *= parm_HB_Long_BB_Weight; 
				AddTerm(hb_total, total, output, PTERM_HB_BB_BB_LR);
				break;
		}
		
		if (hbType == HBOND_BB_BB_SR || hbType == HBOND_BB_BB_LR)
			RecordBackboneHBond();
	}

	hb_end: 
		return;
}


