// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_PARAMETERS_H
#define IVL_PARAMETERS_H

#include "general.h"

//////////////////////////////////////////////////////////////////////////
/////                        DISTANCE CUTOFFS                        /////
//////////////////////////////////////////////////////////////////////////

// NOTE: changing this type of parameter without changing the corresponding data tables
// may cause weird errors, if you end up accessing memory past the bounds of your table!
// These may be difficult to detect and may only show up as subtly "wrong" results, so 
// be careful.

FLOAT parm_HCutoff = 4.6 * 4.6;	///< minimum distance between heavy atoms for their bonded hydrogens to interact
FLOAT parm_ACutoff = 5.5 * 5.5 - 0.0001; ///< minimum distance between any two atoms for them to interact

int   parm_HBond_srCutoff = 4;

FLOAT parm_HBond_scInterpMin   = 1.9;
FLOAT parm_HBond_scInterpMax   = 2.3;
FLOAT parm_HBond_RInterpEdge   = 2.1;
FLOAT parm_HBond_AngInterpEdge = .05;

FLOAT parm_HBond_RMin		= 1.4;
FLOAT parm_HBond_RMax		= 3.0;
FLOAT parm_HBond_xDMin		= 0.0;
FLOAT parm_HBond_xDMax		= 1.0;
FLOAT parm_HBond_xHMin		= 0.0;
FLOAT parm_HBond_xHMax		= 1.0;

int   parm_RPair_MinSeqSep	= 1;
int   parm_RPair_MaxBin1	= 2;

FLOAT parm_RPair_MinCenDist	= 3.00;
FLOAT parm_RPair_BinsPerCenDist = 0.6667;

//////////////////////////////////////////////////////////////////////////
/////                             WEIGHTS                            /////
//////////////////////////////////////////////////////////////////////////

FLOAT parm_LJAttrWeight = 0.8; ///< Energy function parameter -- Lennard Jones attractive potential weighting factor
FLOAT parm_LJRepWeight = 0.44; ///< Energy function parameter -- Lennard Jones repulsive potential weighting factor
FLOAT parm_SolvWeight  = 0.65; ///< Energy function parameter -- Solvation energy weighting factor
FLOAT parm_LJRepWeight_IntraRes = 0.004;

FLOAT parm_HB_SC_SC_Weight = 1.1;
FLOAT parm_HB_SC_BB_Weight = 1.17;
FLOAT parm_HB_Long_BB_Weight = 1.17;
FLOAT parm_HB_Short_BB_Weight = 1.17;

FLOAT parm_DunbrackWeight = 0.56;
FLOAT parm_BBStrainWeight = -1.0;
FLOAT parm_Paa_ppE_Weight = 0.64;
FLOAT parm_RPairWeight	  = 0.49;

//////////////////////////////////////////////////////////////////////////
/////                              MISC.                             /////
//////////////////////////////////////////////////////////////////////////

int   parm_RPair_BurialThresh	= 16;

#endif


