// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

//////////////////////////////////////////////////////////////////////////
/////                           PARAMETERS                           /////
//////////////////////////////////////////////////////////////////////////

extern FLOAT parm_DunbrackWeight;
extern FLOAT parm_BBStrainWeight;
extern FLOAT parm_Paa_ppE_Weight;

//////////////////////////////////////////////////////////////////////////
/////                        PUBLIC INTERFACE                        /////
//////////////////////////////////////////////////////////////////////////

using namespace CHOMPMath;

template < void AddTerm(FLOAT, FLOAT*, FLOAT*, OneBodyEnergyTerm) >
inline void OneBodyTerm_PerRotamer(FLOAT* total, FLOAT* output)
{
	FLOAT phi = _res->GetPhiPsi()(0);
	FLOAT psi = _res->GetPhiPsi()(1);

	bool firstRes = _res->GetSystem()->IsFirstInChain(*_res);
	bool lastRes  = _res->GetSystem()->IsLastInChain(*_res);

	DunbrackLibrary::AngleBinData phipsi = dunLib_->GetInterpData(phi, psi, firstRes, lastRes);
	int nRot = dunLib_->GetBaseRotamer((int)_res->GetAminoAcid().aminoAcid, *_chi);

	Vector4 baseChi = dunLib_->GetInterpChi(_res->GetAminoAcid().aminoAcid, phipsi, nRot);
	Vector4 baseSD  = dunLib_->GetInterpSD(_res->GetAminoAcid().aminoAcid, phipsi, nRot);
	FLOAT   lnpbase = std::log( dunLib_->GetInterpPercent(_res->GetAminoAcid().aminoAcid, phipsi, nRot) );

	// TODO: Factor out chain-break phi,psi into constants
	if ( !(phi == -90.000 || psi == 130.000) ) // check for chainbreaks
		for (int i = 0; i < 4; ++i)
		{
			// ignore rotamers that actually aren't in the Dunbrack library,
			// such as the additional proton rotamers
			if (baseSD(i) == 0.0)
				continue;

			FLOAT chiDiff = AngleSub((*_chi)(i), baseChi(i));
		
			FLOAT SD2 = baseSD(i) * baseSD(i);
			FLOAT lnp = -((chiDiff * chiDiff)/(2.0 * SD2));

			lnpbase += lnp;
		}
	
	AddTerm(-lnpbase * parm_DunbrackWeight, total, output, STERM_DUNBRACK); 
}

template < void AddTerm(FLOAT, FLOAT*, FLOAT*, OneBodyEnergyTerm) >
inline void OneBodyTerm_PerResidue(FLOAT* total, FLOAT* output)
{
	int aa = (int)_res->GetAminoAcid().aminoAcid;

	FLOAT phi = _res->GetPhiPsi()(0);
	FLOAT psi = _res->GetPhiPsi()(1);

	FLOAT strain = directory_->GetBBStrainWeight()(aa);
	
	FLOAT Paa_ppE; 
	if ( phi == -90.000 || psi == 130.000 ) // chainbreak
		Paa_ppE = 0.0;
	else
	{
		FLOAT const phi_t = phi + 180.0;
		FLOAT const psi_t = psi + 180.0;

		FLOAT interp_p, dummy;
	
		int phi1, phi2, psi1, psi2;
		FLOAT phi_frac, psi_frac;

		GetAngleInterpBins(phi_t, 36, &phi1, &phi2, &phi_frac);
		GetAngleInterpBins(psi_t, 36, &psi1, &psi2, &psi_frac);

		InterpBilinear( directory_->GetAAProbabilityByAngle()(phi1, psi1, aa),
				directory_->GetAAProbabilityByAngle()(phi2, psi1, aa),
 				directory_->GetAAProbabilityByAngle()(phi1, psi2, aa),
 				directory_->GetAAProbabilityByAngle()(phi2, psi2, aa),
				phi_frac, psi_frac, 36, &interp_p, &dummy, &dummy );
	
		Paa_ppE = -std::log( interp_p / directory_->GetAAProbability()(aa) );
	}

	AddTerm(Paa_ppE * parm_Paa_ppE_Weight, total, output, STERM_PAAPPE);
	AddTerm(strain * parm_BBStrainWeight, total, output, STERM_STRAIN);
}


