// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

//////////////////////////////////////////////////////////////////////////
/////                           PARAMETERS                           /////
//////////////////////////////////////////////////////////////////////////

extern int   parm_RPair_MinSeqSep;
extern int   parm_RPair_BurialThresh;
extern int   parm_RPair_MaxBin1;

extern FLOAT parm_RPair_MinCenDist;
extern FLOAT parm_RPair_BinsPerCenDist;

extern FLOAT parm_RPairWeight;

//////////////////////////////////////////////////////////////////////////
/////                        PUBLIC INTERFACE                        /////
//////////////////////////////////////////////////////////////////////////

// TODO: overall, interpolation happens a bunch of different random ways
// in this energy function. It would be good to factor all that out into
// one central set of functions...

using namespace CHOMPMath;

template < void AddTerm(FLOAT, FLOAT*, FLOAT*, PairwiseEnergyTerm) >
inline void Rotamer_PairwiseTerm(FLOAT* total, FLOAT* output)
{
	const AminoAcidIndex aa1 = _res1->GetAminoAcid().aminoAcid;
	const AminoAcidIndex aa2 = _res2->GetAminoAcid().aminoAcid;

	// only compute this for polar pairs
	if (!_res1->GetAminoAcid().polar || !_res2->GetAminoAcid().polar)
		return;

	// ignore pairs that are too close
	if ( std::abs((int)_res1->GetID() - (int)_res2->GetID()) < parm_RPair_MinSeqSep )
		return;

	int burialState1 = nmap_->NumNeighbors(*_res1) > parm_RPair_BurialThresh ? 1 : 0;
	int burialState2 = nmap_->NumNeighbors(*_res2) > parm_RPair_BurialThresh ? 1 : 0;

	Vector3 dCen = *_centroid1 - *_centroid2;
	FLOAT cenDist = std::sqrt( blitz::dot(dCen, dCen) );

	FLOAT frac;
	int bin1 = LinearInterpolateBinCentered(cenDist, parm_RPair_BinsPerCenDist, parm_RPair_MinCenDist, &frac);

	if (bin1 > parm_RPair_MaxBin1)
		return;
	
	FLOAT lower = (  directory_->GetPairLikelihoodTable()((int)aa1, (int)aa2, burialState1, burialState2, bin1)
		       + directory_->GetPairLikelihoodTable()((int)aa2, (int)aa1, burialState2, burialState1, bin1) ) * 0.5;

	FLOAT upper;
	if (bin1 == parm_RPair_MaxBin1)
		upper = 1.0;
	else if (bin1 == 0 && frac == 0.0) 
		upper = lower;
	else
		upper = (  directory_->GetPairLikelihoodTable()((int)aa1, (int)aa2, burialState1, burialState2, bin1+1)
		         + directory_->GetPairLikelihoodTable()((int)aa2, (int)aa1, burialState2, burialState1, bin1+1) ) * 0.5;

	FLOAT term = lower + frac * (upper - lower);

	// energy is -log(likelihood ratio)
	AddTerm(-std::log( term ) * parm_RPairWeight, total, output, PTERM_RP_STAT);
}


