// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#include "stdafx.h"
#include "general.h"
#include "residue.h"
#include "data_table.h"
#include "rotamer_trie.h"
#include "dunbrack.h"
#include "system.h"

#include <blitz/tinyvec-et.h>

#include "applicators.h"
#include "energy_terms.h"
#include "rotamer_onebody.h"

//////////////////////////////////////////////////////////////////////////
/////                         LOCAL VARIABLES                        /////
//////////////////////////////////////////////////////////////////////////

// (these are global to make it easier to use functions in rotamer_pairwise_util.h without
// passing in a ton of parameters, cluttering up the user-defined energy function code)

Residue const* _res;

Vector4 const* _chi;
Vector3 const* _centroid;

//////////////////////////////////////////////////////////////////////////
/////                          EXTERNAL DATA                         /////
//////////////////////////////////////////////////////////////////////////

extern DunbrackLibrary::ConstPointer dunLib_;
extern DataDirectory::ConstPointer directory_;
extern AtomTypes::ConstPointer atypes_;

//////////////////////////////////////////////////////////////////////////
/////                            INCLUDES                            /////
//////////////////////////////////////////////////////////////////////////

#include "rotamer_onebody_util.h"
#include "rotamer_onebody_ef.h"

//////////////////////////////////////////////////////////////////////////
/////                        PUBLIC INTERFACE                        /////
//////////////////////////////////////////////////////////////////////////

bool Rotamer_OneBody_Trie(RotamerLibrary::ConstPointer rotlib,
			  RotamerSet const& trie, 
			  blitz::Array<FLOAT, 1>& output)
{
	_res = &trie.GetBaseResidue();

	FLOAT res_term = 0.0;
	OneBodyTerm_PerResidue<CombinedEnergyTerms>(&res_term, 0);

	for (int rot = 0; rot < trie.NumRotamers(); ++rot)
	{
		_chi = &trie.GetChi(rot);
		_centroid = &trie.GetCentroid(rot);
		 
		FLOAT rot_term = 0.0;
		OneBodyTerm_PerRotamer<CombinedEnergyTerms>(&rot_term, 0);
		output(rot) = res_term + rot_term;
	}

	return true;
}

FLOAT Rotamer_OneBody_Residue(System::ConstPointer system,
			      Residue const& residue, 
			      blitz::Array<FLOAT, 1>& energiesOut)
{
	FLOAT total = 0.0;
	FLOAT* output = &(energiesOut(0));

	_res = &residue;
	_chi = &residue.GetChi();
	_centroid = &residue.GetCentroid();
	
	OneBodyTerm_PerResidue<SeparateEnergyTerms>(&total, output);
	OneBodyTerm_PerRotamer<SeparateEnergyTerms>(&total, output);
	return total;
}



