// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Contributor(s): Albert Ng 
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_HBOND_UTIL
#define IVL_HBOND_UTIL

inline bool FillHBondVars()
{
	if (_flags1 & ATM_FLAG_POL_H && _flags2 & ATM_FLAG_HB_ACCEPTOR)
	{
		_hydrogen = _atom1;
		_acceptor = _atom2;
		_hres = _res1;
		_ares = _res2;
		_hflags = _flags1;
		_aflags = _flags2;
	
		return true;
	}
	else if (_flags2 & ATM_FLAG_POL_H && _flags1 & ATM_FLAG_HB_ACCEPTOR)
	{
		_hydrogen = _atom2;
		_acceptor = _atom1;
		_hres = _res2;
		_ares = _res1;
		_hflags = _flags2;
		_aflags = _flags1;
	
		return true;
	}

	return false;
}

/// \brief Returns the type of the atom currently being evaluated within the given type system.
///
/// \param typeSystem One of the enumerated type systems (see atom_types, etc.)
inline uint Hydrogen_GetType(AtomType::TypeSystem typeSystem)
{
	return atypes_->GetType(_hres->GetAminoAcid().atomType[_hydrogen->depth], typeSystem); 
}

/// \brief Returns the type of the atom currently being evaluated within the given type system.
///
/// \param typeSystem One of the enumerated type systems (see atom_types, etc.)
inline uint Acceptor_GetType(AtomType::TypeSystem typeSystem)
{
	return atypes_->GetType(_ares->GetAminoAcid().atomType[_acceptor->depth], typeSystem); 
}

inline bool Acceptor_IsBackbone()
{
	return _aflags & ATM_FLAG_BB;
}

inline bool Hydrogen_IsBackbone()
{
	return _hflags & ATM_FLAG_BB;
}

inline bool ResidueIsAlreadyDonor()
{
	return HBChk((int)_hres->GetID(), 0);
}

inline bool ResidueIsAlreadyAcceptor()
{
	return HBChk((int)_ares->GetID(), 1);
}

inline void RecordBackboneHBond()
{
	HBChk((int)_hres->GetID(), 0) = true;
	HBChk((int)_ares->GetID(), 1) = true;
}	

inline int GetResidueSequenceDist()
{
	return std::abs((int)_hres->GetID() - (int)_ares->GetID());
}
		
#endif


