// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_GENERAL
#define IVL_GENERAL

#include <blitz/tinyvec.h>
#include <cmath>

#include <boost/foreach.hpp>
#include <boost/lexical_cast.hpp>
#include <boost/shared_ptr.hpp>

#include <boost/serialization/serialization.hpp>
#include <boost/serialization/nvp.hpp>
#include <boost/serialization/shared_ptr.hpp>

#include <boost/graph/adjacency_list.hpp>

/** \file general.h
 *
 * \brief This file contains general definitions that are used throughout the code.
 *
 * Keep this as small as possible!
 */

// define MSVC DLL export macros
#ifdef _MSC_VER
#define DllExport __declspec(dllexport)
#else
#define DllExport
#endif

#define 	foreach				BOOST_FOREACH
#define		to_str(val)			(boost::lexical_cast<std::string>(val))

// one level of indirection for type definitions, so that the size of our data types
// can be changed later on if necessary
typedef         unsigned char                   BYTE;
typedef         unsigned short			DBYTE;
typedef		float				FLOAT;
typedef 	unsigned int			uint;

typedef		blitz::TinyVector<FLOAT,3>	Position;
typedef		blitz::TinyVector<FLOAT,4>	Vector4;
typedef		blitz::TinyVector<FLOAT,3>	Vector3;
typedef		blitz::TinyVector<FLOAT,2>	Vector2;

typedef 	blitz::TinyVector<int, 4>	VectorI4;

typedef		blitz::TinyVector<Vector3, 3>	Matrix3;

//convenient typedefs used for the atom bonding graph
typedef	boost::adjacency_list<boost::vecS, boost::vecS, boost::undirectedS, boost::property<boost::vertex_name_t, std::string> >	BondGraph;
typedef	boost::graph_traits <BondGraph>::vertex_iterator	VertexIterator;
typedef	boost::graph_traits <BondGraph>::adjacency_iterator	AdjacencyIterator;
typedef	boost::property_map <BondGraph, boost::vertex_index_t >::type	IndexMap;	
typedef	std::pair<VertexIterator,VertexIterator>		VertexRange;	
typedef	std::pair<AdjacencyIterator, AdjacencyIterator>		AdjacencyRange;	

// some global maximums for the input CHOMP will accept
#define         MAX_RESIDUE_ROTAMERS   	 2000	// max. # of rotamers for a single residue (default)
#define         MAX_RESIDUE_ATOMS	 30	// max. # of atoms in a single residue
#define  	MAX_RESIDUE_HEAVY_ATOMS  14	// max. # of non-Hydrogen atoms in a residue
#define		MAX_RESIDUES    	 2000	// max. # of residue positions in a packing problem
#define         MAX_DIHEDRAL_ANGLES      4	// max. # of chi dihedral angles in a residue

#define		SMALL_E 		 1.0E-4

#endif

