// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_OPT_BASE_H
#define IVL_OPT_BASE_H

#include "general.h"

#include <vector>
#include <boost/serialization/vector.hpp>
#include <boost/random.hpp>

#include "discrete_search_space.h"
#include "energy_graph.h"
#include "energy_comb_list.h"
#include "runtime_stats.h"

/** \file opt_base.h
 *  \brief Common traits of generic optimization. Seeds, Stats, Initialization.
 **/

class DllExport OptimizationBase
{
public:
	///////////////////////////////////////////////////////////////////////////
	//////                         CONSTRUCTORS                         ///////
	///////////////////////////////////////////////////////////////////////////

	OptimizationBase();

	///////////////////////////////////////////////////////////////////////////
	//////                      PARAMETER ACCESSORS                      //////
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	//////                        MAIN INTERFACE                        ///////
	///////////////////////////////////////////////////////////////////////////
 
	void Seed(unsigned int seed);
	void SeedWithTime();

	void RecordRuntimeStats(RuntimeStats::Pointer stats);	

protected:
	///////////////////////////////////////////////////////////////////////////
	//////                      PROTECTED MEMBER FNS                    ///////
	///////////////////////////////////////////////////////////////////////////

	void InitState(EnergyGraph& graph, std::vector<int> const& initialState);
	
	///////////////////////////////////////////////////////////////////////////
	//////                         PROTECTED DATA                       ///////
	///////////////////////////////////////////////////////////////////////////

	std::vector<EnergyGraph::vertex_iterator> 	stateVector_;
	std::vector<EnergyGraph::vertex_iterator> 	stateIDStart_;
	std::vector<int>			  	freeDimList_;

	boost::mt19937 				mtrand_;
	DiscreteSearchSpace::ConstPointer 	searchSpace_;
	
	RuntimeStats::Pointer		stats_;

	///////////////////////////////////////////////////////////////////////////
	//////                         SERIALIZATION                        ///////
	///////////////////////////////////////////////////////////////////////////

	friend class boost::serialization::access;
	
	template <class Archive>
	void serialize (Archive& ar, const unsigned int version)
	{
	}
};

#endif


