// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_OPT_BASE_SIMANNEAL_H
#define IVL_OPT_BASE_SIMANNEAL_H

#include "general.h"

#include <vector>
#include <boost/serialization/vector.hpp>
#include <boost/random.hpp>

#include "discrete_search_space.h"
#include "energy_graph.h"
#include "energy_comb_list.h"
#include "runtime_stats.h"

#include "opt_base.h"

/** \file opt_base_simanneal.h
 *  \brief Provides basics needed for Simulated Annealing: Temperature Schedule, KeepChange.
 **/

class DllExport OptimizationBaseSimulatedAnnealing : public OptimizationBase
{
public:
	///////////////////////////////////////////////////////////////////////////
	//////                         CONSTRUCTORS                         ///////
	///////////////////////////////////////////////////////////////////////////

	OptimizationBaseSimulatedAnnealing();

	///////////////////////////////////////////////////////////////////////////
	//////                      PARAMETER ACCESSORS                      //////
	///////////////////////////////////////////////////////////////////////////

	std::vector<FLOAT> GetSchedule() const;
	void SetSchedule(std::vector<FLOAT> const& schedule);

	FLOAT GetJumpCutoff() const;
	void SetJumpCutoff(FLOAT cutoff);

	FLOAT GetIterationMultiplier() const;
	void SetIterationMultiplier(FLOAT multiplier);

protected:
	///////////////////////////////////////////////////////////////////////////
	//////                      PRIVATE MEMBER FNS                      ///////
	///////////////////////////////////////////////////////////////////////////

	FLOAT GetTemperature(bool quench);
	
	std::pair<int, int> PickMove(bool quench);

	bool KeepChange (FLOAT prevEnergy, FLOAT diff, FLOAT temperature, bool quench);

	///////////////////////////////////////////////////////////////////////////
	//////                         PRIVATE DATA                         ///////
	///////////////////////////////////////////////////////////////////////////

	std::vector<FLOAT>		energies_;		
	std::vector<FLOAT>		schedule_;
	FLOAT				lowTemp_;
	FLOAT				highTemp_;
	FLOAT				jumpCutoff_;
	FLOAT				iterMultiplier_;

	int 				scheduleIndex_;

	int				quenchDim_;
	int				quenchID_;

	///////////////////////////////////////////////////////////////////////////
	//////                         SERIALIZATION                        ///////
	///////////////////////////////////////////////////////////////////////////

	friend class boost::serialization::access;
	
	template <class Archive>
	void serialize (Archive& ar, const unsigned int version)
	{
		ar & BOOST_SERIALIZATION_NVP(schedule_);
		ar & BOOST_SERIALIZATION_NVP(lowTemp_);
		ar & BOOST_SERIALIZATION_NVP(highTemp_);
		ar & BOOST_SERIALIZATION_NVP(jumpCutoff_);
		ar & BOOST_SERIALIZATION_NVP(iterMultiplier_);
	}
};

#endif


