// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_RUNTIME_STATS_H
#define IVL_RUNTIME_STATS_H

#include "general.h"

#define N_STAT_TYPES 9

enum RuntimeStatType
{
	RSTATS_ATOM_PAIRWISE,
	RSTATS_ATOM_PAIRWISE_PASSED_CUTOFF,
	RSTATS_ATOM_PAIRWISE_HB,

	RSTATS_SIMANNEAL_ITER,
	RSTATS_SIMANNEAL_DIFFENERGY,
	RSTATS_SIMANNEAL_DIFFENERGY_HG,
	RSTATS_SIMANNEAL_LISTOP,
	
	RSTATS_MONTECARLO_ITER,

	RSTATS_BRUTEFORCE_ITER,

	RSTATS_ALL,
};

/** \file runtime_stats.h
 *  \brief Tools for counting computationally expensive operations
 **/

class DllExport RuntimeStats
{
public:
	typedef boost::shared_ptr<RuntimeStats>		Pointer;
	typedef boost::shared_ptr<const RuntimeStats> 	ConstPointer;

	///////////////////////////////////////////////////////////////////////////
	//////                         CONSTRUCTORS                         ///////
	///////////////////////////////////////////////////////////////////////////
	
	static Pointer Create();

	RuntimeStats();

	///////////////////////////////////////////////////////////////////////////
	//////                       PUBLIC INTERFACE                       ///////
	///////////////////////////////////////////////////////////////////////////

	void ResetCount(RuntimeStatType type = RSTATS_ALL);
	
	inline void Increment(RuntimeStatType type, unsigned long long n = 1)
	{
		data_[type] += n;
	}

	unsigned long long Get(RuntimeStatType type) const;

	double GetTotalWork() const;

private:
	///////////////////////////////////////////////////////////////////////////
	//////                         PRIVATE DATA                         ///////
	///////////////////////////////////////////////////////////////////////////

	unsigned long long data_[N_STAT_TYPES];

	///////////////////////////////////////////////////////////////////////////
	//////                         SERIALIZATION                        ///////
	///////////////////////////////////////////////////////////////////////////

	friend class boost::serialization::access;
	
	template <class Archive>
	void serialize (Archive& ar, const unsigned int version)
	{
		ar & BOOST_SERIALIZATION_NVP(data_);
	}
};

#endif


