// SHARPEN is copyright (c) 2008  Ilya Loksha (primary author), Christopher Snow (project lead).
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files 
// (the "Software") to deal in the Software without restriction, including without limitation the rights to use, copy, modify, 
// merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is 
// furnished to do so, subject to the following conditions: 
// 
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. 
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY 
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH 
// THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
// 

#ifndef IVL_WRAP_HEADERS_H
#define IVL_WRAP_HEADERS_H

#include "chomp_headers.h"
#include "aliases.h"

//////////////////////////////////////////////////////////////////////////
/////                         INSTANTIATIONS                         /////
//////////////////////////////////////////////////////////////////////////

#define SERIALIZE_INST_FN(type, alias)						\
	template <class A>							\
	type Load##alias##FromArchive(A& ar)					\
	{									\
		return LoadBasicFromArchive<type>(ar);				\
	}

SERIALIZE_INST_FN(int, Int)
SERIALIZE_INST_FN(FLOAT, Float)
SERIALIZE_INST_FN(std::vector<int>, IntVector)
SERIALIZE_INST_FN(std::vector<System::Pointer>, SystemVector)

namespace py_details
{

inline void instantiate()
{
	int dummy;

	dummy = sizeof( blitz::TinyVector<FLOAT, 2> );
	//dummy = sizeof( blitz::Array<FLOAT, 2> );
	//dummy = sizeof( blitz::Array<FLOAT, 3> );
	dummy = sizeof( std::vector<int> );
	dummy = sizeof( std::vector<bool> );
	dummy = sizeof( std::vector<std::string> );
	dummy = sizeof( std::vector<System::Pointer> );
	dummy = sizeof( std::vector<System::ConstPointer> );

	dummy = sizeof( std::pair<int, int> );
	dummy = sizeof( std::pair<int, FLOAT> );
	dummy = sizeof( std::vector<std::pair<int, int> > );
	dummy = sizeof( std::vector<std::pair<int, FLOAT> > );
	dummy = sizeof( std::pair<std::vector<int>, FLOAT> );
	dummy = sizeof( std::vector<std::pair<std::vector<int>, FLOAT> > );

	dummy = sizeof( std::map<int, Vector4> );
	dummy = sizeof( std::vector<EnergyGraph::vertex_iterator> );
}

#define SERIALIZE_INST(type, obj) {	LoadFromArchive<type>(xi);		\
					SaveToArchive(xo, "", obj);		\
					LoadFromArchive<type>(bi);		\
					SaveToArchive(bo, "", obj); 		\
					LoadFromArchive<type>(sxi);		\
					SaveToArchive(sxo, "", obj);		\
					LoadFromArchive<type>(sbi);		\
					SaveToArchive(sbo, "", obj); }


#define SERIALIZE_INST_B(type, obj) {	LoadBasicFromArchive<type>(xi);		\
					SaveToArchive(xo, "", obj);		\
					LoadBasicFromArchive<type>(bi);		\
					SaveToArchive(bo, "", obj); 		\
					LoadBasicFromArchive<type>(sxi);	\
					SaveToArchive(sxo, "", obj);		\
					LoadBasicFromArchive<type>(sbi);	\
					SaveToArchive(sbo, "", obj); }


#define SERIALIZE_INST_D(alias, obj) {	Load##alias##FromArchive(xi);		\
					SaveToArchive(xo, "", obj);		\
					Load##alias##FromArchive(bi);		\
					SaveToArchive(bo, "", obj); 		\
					Load##alias##FromArchive(sxi);		\
					SaveToArchive(sxo, "", obj);		\
					Load##alias##FromArchive(sbi);		\
					SaveToArchive(sbo, "", obj); }

inline void serialize_instantiate()
{
	XMLOutputArchive xo("");
	XMLInputArchive xi("");
	BinaryOutputArchive bo("");
	BinaryInputArchive bi("");

	XMLStringOutputArchive sxo;
	XMLStringInputArchive sxi("");
	BinaryStringOutputArchive sbo;
	BinaryStringInputArchive sbi("");

	// basic types
	int myint;
	std::string str;
	FLOAT myfloat;
	std::vector<int> ivec;
	std::vector<System::Pointer> svec;

	SERIALIZE_INST_D(Int, myint);
	SERIALIZE_INST_D(Float, myfloat);
	SERIALIZE_INST_D(String, str);
	SERIALIZE_INST_D(IntVector, ivec);
	SERIALIZE_INST_D(SystemVector, svec);
	
	// CHOMP types
	
	AminoAcid aminoAcid;
	EnergyComb energyComb;
	EnergyGraph energyGraph;
	EnergyHyperGraph energyHyperGraph;
	Residue residue;
	RotamerGenerationParameters rotamerGenerationParameters;
	SimulatedAnnealingPacker simulatedAnnealingPacker;
	SimulatedAnnealingPackerMS simulatedAnnealingPackerMS;
	MonteCarloPacker monteCarloPacker;
	MGMonteCarloPacker mGMonteCarloPacker;
	std::vector<System::Pointer> systemVector;
	AtomTypes::Pointer atomTypes = AtomTypes::Create();
	AminoAcidDirectory::Pointer aminoAcidDirectory = AminoAcidDirectory::Create(atomTypes);
	System::Pointer system = System::Create(atomTypes, aminoAcidDirectory);
	NeighborMap::Pointer neighborMap = NeighborMap::Create(system, blitz::Array<FLOAT, 2>());
	PoseLibrary::Pointer poseLibrary = PoseLibrary::Create(1);
	RotamerLibrary::Pointer rotamerLibrary = RotamerLibrary::Create(system);
	RotamerIterationMap::Pointer rotamerIterationMap = RotamerIterationMap::Create(rotamerLibrary);
	RuntimeStats::Pointer runtimeStats = RuntimeStats::Create();
	
	SERIALIZE_INST(AminoAcid, aminoAcid);
	SERIALIZE_INST_B(AminoAcidDirectory::Pointer, aminoAcidDirectory);
	SERIALIZE_INST_B(AtomTypes::Pointer, atomTypes);
	SERIALIZE_INST(EnergyComb, energyComb);
	SERIALIZE_INST(EnergyGraph, energyGraph);
	SERIALIZE_INST(EnergyHyperGraph, energyHyperGraph);
	SERIALIZE_INST_B(NeighborMap::Pointer, neighborMap);
	SERIALIZE_INST_B(PoseLibrary::Pointer, poseLibrary);
	SERIALIZE_INST(Residue, residue);
	SERIALIZE_INST(RotamerGenerationParameters, rotamerGenerationParameters);
	SERIALIZE_INST_B(RotamerIterationMap::Pointer, rotamerIterationMap);
	SERIALIZE_INST_B(RotamerLibrary::Pointer, rotamerLibrary);
	SERIALIZE_INST_B(RuntimeStats::Pointer, runtimeStats);
	SERIALIZE_INST(SimulatedAnnealingPacker, simulatedAnnealingPacker);
	SERIALIZE_INST(SimulatedAnnealingPackerMS, simulatedAnnealingPackerMS);
	SERIALIZE_INST(MonteCarloPacker, monteCarloPacker);
	SERIALIZE_INST(MGMonteCarloPacker, mGMonteCarloPacker);
	SERIALIZE_INST_B(System::Pointer, system);
	SERIALIZE_INST_B(std::vector<System::Pointer>, systemVector);
}

}

#endif

